c ABFLARE Modules

c Program Versions
      character*12  :: mmodel,mversion,mlevel
c --- Identify version and level of ABFLARE
      common /progver/ mmodel,mversion,mlevel

c      module stddefs
      parameter (mxprop = 31  )
      parameter (mxcomp = 22 ) 
      parameter (mxchem = 10 ) 
      parameter (mxpuff=10000)
      parameter (mxkd=8)
      parameter (mxmet=44000)
      parameter (mxvsrc=32)

      parameter (iolst = 20)	! lst file of inputs
      parameter (ioab  = 21)	! output io for abflare model output
      parameter (iomet = 22)	! met timeseries input file
      parameter (iolog = 23)	! lst/log file for debugging
      parameter (ioin  = 24)	! inp file of inputs

c      real   :: pi      ! uh... pi
c      real   :: ru      ! Universal Gas Constant (Ru),(kPam)/(Kkmol)
c      real   :: rk0     ! Absolute zero, reference C
c      real   :: grav    ! Gravitational Constant (g),m/s2
c      real   :: lapse   ! atmospheric default lapse rate
c      real   :: rk13    ! one third 
c      real   :: rk23    ! two thirds
c	   real   :: rmxfrel ! maximum permitted fraction of volume released
c	   real   :: tch2o   ! h2o critical temp, K
c      real   :: pch2o   ! h2o critical pressure
c      real   :: rhoch2o ! ch2o critical density
      parameter (pi       = 3.14159265358979323846D0 )
      parameter (ru       = 8.3145D0   )
      parameter (rk0      = 273.15D0   )
      parameter (grav     = 9.80665D0  )
      parameter (rk13     = 1.D0/3.D0 )
      parameter (rk23     = 2.D0/3.D0 )
	  parameter (rmxfrel = 0.999D0)	! maximum permitted fraction of volume released
	  parameter (tch2o = 647.096D0) ! critical temperature of water, K
	  parameter (pch2o = 22064.D0)
	  parameter (rhoch2o = 322.D0)


	  real(8)   :: effminERCB ! Minimum Conversion and Combustion Efficiency ERCB
      real(8)   :: effmin  ! Minimum Conversion and Combustion Efficiency
      real(8)   :: radloss ! flame radiation loss, %
      real(8)   :: tgas    ! default gas temperature, K 
      real(8)   :: tamb    ! default ambient temperature, K
      real(8)   :: tradbkg ! Background Thermal Radiation, kW/m2
      real(8)   :: tradtot ! Maximum Recommended Total Thermal Radiation, kW/m2
      real(8)   :: u10min  ! Minimum Wind Speed at 10 m reference height, m/s
      real(8)   :: ustabf  ! Stability Class F Plume Rise Criterion Wind Speed
      real(8)   :: tgrad   ! Stability Class F Vertical Potential Temperature Gradient
      real(8)   :: slopemx ! Maximum Slope Criterion, %
      real(8)   :: uofaa   ! UofA Flare Research Project constant A, (MJ/kg)^3
      real(8)   :: uofab   ! UofA Flare Research Project constant B
      real(8)   :: tref    ! reference Temperature,K
      real(8)   :: pref    ! reference Pressure,kPa
      real(8)   :: cpv     ! Specific Volume at reference conditions,m/kmol
      real(8)   :: rkm     ! 
      real(8)   :: rlapse  ! [K/m] standard lapse rate (troposphere)
      real(8)   :: rk1     ! Heat Release Rate Constant (K1),(caldm)/(s10mMJ)
      real(8)   :: rk2     ! SO2 Emission Rate Constant (K2),(gd)/(s10m%H2S)
      real(8)   :: rk4     ! Heat Release Rate Constant (K4),(Jdm)/(s10mMJ)
      real(8)   :: nhvrec  ! net heating value required, MJ/m (15C,101.325 kPa) 
      real(8)   :: nhvmin  ! net heating value mininum, MJ/m (15C,101.325 kPa)

      common   /constants1/ effminERCB,effmin, radloss, tgas, tamb, 
     &   tradbkg, tradtot, u10min, ustabf, tgrad, slopemx,
     &   uofaa, uofab, tref, pref, cpv, rkm, rlapse,
     &   rk1, rk2, rk4, nhvrec, nhvmin 

      real(8)  :: plocal  ! local pressure, kPa
      common   /local1/ plocal

      integer :: debug ! 0 no debug; 1=debug				
	  common /flags/ debug

c Property definitions      
      parameter (jph2o=1      )
      parameter (jph2=2      )
      parameter (jphe=3         )
      parameter (jpn2=4         )
      parameter (jpco2=5      )
      parameter (jph2s=6      )
      parameter (jpch4=7      )
      parameter (jpc2h6=8      )
      parameter (jpc3h8=9      )
      parameter (jpic4h10=10   )
      parameter (jpnc4h10=11   )
      parameter (jpic5h12=12   )
      parameter (jpnc5h12=13   )
      parameter (jpnc6h14=14   )
      parameter (jpnc7h16x=15  )
      parameter (jpco=16      )
      parameter (jpnh3=17      )
      parameter (jpar=18      )
      parameter (jpo2=19      )
      parameter (jpso2=20      )
      parameter (jps=21         )
      parameter (jpair=22      )

      parameter (ipmmass=1  )
      parameter (ipnhv=2  )
      parameter (ipghv=3  )
      parameter (ipcpmass=4  )
      parameter (ipcpmole=5  )
      parameter (ipstoico2=6  )
      parameter (ipstoicc=7  )
      parameter (ipstoicp=8  )
      parameter (ipstoich2o=9 )
      parameter (iplfl=10  )
      parameter (ipcomb=11  )
      parameter (ipautoig=12 )
      parameter (ipa1=13  )
      parameter (ipa2=14  )
      parameter (ipa3=15  )
      parameter (ipa4=16  )
      parameter (ipa5=17  )
      parameter (ipa6=18  )
      parameter (ipa7=19  )
      parameter (ipb1=20  )
      parameter (ipb2=21  )
      parameter (ipb3=22  )
      parameter (ipb4=23  )
      parameter (ipb5=24  )
      parameter (ipb6=25  )
      parameter (ipb7=26  )
      parameter (ipcalc=27  )
      parameter (iph=28  )
      parameter (ipexh=29  )
      parameter (ipinert=30  )
      parameter (ipair=31  )

      common  /chem1/ prop
      real(8)   :: prop(mxprop,mxcomp)

	  common /units/ MUNITS
	  integer :: MUNITS				! units for coordinates in FLarb file

c      end module ! stddefs


