c ---------------------------------------------------------------------
      subroutine methdr(io,ilog,fmet,ws,wd,tamb,ipg)
c ---------------------------------------------------------------------
c
c --- METSERIES   Version: 1.00       Level: 110828
c --- B.Zelt
c
c --- PURPOSE: Extract header information from TSERIES.DAT
c
c --- UPDATES:
c --- v1.0(110828)
c        - 
c
c ---------------------------------------------------------------------
c --- INPUTS
      integer :: io     ! read io
      integer :: ilog   ! debug and list logging
      character*128 :: fmet ! path for met
      real(8) :: ws
      real(8) :: wd
      real(8) :: tamb
      integer :: ipg
c --- DECLARATIONS
      include 'h_met1.for'
      real(8) :: zone
      character*8   :: pmap8
      character*16  :: tsernam,tserver
      character*64  :: tsermod
      character*320 :: hdr(50)
      character*12  :: avar,aunits
      character*320 :: char320m
      character*8   :: azonemet 

c --- initialize
      indws = -1
      indwd = -1
      indt  = -1
      indpg = -1
      indmo = -1

c --- Check the MET header that it is a TIMESERIES.TSF output file
c --- with correct location and extraction elevation
c --------------------------------------------------
      open(io,file=fmet,status='old',action='read')

c ---  read header information from meteorological data file
c ---  Test first header record to determine dataset version
       read(io,'(2a16,a64)') tsernam,tserver,tsermod
        if(tsernam .EQ.'TIMESERIES.TSF  ') then
c ---       Read comment records
            read(io,*) ncomm
            do n=1,ncomm
               read(io,'(a80)') comment
            enddo
        else
            write(ilog,*)'Invalid timeseries format!'
            write(ilog,*)'Found ',tsernam
            write(ilog,*)'Expected TIMESERIES.TSF'
            stop 'Halted in METHDR --- See log file'
        endif

c ---       Set a default map projection to NONE
            pmap8='NONE    '

c ---       Read header section before data description
            if(tserver.EQ.'1.0             ') then
               read(io,*) nvars
            elseif(tserver.NE.'1.3             ') then
               read(io,*) azonemet
               read(io,*) nvars
            elseif(tserver.EQ.'1.3             ') then
               read(io,*) ntitles
               if ( ntitles .LT. 2 ) then
                  write(ilog,*)' '
                  write(ilog,*)'Wrong number of title lines in file'
                  write(ilog,*)'Found    ',ntitles
                  write(*,*)'Needed 2'
                  write(*,*)'Wrong number of title lines in file'
                  write(*,*)'Found    ',ntitles
                  write(*,*)'Needed 2'
                  stop 'Halted in METHDR --- See log file'
               endif
c
c ---          (CEC - 090411 - Store all lines from the description lines
c               read(io,'(a)')hdr1
c               read(io,'(a)')hdr2
c               do k=3,ntitles
                do k=1,ntitles
c                  read(io,'(a)')hdr3
                   read(io,'(a)')hdr(k)
               enddo
               read(io,*) pmap8
               if(pmap8.EQ.'NONE    ') then
                  nskip=0
               elseif(pmap8.EQ.'LL      ') then
                  nskip=2
               elseif(pmap8.EQ.'UTM     ') then
                  nskip=3
               else
                  nskip=4
               endif
               do n=1,nskip
                  read(io,*)
               enddo
               read(io,*) azonemet
               read(io,*)
c --- Read date range in met file
               read(io,'(a320)') char320m  
c ---    Date-Time - read begining time
         read(char320m(1:4),'(i4)') iyrb
         read(char320m(6:7),'(i2)') imob
         read(char320m(9:10),'(i2)') idayb
         read(char320m(12:13),'(i2)') ihourb
         read(char320m(15:18),'(i4)') isecbm
         read(char320m(20:23),'(i4)') iyre
         read(char320m(25:26),'(i2)') imoe
         read(char320m(28:29),'(i2)') idaye
         read(char320m(31:32),'(i2)') ihoure
         read(char320m(34:37),'(i4)') isecem
         read(char320m(42:45),'(i4)') itstep

      call JULDAY(ilog,iyrb,imob,idayb,jdayb)
      call JULDAY(ilog,iyre,imoe,idaye,jdaye)
      call MKDAT(iyrb, jdayb, ihourb, jdatebm)
      call MKDAT(iyre, jdaye, ihoure, jdateem)
      jdatebmi = jdatebm
      jdateemi = jdateem
      isecbmi = isecbm
      isecemi = isecem

               read(io,*) nvars
            endif

c --- Trap time-zone difference in TIMESERIES.TSF file (MET)
      call UTCBASR(azonemet,zone)
      izonemet=NINT(zone)
c      if(izonemet.NE.izonein) then
c         write(ilog,*)
c         write(ilog,*)' Error processing meteorological TSF file'
c         write(ilog,*)'          Data are in time zone: ',azonemet
c         write(ilog,*)'             Expected time zone: ',azonein
c         write(ilog,*)' Times zones must match'
c         stop 'Halted in methdr --- See log file'
c      endif

c --- Read data description records
      do n=1,nvars
          zwind=0.0
          xloc1=0.0
          yloc1=0.0
          if(tserver.NE.'1.3             ') then
              read(io,'(2a12,2f12.3)') avar,aunits,
     &                                     xmissval,zwind
          else
              read(io,'(2a12,5f12.3)') avar,aunits,xmwt,
     &                                     xmissval,zwind,xloc1,yloc1
          endif

c --- wind direction
          if(avar(1:6).eq.'WDIR  ') then 
            indwd=n
            zwd=zwind
          end if   
c --- wind speed
          if(avar(1:6).eq.'WSPEED') then 
            indws=n
              if(zwd.eq.zwind) then
                  zws=zwind
              else
                  write(ilog,*)'Error- wind speed ht not provided'
                  stop 'Halted in methdr --- See log file'
              endif
            iwsunits = 1
            if(aunits(1:3).eq.'M/S') then 
                iwsunits = 1
            else if(aunits(1:4).eq.'KM/H') then 
                iwsunits = 2
            else if(aunits(1:2).eq.'KN') then 
                iwsunits = 3
            end if 
          end if   
c --- temperature 
          if(avar(1:6).eq.'TEMP  ') then 
            indt=n
            itunits = 1
            if(aunits(1:1).eq.'K') then 
                itunits = 1
            else if(aunits(1:1).eq.'C') then 
                itunits = 2
            else if(aunits(1:1).eq.'R') then 
                itunits = 3
            else if(aunits(1:1).eq.'F') then 
                itunits = 4
            end if 
          end if 
c --- stability Class 
          if(avar(1:10).eq.'STAB_CLASS') then 
            indpg=n
          end if 
c --- wind speed
          if(avar(1:8).eq.'MONIN_OB') then 
            indmo=n
          end if 
      enddo

      write(ilog,*) 'METSERIES anemometer height set to(m):',zws      

c ---  Final section of datasets before v1.3
c ---  Expect 2 fixed header records for title information
       if(tserver.NE.'1.3             ') then
           read(io,*) ntitles
           if(ntitles.LT.2) then
               write(ilog,*)'Wrong number of title lines in file'
               write(ilog,*)'Found    ',ntitles
               write(ilog,*)'Needed 2'
               write(*,*)'Wrong number of title lines in file'
               write(*,*)'Found    ',ntitles
               write(*,*)'Needed 2'
               stop 'Halted in methdr --- See log file'
            endif
c               read(io,'(a)')hdr1
c               read(io,'(a)')hdr2
c               do k=3,ntitles
                do k=1,ntitles
c                  read(io,'(a)')hdr3
                   read(io,'(a)')hdr(k)
               enddo
       endif

c read first line to initialize 
      call rdmet(io,ilog,ws,wd,tamb,ipg,iend)

      return
      end

c ---------------------------------------------------------------------
      subroutine metget(io,ilog,jdatex,isecx,ws,wd,tamb,ipg)
c ---------------------------------------------------------------------
c
c --- METSERIES   Version: 1.00       Level: 110828
c --- B.Zelt 
c
c --- PURPOSE: Extract met information from TSERIES.DAT 
c
c --- UPDATES:
c --- v1.0(110828)
c        - 
c
c ---------------------------------------------------------------------
c --- INPUTS
      integer :: io
      integer :: ilog
      integer :: jdatex
      real(8) :: ws        ! wind speed 
      real(8) :: wd        ! wind direction
      real(8) :: tamb      ! ammbient temperature
      integer :: ipg    ! PG stability class

c --- DECLARATIONS
      include 'h_met1.for'

c ---    Compute difference in seconds between two dates
         call DELTSEC(jdatebm,isecbm,jdatex,isecx,ndelsec1)
         call DELTSEC(jdatex,isecx,jdateem,isecem,ndelsec2)
 10      call DELTSEC(jdatebmi,isecbmi,jdatex,isecx,ndelsec3)
         call DELTSEC(jdatex,isecx,jdateemi,isecemi,ndelsec4)

         if(ndelsec1.lt.0) then
c --- stop if beginning date requested by user is earlier
c                     than data availability
         write( ilog,*)'ERROR- beginning date/time requested is earlier'
            write( ilog,*)'than data in CALPOST TIME SERIES file'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'beginning date in file = ',jdatebm,isecbm
            write(*,*)'ERROR- beginning date/time requested is earlier'
            write(*,*)'than data in CALPOST TIME SERIES file'
            write(*,*)'date requested = ',jdatex,isecx
            write(*,*)'beginning date in file = ',jdatebm,isecbm
          stop
         elseif(ndelsec2.lt.0) then
c --- stop if end date requested by user is earlier
c                     than data availability
            write( ilog,*)'ERROR- end date/time requested is later'
            write( ilog,*)'than data in CALPOST TIME SERIES file'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'end date in file = ',jdateem,isecem
            write(*,*)'ERROR- end date/time requested is later'
            write(*,*)'than data in CALPOST TIME SERIES file'
            write(*,*)'date requested = ',jdatex,isecx
            write(*,*)'end date in file = ',jdateem,isecem
          stop
         elseif(ndelsec3.lt.0) then
            write( ilog,*)'***ERROR***- bad read sequence'
            write( ilog,*)'date requested = ',jdatex,isecx
            write( ilog,*)'beginning date in file = ',jdatebm,isecbm
            write( ilog,*)'end date in file = ',jdateem,isecem
            write( ilog,*)'beginning date read = ',jdatebmi,isecbmi
            write( ilog,*)'end date read = ',jdateemi,isecemi
            write( *,*)'***ERROR***- bad read sequence'
            write( *,*)'date requested = ',jdatex,isecx
            write( *,*)'beginning date in file = ',jdatebm,isecbm
            write( *,*)'end date in file = ',jdateem,isecem
            write( *,*)'beginning date read = ',jdatebmi,isecbmi
            write( *,*)'end date read = ',jdateemi,isecemi
          stop
         elseif((ndelsec1.ge.0) .and. (ndelsec2.ge.0)
     &         .and. (ndelsec4 .gt. 0)) then
c --- current met period is still valid
c     just return input values
c           ws = still good
c           wd = still good
            return
         endif

c         elseif(ndelsec4.lt.0) then
c --- read next time step
        call rdmet(io,ilog,ws,wd,tamb,ipg,iend)

        if(iend .gt. 0) goto 3001
c --- recursive call
c     call metget(io,ilog,jdatex,isecx,ws,wd)            
c        or
c     start at top again and see if date is valid
         goto 10

c --- end of file
c     set dates to end of file dates
 3001 continue
      jdatebmi = jdatebm
      isecemi = isecem
      jdateemi = jdatebm
      isecemi = isecem

      return
      end


      subroutine rdmet(io,ilog,ws,wd,tamb,ipg,iend)
c --- INPUTS
      integer :: io     ! read io
      integer :: ilog   ! debug and list logging
      real(8) :: ws
      real(8) :: wd
      real(8) :: tamb
      integer :: ipg
c --- DECLARATIONS
      include 'h_met1.for'
      character*320 char320m,blnk320

      do i=1,320
         blnk320(i:i)=' '
      enddo

c
c ---    Step through meteorological fields
c -----------------------------------------
c ---    Read line
20       char320m=blnk320
         wd=9999.
         ws=9999.
         read(io,'(a320)',end=3001) char320m
c
         isecbmi=0
         isecemi=0
c ---    Date-Time - read begining time
         read(char320m(2:5),'(i4)') iyrb
         read(char320m(7:8),'(i2)') imob
         read(char320m(10:11),'(i2)') idayb
         read(char320m(13:14),'(i2)') ihourb
         read(char320m(16:19),'(i4)') isecbmi
         read(char320m(21:24),'(i4)') iyr
         read(char320m(26:27),'(i2)') imo
         read(char320m(29:30),'(i2)') iday
         read(char320m(32:33),'(i2)') ihour
         read(char320m(35:38),'(i4)') isecemi
c
c ---    Skip blank record (based on date)
         if(iyr.EQ.0 .OR. imo.EQ.0 .OR. iday.EQ.0) goto 20
         call JULDAY(ilog,iyr,imo,iday,jday)
         call JULDAY(ilog,iyrb,imob,idayb,jdayb)

c ---    Get month/day
c         call GRDAY(ilog,iyr,jday,imon,iday)
c         call GRDAY(ilog,iyrb,jdayb,imonb,idayb)

c ---    Create timestamp with time at beginning
         call TIMESTAMP(iyr,jday,ihour,jdateemi)
         call TIMESTAMP(iyrb,jdayb,ihourb,jdatebmi)

c ----------------------
c --- Wind Speed
        i1= 39+(indws-1)*10
        if(indws .gt. indmo) i1 = i1+ 3
        i2= i1+ 9
        read( char320m(i1:i2),'(f10.3)')ws
        if( iwsunits.eq.1 ) then
c --- convert M/S to M/S
        else if( iwsunits.eq.2 ) then
c --- convert KM/H to M/S
            ws = ws /3.6
        else if( iwsunits.eq.3 ) then
c --- convert KNOTTS to M/S
            ws = ws * 0.51444
        end if 

c --- Wind Direction
        i1= 39+(indwd-1)*10
        if(indwd .gt. indmo) i1 = i1+ 3
        i2= i1+ 9
        read( char320m(i1:i2),'(f10.3)')wd

c --- Temperature
        i1= 39+(indt-1)*10
        if(indt .gt. indmo) i1 = i1 + 3
        i2= i1+ 9
        read( char320m(i1:i2),'(f10.3)')tamb
        if( itunits.eq.1 ) then
c convert K to K
            tamb = tamb
        else if( itunits.eq.2 ) then
c convert C to K
            tamb = tamb + 273.15
        else if( itunits.eq.3 ) then
c convert R to K
            tamb = tamb*5./9. 
        else if( itunits.eq.4 ) then
c convert F to K
            tamb = (tamb-32.)* 5./9. + 273.15
        end if 

c --- PG stability 
        if(indpg .gt. 0) then 
          i1= 39+(indpg-1)*10
          if(indpg .gt. indmo) i1  = i1 + 3
          i2= i1+ 9
          read( char320m(i1:i2),'(i10)') ipg
        else
          ipg = -1
        end if 

      iend = 0
      return

 3001 iend =1
      return
      end
