c-----------------------------------------------------------------------
      module header
c-----------------------------------------------------------------------
c --- CALMAX    Version: 1.1            Level: 120715          /HEADER/
c               D. Strimaitis, Exponent Inc.
c
c --- Header Data from input CALPUFF CONC/WET/DRY files
c --- Use variable names as listed in the CALPUFF Users Guide
c
c --- Use arrays for ALL variables so that information from multiple
c --- data files can be stored (allocate)
c-----------------------------------------------------------------------

      logical, allocatable :: lcompr(:),lsamp(:),ldisc(:),lctsg(:),
     &                        lspunit(:)

      character*12,  allocatable :: cmodel(:),ver(:),level(:)
      character*4,   allocatable :: utmhem(:)
      character*8,   allocatable :: pmap(:),datum(:),axbtz(:)
      character*12,  allocatable :: daten(:)
      character*15,  allocatable :: csout(:,:)
      character*16,  allocatable :: clat0(:),clon0(:),clat1(:),clat2(:)
      character*16,  allocatable :: dataset(:),dataver(:)
      character*16,  allocatable :: csunit(:,:)
      character*16,  allocatable :: cnampt1(:,:),cnampt2(:,:)
      character*16,  allocatable :: cnamar1(:,:),cnamar2(:,:)
      character*16,  allocatable :: cnamln1(:,:),cnamln2(:,:)
      character*16,  allocatable :: cnamvl1(:,:),cnamvl2(:,:)
      character*64,  allocatable :: datamod(:)
      character*80,  allocatable :: title(:,:)

      integer, allocatable :: ncom(:),ibyr(:),ibjul(:),ibhr(:),ibsec(:),
     &                        irlg(:),iavg(:),nsecdt(:),
     &                        nxm(:),nym(:),nz(:),nssta(:),
     &                        ibcomp(:),iecomp(:),jbcomp(:),jecomp(:),
     &                        ibsamp(:),jbsamp(:),iesamp(:),jesamp(:),
     &                        meshdn(:),npt1(:),npt2(:),nar1(:),nar2(:),
     &                        nln1(:),nln2(:),nvl1(:),nvl2(:),
     &                        msource(:),ndrec(:),nctrec(:),nspout(:),
     &                        i2dmet(:),iutmzn(:),ihill(:,:),
     &                        nsrc(:),ngx(:),ngy(:)
      
      real, allocatable :: xbtz(:),dxkm(:),dykm(:),xorigkm(:),
     &                     yorigkm(:),feast(:),fnorth(:),rnlat0(:),
     &                     relon0(:),xlat1(:),xlat2(:),
     &                     xrec(:,:),yrec(:,:),zrec(:,:),
     &                     xctr(:,:),yctr(:,:),zctr(:,:)

      end module header


c-----------------------------------------------------------------------
      module userinf
c-----------------------------------------------------------------------
c --- CALMAX    Version: 1.1            Level: 120915         /USERINF/
c               D. Strimaitis, Exponent Inc.
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change pathname variables from C*132 to C*256
c        2. Add switch for processing individual sources in file or
c           just the total
c        3. Change the calendar-day output file to a time-block period
c           file
c
c --- Control file information
c-----------------------------------------------------------------------

      logical :: lbinout,lcfiles,ltotal

      character(len=256), allocatable :: inpfile(:)
      character(len=256)              :: lstfile, perfile, binfile

      integer, allocatable :: ioinparray(:)
      integer :: nfiles
      integer :: start_year,start_juld,start_hour,start_second
      integer :: end_year,end_juld,end_hour,end_second

c --- Calculated
      integer (kind=8) :: start_datetime,end_datetime
      integer :: nperiods

c --- Establish initial (default) values
c --- Always make BIN file
c      data lbinout/.FALSE./
      data lbinout/.TRUE./
      data lcfiles/.FALSE./
      data ltotal/.TRUE./

      end module userinf


c-----------------------------------------------------------------------
      module inpdata
c-----------------------------------------------------------------------
c --- CALMAX    Version: 1.1            Level: 120915            /DATA/
c               D. Strimaitis, Exponent Inc.
c
c --- Data from input CALPUFF CONC/WET/DRY files
c --- Use variable names as listed in the CALPUFF Users Guide
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Add flag array for time-period status
c            istatus=-1   start-time of data younger than current period
c            istatus= 0   start-time of data within current period
c            istatus= 1   start-time of data older than current period
c            istatus= 9   no data reached end of file
c-----------------------------------------------------------------------

      character(len=16), allocatable :: sname(:,:)                      ! (sources,files)
      character(len=15), allocatable :: cspecg(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecd(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecct(:,:)                    ! (spec-lev,files)

      integer, allocatable :: istype(:,:),isnum(:,:)                    ! (sources,files)
      integer, allocatable :: nyrb(:),njulb(:),nhrb(:),nsecb(:)         ! (files)
      integer, allocatable :: nyre(:),njule(:),nhre(:),nsece(:)         ! (files)
      integer(kind=8), allocatable  :: datetime_b(:),datetime_e(:)      ! (files)
      integer, allocatable :: istatus(:)                                ! (files)

      real, allocatable :: sxkm(:,:),sykm(:,:)                          ! (sources,files)
      real, allocatable :: datag(:,:,:,:,:)                             ! (ngx,ngy,spec-lev,sources,files)
      real, allocatable :: datad(:,:,:,:)                               ! (ndrec,spec-lev,sources,files)
      real, allocatable :: datact(:,:,:,:)                              ! (nctrec,spec-lev,sources,files)

      real, allocatable :: workg(:,:)                                   ! (ngx,ngy)
      real, allocatable :: workd(:)                                     ! (ndrec)
      real, allocatable :: workct(:)                                    ! (nctrec)

      end module inpdata

c-----------------------------------------------------------------------
      module maxdata
c-----------------------------------------------------------------------
c --- CALMAX    Version: 1.1            Level: 120915         /MAXDATA/
c               D. Strimaitis, Exponent Inc.
c
c --- Maximums for output CALPUFF CONC/WET/DRY files
c
c --- UPDATES:
c --- v1.0 (120715) to v1.1 (120915)
c        1. Change the calendar-day output to a time-block period
c-----------------------------------------------------------------------

      real, allocatable :: maxdatag(:,:,:)                              ! (nxg,nyg,spec-lev)
      real, allocatable :: maxdatad(:,:)                                ! (ndrec,spec-lev)
      real, allocatable :: maxdatact(:,:)                               ! (nctrec,spec-lev)

      real, allocatable :: maxvals(:,:)                                 ! (spec-lev,period)
      integer (kind=8), allocatable :: maxdatetime(:,:)                 ! (2,period)
      integer           :: current_per                                  ! current period index
      integer           :: istepsec                                     ! length of output time-block

c --- Set initial period index
      data current_per/0/
c --- Set time-block negative
      data istepsec/-1/

      end module maxdata

