c-----------------------------------------------------------------------
      module userinf
c-----------------------------------------------------------------------
c --- CALRANK   Version: 1.1            Level: 120915         /USERINF/
c               D. Strimaitis, Exponent Inc.
c
c --- Control file information
c
c --- UPDATES:
c --- v1.0 (120630) to v1.1 (120915)
c        1. Change pathname variables from C*132 to C*256
c        2. Add output mass units selector
c-----------------------------------------------------------------------

      logical :: lcfiles

      character(len=256)   :: lstfile, datfile

      real,    allocatable :: pct(:),pct_from_nth(:)                    ! (ranks)
      integer, allocatable :: nth(:),nth_from_pct(:)                    ! (ranks)
      integer              :: n_nth, n_pct, icday, n_ranks, mass_u

c --- Establish initial (default) values
      data lcfiles/.FALSE./
      data icday/0/
      data mass_u/1/

      end module userinf

c-----------------------------------------------------------------------
      module header
c-----------------------------------------------------------------------
c --- CALRANK   Version: 1.1            Level: 120630          /HEADER/
c               D. Strimaitis, Exponent Inc.
c
c --- Header Data from input CALPUFF CONC/WET/DRY files
c --- Use variable names as listed in the CALPUFF Users Guide
c
c --- Use arrays for ALL variables so that information from multiple
c --- data files can be stored (allocate)
c
c --- UPDATES:
c --- v1.0 (120630) to v1.1 (120915)
c        1. Add units array for storing output units requested
c-----------------------------------------------------------------------

      logical, allocatable :: lcompr(:),lsamp(:),ldisc(:),lctsg(:),
     &                        lspunit(:),lrunning(:)

      character*12,  allocatable :: cmodel(:),ver(:),level(:)
      character*4,   allocatable :: utmhem(:)
      character*8,   allocatable :: pmap(:),datum(:),axbtz(:)
      character*12,  allocatable :: daten(:)
      character*15,  allocatable :: csout(:,:)
      character*16,  allocatable :: clat0(:),clon0(:),clat1(:),clat2(:)
      character*16,  allocatable :: dataset(:),dataver(:)
      character*16,  allocatable :: csunit(:,:)
      character*16,  allocatable :: cnampt1(:,:),cnampt2(:,:)
      character*16,  allocatable :: cnamar1(:,:),cnamar2(:,:)
      character*16,  allocatable :: cnamln1(:,:),cnamln2(:,:)
      character*16,  allocatable :: cnamvl1(:,:),cnamvl2(:,:)
      character*64,  allocatable :: datamod(:)
      character*80,  allocatable :: title(:,:)

      integer, allocatable :: ncom(:),ibyr(:),ibjul(:),ibhr(:),ibsec(:),
     &                        irlg(:),iavg(:),nsecdt(:),
     &                        nxm(:),nym(:),nz(:),nssta(:),
     &                        ibcomp(:),iecomp(:),jbcomp(:),jecomp(:),
     &                        ibsamp(:),jbsamp(:),iesamp(:),jesamp(:),
     &                        meshdn(:),npt1(:),npt2(:),nar1(:),nar2(:),
     &                        nln1(:),nln2(:),nvl1(:),nvl2(:),
     &                        msource(:),ndrec(:),nctrec(:),nspout(:),
     &                        i2dmet(:),iutmzn(:),ihill(:,:),
     &                        nsrc(:),ngx(:),ngy(:)
      
      real, allocatable :: xbtz(:),dxkm(:),dykm(:),xorigkm(:),
     &                     yorigkm(:),feast(:),fnorth(:),rnlat0(:),
     &                     relon0(:),xlat1(:),xlat2(:),
     &                     xrec(:,:),yrec(:,:),zrec(:,:),
     &                     xctr(:,:),yctr(:,:),zctr(:,:)

c --- Derived data
c --- All receptor locations (ntotrec,file)
      real, allocatable :: xreckm(:,:),yreckm(:,:)
c --- Total receptors
      integer, allocatable :: ngrec(:),ntotrec(:)
c --- Output units (user)
      character*16,  allocatable :: counit(:,:)
      real,          allocatable :: xounit(:,:)


      end module header



c-----------------------------------------------------------------------
      module inpdata
c-----------------------------------------------------------------------
c --- CALRANK   Version: 1.1            Level: 120630            /DATA/
c               D. Strimaitis, Exponent Inc.
c
c --- Data from input CALPUFF CONC/WET/DRY files for 1 time
c --- Use variable names as listed in the CALPUFF Users Guide
c-----------------------------------------------------------------------

      character(len=16), allocatable :: sname(:,:)                      ! (sources,files)
      character(len=15), allocatable :: cspecg(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecd(:,:)                     ! (spec-lev,files)
      character(len=15), allocatable :: cspecct(:,:)                    ! (spec-lev,files)

      integer, allocatable :: istype(:,:),isnum(:,:)                    ! (sources,files)
      integer, allocatable :: nyrb(:),njulb(:),nhrb(:),nsecb(:)         ! (files)
      integer, allocatable :: nyre(:),njule(:),nhre(:),nsece(:)         ! (files)
      integer(kind=8), allocatable  :: datetime_b(:),datetime_e(:)      ! (files)

      real, allocatable :: sxkm(:,:),sykm(:,:)                          ! (sources,files)
      real, allocatable :: datag(:,:,:,:,:)                             ! (ngx,ngy,spec-lev,sources,files)
      real, allocatable :: datad(:,:,:,:)                               ! (ndrec,spec-lev,sources,files)
      real, allocatable :: datact(:,:,:,:)                              ! (nctrec,spec-lev,sources,files)

      real, allocatable :: workg(:,:)                                   ! (ngx,ngy)
      real, allocatable :: workd(:)                                     ! (ndrec)
      real, allocatable :: workct(:)                                    ! (nctrec)

      end module inpdata


c-----------------------------------------------------------------------
      module rankdata
c-----------------------------------------------------------------------
c --- CALRANK   Version: 1.1            Level: 120630        /RANKDATA/
c               D. Strimaitis, Exponent Inc.
c
c --- Ranked Data selected from input CALPUFF file
c-----------------------------------------------------------------------

      integer              :: ntimesda                                  ! Number of times in DA file
      integer              :: ntimes                                    ! Number of times used

c --- Working vectors for 1 species, 1 receptor
      integer, allocatable :: irank(:)                                  ! (times) Pointer to time for rank
      real, allocatable    :: tservalue(:)                              ! (times) Original timeseries of values
      character(len=8), allocatable :: atserdate(:)                     ! (times) Original timeseries of dates YYYY_JJJ
      character(len=8), allocatable :: atsertime(:)                     ! (times) Original timeseries of times HH:MM:SS

c --- Array of values (all species, receptors) for requested ranks
      real, allocatable    :: rankvalue(:,:,:)                          ! (ntotrec,nspout,n_ranks)
      character(len=8), allocatable :: arankdate(:,:,:)                 ! (ntotrec,nspout,n_ranks) Day YYYY_JJJ
      character(len=8), allocatable :: aranktime(:,:,:)                 ! (ntotrec,nspout,n_ranks) Start-time HH:MM:SS

c --- Array of MAX over receptors (all species) for requested ranks
      real, allocatable    :: rankvaluemax(:,:)                         ! (nspout,n_ranks)
      real, allocatable    :: rankxmax(:,:)                             ! (nspout,n_ranks)
      real, allocatable    :: rankymax(:,:)                             ! (nspout,n_ranks)
      character(len=8), allocatable :: arankdatemax(:,:)                ! (nspout,n_ranks) Day YYYY_JJJ
      character(len=8), allocatable :: aranktimemax(:,:)                ! (nspout,n_ranks) Start-time HH:MM:SS

      end module rankdata


c-----------------------------------------------------------------------
      module daccess
c-----------------------------------------------------------------------
c --- CALRANK   Version: 1.1            Level: 120630         /DACCESS/
c               D. Strimaitis, Exponent Inc.
c
c --- Configuration & storage associated with Direct Access scratch file
c       Each record contains N times per day + 1 (for daily max)
c       Block of records written daily for all species & receptors:
c         1   HHSSSS(1)-HHSSSS(N), YYYYJJJ  (start time & date for each)
c        ---  IS Loop on NSPOUT species
c        ---  IR Loop on gridded + discrete receptors
c             VAL(1,IR,IS)-VAL(N,IR,IS), VALMAX(IR,IS)
c        There are NSPOUT*(NGX*NGY+NDREC+NCTREC)+1 records in day-block
c-----------------------------------------------------------------------

      integer              :: nvarrcrd                                  ! variables per record (real*4)
      integer              :: nrcrdday                                  ! records for 1 day
      integer              :: numdays                                   ! days in file
      integer              :: nrcrdfile                                 ! records in file

      integer, allocatable :: idatim(:)                                 ! (nvarrcrd) date-time record
      real, allocatable    :: rdatim(:)                                 ! (nvarrcrd) values on 1 record
      real, allocatable    :: val(:,:,:)                                ! (nvarrcrd,ntotrec,nspout)

      end module daccess
