c----------------------------------------------------------------------
c     ABflare
c----------------------------------------------------------------------
c     
c --- Version: 1.001    Level: 110706
c
c     Copyright (c) 2012 by Zelt Professional Services Inc.         
c
c Developed by 
c     B.Zelt, Zelt PSI
c     brian@zeltpsi.com
c     www.zeltpsi.com
c     Okotoks, Alberta, CANADA
c
c UPDATES
c----------------------------------------------------------------------
c
c ---   Version: 1.001    Level: 110706
c       (1) Initial Development
c
c
c----------------------------------------------------------------------     
c     
      program ABflare          
      
      include 'h_stddefs.for'
      include 'h_read1.for'
      include 'h_combustion.for'
      include 'h_met1.for'
      include 'h_file1.for'
c----------------------------------------------------------------------
c DECLARATIONS
      real(8) :: time_begin, time_end

      character*255 :: FLOOP
      character*255 :: FIN
      character*255 :: FROOT
c      character*128 :: FFOLDER
c      character*255 :: FOUT1
      character*255 :: inpfil
      character*255 :: cl_arg1
      character*8   :: timestr
      real(8) :: tempinitx

            
      character(len=5)    ::  ulcase(2)
      data ulcase/'UPPER','LOWER'/
      data mversion/'1.0 '/,mlevel/'140415'/
      data mmodel/'ABFLARE'/
      
      call cpu_time(time_begin)

c -- reset standard output 
      open(6,CARRIAGECONTROL='FORTRAN') 

c --- get commandline input file name      
      inpfil = 'abflare.inp'
      call getcom(cl_arg1)
      if(len_trim(cl_arg1)>0) then
          call getcom(inpfil)
          write(6,*) 'Control file:[', trim(inpfil),']'
          write(6,*)
      endif         
      FIN = inpfil

c set gas compositions to zero      
      ier = initialize(gcraw,0.0d0)
      ier = initialize(gcraw_x,0.0d0)
      ier = initialize(gcfuel,0.0d0)
      ier = initialize(gcmix,0.0d0)
      ier = initialize(gclift,0.0d0)

c read inputs       
      ier = irdcontrol(FIN )

c
      icase = 1    
      if( lcfiles ) then
          icase = 2
      end if 
      
c --- open log file 'abflare.log'
c     to record overall status and messages
      call mk_case(-1, ulcase(icase), LSTDAT, ierr)
      open(iolst,file=LSTDAT,iostat=ierr,status='unknown')
      if(ierr .ne. 0) then
          write(6,*) 'Error opening LSTDAT:', LSTDAT    
          goto 9999
      end if 

      call mk_case(iolst, ulcase(icase), LOGDAT, ierr)
      call mk_case(iolst, ulcase(icase), METDAT, ierr)
      call mk_case(iolst, ulcase(icase), METDAT2, ierr)
      call mk_case(iolst, ulcase(icase), OUTNAME, ierr)
      call mk_case(iolst, ulcase(icase), OUTPATH, ierr)

c open log file for recording debug information      
      DEBUG = 0
      if(MDEBUG .eq. 1) then
      if( len_real(LOGDAT) .gt. 0) then
          open(iolog,file=LOGDAT,iostat=ierr,status='unknown')
      if(ierr .ne. 0) then
          write(6,*) 'Error opening LOGDAT:', LOGDAT
          goto 9999
      end if 
          DEBUG = 1
      end if           
      end if 
      
c --- brand lst file
      write(iolst,*)
     & 'Model:',mmodel,' Version:',mversion,' Level:',mlevel
      
c --- brand log file
      if(debug .gt. 0 ) then
      write(iolog,*)
     & 'Model:',mmodel,' Version:',mversion,' Level:',mlevel
      end if 
      
      FIN = LSTDAT  

c write out the read inputs       
      ier = iwrcontrol(iolst )

c check inputs
      ier = ichkinput(iolst)
      if( ier .gt.0 ) then
          write(iolst,*) 'Number of errors in input file:', ier
          call time(timestr)
          write(iolst,*) timestr
          goto 9999
      end if 

c determine the units
      MUNITS = 2 ! assume km
      if( leq(units,'m').eq.1 .or. leq(units,'M').eq.1 ) then
          MUNITS = 1  ! set m
      end if 
      
c load default variables
      call setup1()
          
c calculate site pressure based upon the elevation
      call InitLocal(zcoord)

c adjust steam assist from mass input (kg/s) to volume input (e3m3/d)
      if( MFASSIST.eq.1 .or. MFASSIST.eq.3) then
      if( stm_imassvol .eq. 1) then
          stm_flow = stm_flow/prop(ipmmass,jph2o)*cpv*0.001*3600.*24.
      end if 
      if( stm_press .le. 0. .and. stm_temp .le. 0.) then
          write(6,*) 
     &         'FATAL ERROR: steam temp and steam pressure both missing'
          write(iolst,*) 
     &         'FATAL ERROR: steam temp and steam pressure both missing'
          goto 9999
      end if
      if( stm_press .le. 0. ) then
           stm_press = p_sat(stm_temp)   
      end if                      
      if( stm_temp .le. 0. ) then
           stm_temp = t_sat(stm_press)   
      end if                      
      end if
      
c calculate start date stamp    
      call julday(iolst,iyys,imms,idds,ijjs)  
      call mkdat(iyys,ijjs,ihhs,idatestart)  
      call julday(iolst,iyye,imme,idde,ijje)
      call mkdat(iyye,ijje,ihhe,idateend)  

c calculated mixed gas
      if(rh2s .gt. 0.) then
          ierr = inormalizeh2s(gcraw, rh2s, gcraw_x)
      else
c basically, make a copy into gcraw_x
          ierr = inormalizeh2s(gcraw, gcraw(jph2s), gcraw_x) 
      end if 

c determine if this is a split puff file or sequential
c      isplit= 0: file is sequential, puffs must be less than 1h duration,
c               each puff-hour follows the next. Simulates continuous.
c      isplit = 1: file is not sequential.  each puff begins on a new hour.

      if(MINPUT .eq. 1 ) then
c --- Calculate blowdown curve
c
c  mblowdown = 1  : no blowdown calculations
c            = 2  : Exponential blowdown
c
c     This duplicates the results with 'blowdown model with equal puffs.rev1.xls'
          if( mblowdown .eq. 1) then
              tempinitx = tempinit 
              if( tempinitx .lt. 0. ) then
                 tempinitx = tamb                  
              end if 
              if(npuffs .le. 1) then    
c sequential puffs must be maximum of hour long
                  ipuffdur = max(itiminc,ipuffdur)
                  qtotal   = qmax*ipuffdur/24./3600.
                  pseq(1,1) = qmax
                  pseq(2,1) = ipuffdur 
                  pseq(3,1) = -1. ! use default
                  pseq(4,1) = -1. ! use default
              else    
          ierr = iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
              end if 
          else if( mblowdown .eq. 2) then
c --- vessel exponential blowdown 
          tempinitx = tempinit 
          if( tempinitx .lt. 0. ) then
              tempinitx = tamb                  
          end if 
          ierr= ivessel(gcraw_x,pressinit,tempinitx,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
          ierr = iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &                mblowdown,mdist)
      if(debug .gt.0 )then
         write(iolog,*) 'npuffs=',npuffs
         do i=1,npuffs
            write(iolog,*) pseq(1,i), pseq(2,i)
         end do            
      end if 

          else 
              write(6,*) 'FATAL ERROR:  Unknown source type'
              write(6,*) 'MBLOWDOWN must be 1 or 2 '
              write(iolst,*) 'FATAL ERROR: Unknown source type'
              write(iolst,*) 'MBLOWDOWN must be 1 or 2'
              goto 9999
          end if     
          qmax0 = qmax
      else if (MINPUT .eq. 2) then
c --- User input blowdown curve
          npuffs = nseq  
          qmax0 = pseq(1,1)
          if( qmax0 .le. 0. .and. MFGR .eq. 2) then
              qmax0 = qmax
              write(6,*) '***WARNING*** setting QMAX0=QMAX:',QMAX
              write(iolst,*) '***WARNING*** setting QMAX0=QMAX:',QMAX
          end if 
      else if (MINPUT .eq. 3) then
c --- Static user input source 
c sequential puffs must be maximum of hour long
          ipuffdur = max(itiminc,ipuffdur)
          npuffs = 1  
          pseq(1,1) = rqemit ! note this is kg/s
          pseq(2,1) = ipuffdur 
          pseq(3,1) = -1. ! use default
          pseq(4,1) = -1. ! use default
      else 
          write(6,*) 'FATAL ERROR:  Unknown source type'
          write(6,*) 'MINPUT must be 1 or 2 or 3'
          write(iolst,*) 'FATAL ERROR: Unknown source type'
          write(iolst,*) 'MINPUT must be 1 or 2 or 3'
          goto 9999
      end if 

      if( debug ) then
              write(iolog,*) 'pseq qmax,duration(s),h2s,fgr   ip=start'
              write(iolog,"(4(G11.4,1x))")
     &              ((pseq(1,ip),pseq(2,ip),pseq(3,ip),
     &              pseq(4,ip)),ip=1,npuffs)
      end if 

      if(npuffs.gt.1 .or. npuffs.eq.0) then
          isplit = 1
      else  
          isplit = 0
          nsep   = 1
      end if 

      nloops = 1
      if(isplit .eq. 1) then
          nloops = nsep
      end if 

c ======================================================================
c The CALPUFF output format is selected
c
c
c
      if( MDSPMOD.eq.1) then
         call mkfilcp()
         
c ======================================================================
c The AERMOD output format is selected.
c Since AERMOD is a plume model, looping and NSEP are not required
c for the source model, but only separate runs for each stage of an
c NSEQ blowdown.  Post processing can then determine appropriate scaled
c hourly or daily averages by combining the results.
      else if( MDSPMOD.eq.2) then
         call mkfilam()
      end if    
c ======================================================================


c Normal Termination
      call cpu_time(time_end)
      write(iolst,*) 'CPU Time ', time_end-time_begin, ' seconds'
      write(6,*) 'Program Terminated'
      write(iolst,*) 'Program Terminated'
      if(debug .gt. 0) then
          write(iolog,*) 'Program Terminated'
      end if           
      close(iolog)
      close(iolst)


      stop

c warnings/errors Termination
 9999 write(6,*)     'Program Terminated with ***ERRORS***'
      write(iolst,*) 'Program Terminated with ***ERRORS***'
      if(debug .gt. 0) then 
          write(iolog,*)'Program Terminated with ***ERRORS***'
      end if 
      close(iolst)
      close(iolog)
      close(ioab)
      close(iomet)
      stop
      end


c ---------------------------------------------------------------------
c subroutine mkfilcp()
c
c creates the series of output files for CALPUFF formated variable 
c emissions file
c ---------------------------------------------------------------------
      subroutine mkfilcp()
      include 'h_stddefs.for'
      include 'h_read1.for'
      include 'h_combustion.for'
      include 'h_met1.for'
      include 'h_file1.for'
      include 'h_isccalcs.for'
c --- DECLARATIONS
      real(8) :: fgri
      
      integer iloop, ip, ih0, imet
      integer ier
      character*255 :: FROOT
      character*255 :: FLOOP
      character(len=5)    ::  ulcase(2)
      data ulcase/'UPPER','LOWER'/
      
      
      do iloop=1,nloops   
c create output path for this loop
          
          write(FROOT,"('\',A,'.',I4.4,'_',I2.2,'.DAT')")
     &           OUTNAME(1:LEN_REAL(OUTNAME)),iyys,iloop
          call concat(FLOOP,OUTPATH,FROOT)
      if(DEBUG.gt.0) then
          write(iolog,*) '---------------------------------------------'
          write(iolog,*) 'loop=',iloop
          write(iolog,*) FLOOP
      end if                     
c write ABflare.dat header, open a unit
         call mk_case(iolst, ulcase(icase), FLOOP, ierr)
         if( MDSPMOD.eq.1) then
            ier = iwrFLEMARBhdr(ioab, iolst, FLOOP)
         else if( MDSPMOD.eq.2) then
            write(iolst,*) 'Probable bad combination '
            write(6,*) 'Probable bad combination '
            stop
         endif
c calculate start date stamp  
          iyr  = iyys
          ijul = ijjs
          ihr  = ihhs
          isec = isss
          call MKDAT(iyr,ijul,ihr,idatecur)

c Open METDAT
         if(MMET .eq. 1) then
            call methdr(iomet,iolog,METDAT,ws,wd,tamb,ipg)  
         else if(MMET .eq. 2) then
            call methdr_aermet(iomet,iolog,METDAT,ws,wd,tamb,ipg)  
         else            
            ws   = rwindsp
            if( rwinddr .ge. 0 ) then
               wd   = rwinddr
            end if 
            tamb     = rtamb
            ipg      = irpgclass
            ipgreal  = ipg
            zws      = rzref
            if(DEBUG .gt. 0 ) then
c               write(iolog,*) ' ws,wd,tamb,ipg'
c               write(iolog,*)  ws,wd,tamb,ipg
            end if 
         end if 
         tambreal = tamb
         wsreal   = ws
         ipgreal  = ipg
          
c Assume end of puff
          if(iloop.gt. 1) then
              ip = npuffs + 1
              ih0 = ihr - (24-(iloop-1)) 
          else
              ip = 1
              ih0 = ihr
          end if 

c calculate start hour of blowdown puff sequence
          iyr1    = iyr
          ijul1   = ijul
          ihr1    = ihr
          isec1   = isec
          if( ip .gt. 1 ) then
              call incrs(iolst,iyr1,ijul1,ihr1,isec1,(iloop-1)*3600)
c these routines only allow a maximum of -24h backward steps
c so do look backwards in nsep/24 times            
c              call incrs(iolst,iyr1,ijul1,ihr1,isec1,-(24)*3600)
              do i=1,nsep/24
               call incrs(iolst,iyr1,ijul1,ihr1,isec1,-(24)*3600)
              end do
          end if 
          call MKDAT(iyr1,ijul1,ihr1,iblowstart)
          
c ----------------------------------------------------------------------
          do while (idatecur .le. idateend)

          write(6,8001) iloop,idatecur
8001      format('+',I4,1x,I10)               

      if( debug .gt. 0) then
            write(iolog,*) '---------------------------------'
            write(iolog,*) 'iloop: ',iloop
            write(iolog,*) '---------------------------------'
              write(iolog,*)
              write(iolog,*)'idatecur,isec,',
     &        idatecur,isec
      end if 
          
              tamb   = tambreal 
              ws     = wsreal   
              ipg    = ipgreal
              fgri   = fgr

c get meteorology for datecur
            if(MMET .eq. 1) then
                call metget(iomet,iolst,idatecur,isec,ws,wd,tamb,ipg)
            else if(MMET .eq. 2) then
                call metget_aermet(iomet,iolst,idatecur,isec,ws,wd,tamb,
     &                       ipg)
            else
               ws   = rwindsp
               if( rwinddr .ge. 0 ) then
                     wd   = rwinddr
               end if 
               tamb = rtamb ! convert to K
               ipg  = irpgclass
               zws  = rzref
            end if    
            tambreal = tamb
            wsreal   = ws
            ipgreal  = ipg

         if( debug .gt. 0) then
            write(iolog,*) 'tamb=',tambreal
            write(iolog,*) 'ws=',wsreal
            write(iolog,*) 'ipg=',ipgreal
         end if             
      

c --- vessel exponential blowdown 
c repeat blowdown calculations if vessel temperature is varying           
c with ambient temperature
         if( minput.eq. 1. .and. mblowdown .eq. 2) then
            tempinitx = tempinit
            if( tempinitx .lt. 0. ) then
               tempinitx = tambreal
            end if               
          ierr= ivessel(gcraw_x,pressinit,tempinitx,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
              
          end if       

          if( minput.eq. 1. .and. mblowdown .eq. 2) then
              tgas = tgas2 ! tgas2 is calculated in iVessel()
          else    
             if(tginit .lt. 0.) then
               tgas = tamb
             else
               tgas = tginit
             end if 
          end if 
      if( debug .gt. 0) then
          write(iolog,*) 'tgas =',tgas
      end if 
      
c         tamb = 5
c         wd = 90
c         ws = 4.137 
c         tgas = tamb

c set puff 
              iemitt = 1
              ipuffdurx = nint(pseq(2,ip))
              
c adjust puff for start of next period
              if( ip .gt. npuffs) then
                  ip = 0
                  iemitt = 0
c calculate the ipuffdurx for the gap to the next cycle                  
                  if( isplit .eq. 1) then
c                      call deltsec(ihr,isec,ih0+nsep,0,ipuffdurx)
                   call DEDAT(iblowstart,iyr1,ijul1,ihr1)
                   call incrs(iolst,iyr1,ijul1,ihr1,isec1,(nsep)*3600)
                   call MKDAT(iyr1,ijul1,ihr1,iblowend)
                   call deltsec(idatecur,isec,iblowend,0,ipuffdurx)
                  else
c --- because the sequential puffs must be less than an hour, 
c we only need to test the seconds compared to timeinc
                      call deltsec(ihr,isec,ih0,itiminc,ipuffdurx)
                      if(ipuffdurx .ge. nsep*3600) then
                          ipuffdurx  = 0
                      end if 
                  end if
                  if(ipuffdurx.lt.0) then
                     ipuffdurx = 0
                  end if 
                  iy1 = iyr
                  ij1 = ijul
                  ih1 = ihr
                  is1 = isec
                  call incrs(iolst,iy1,ij1,ih1,is1,ipuffdurx)
                  ih0 = ih1
              end if  

c if zero duration, then reset to start of next sequence         
              if( ipuffdurx .eq. 0 .and. ip.eq.0 ) then
                  iemitt = 1
                  ip = 1
                  ipuffdurx = nint(pseq(2,ip))
                  call mkDAT(iyr,ijul,ihr,iblowstart)
              end if 

c set current emission rate 
              qmax1 = 0.0 
              if( ip .ne. 0) then 
                  qmax1     = pseq(1,ip)
              end if 
              if( qmax1 .le. 0.) then
                  iemitt = 0
              end if 
              
c set puff duration based upon source input
              if( MINPUT .eq. 2 .and. iemitt.eq.1) then
c set h2s to user input
                  rh2s1 = rh2s
                  if(pseq(3,ip) .ge. 0) then
                      rh2s1 = pseq(3,ip)
                  end if                       
c set fgr to user input
                  if(pseq(4,ip) .ge. 0) then
                      fgri = pseq(4,ip)
                  end if                       
                  ierr = inormalizeh2s(gcraw, rh2s1, gcraw_x)
              end if 

c calculate source conditions
c get xL, yL, zL
c get FB, FM
c get ds_x, hs_x, ts
              if(iemitt .eq. 1) then 
                  qtotal1   = qmax1*real(ipuffdurx)/24./3600.

c --- fuel gas is added proportional to current puff rate
                  if( MFGR .eq. 1) then
                      fgr1=fgri
c --- fuel gas is added proportional to qmax initial rate
c     need to back calculate effective FGR from qmax0
                  else if( MFGR .eq. 2) then
                      fgr1=(fgri*qmax0)/qmax1
                  end if 

c for a blowdown, the fgr changes with time relative to the blowdown.
c that is, fuel gas rate is fixed
c must calculate the fgr based upon current qmax and the initial fuel 
c gas flow 
c            tamb = 4.
c            tgas = tamb
c            ws = 23.641

         if( debug .gt. 0) then
            write(iolog,*) 'qmax=',qmax1
            write(iolog,*) 'fgri=',fgr1
         end if             

                  if(MINPUT .eq. 1 .or. MINPUT.eq. 2) then  
                      call emissions(gcraw_x,qmax1,gcfuel,fgr1,gclift,
     &  qtotal1,hs,ds,ws,zws,ipgreal,
     &  MSTRIP,MFUELGAS,MFASSIST,MLIFTGAS,
     &  qliftgas,stm_dia,stm_press,stm_temp,stm_flow,
     &  stm_nports,air_dia,air_flow,air_nports)
                  else if(MINPUT .eq. 3) then  
                      qemit  = rqemit
                      extemp = psts
                      ue_x   = psus 
                      de_x   = psds
                      hs_x   = pshs
                  end if 

                  XH(1)   = 0.0
                  DELH(1) = Hs
                  XH(2)   = XL
                  DELH(2) = hs_x

                  call calcDELH(wsreal, tambreal, ipgreal, 
     &            extemp, ue_x, de_x, hs_x, xh,DELH, zws)

      if( debug .gt. 0) then
            write(iolog,*) 'qemit=',qemit
                  write(iolog,*) 'extemp=',extemp
                  write(iolog,*) 'ue_x=',ue_x
                  write(iolog,*) 'de_x=',de_x
                  write(iolog,*) 'hs_x=',hs_x
                  write(iolog,*) 'ipg=',ipgreal
                  write(iolog,*)'FBisc,FMisc',fbisc,fmisc
                  write(iolog,*) 'DELH'
                  write(iolog,*) 'Hs',XH(1),DELH(1)
                  write(iolog,*) 'XL',XH(2),DELH(2)
                  write(iolog,*) 'XF*1/3',XH(3),DELH(3)
                  write(iolog,*) 'XF*2/3',XH(4),DELH(4)
                  write(iolog,*) 'XF', XH(5),DELH(5)
      end if 
                  ts_x  = extemp
                  fmx   = 0.
                  fmy   = 0.
                  fmz   = fm
                  usw   = ue_x
                  usu = 0. 
                  usv = 0.
c --- components
c            usw = uv
c            usu = uu*sin(wd*pi/180.)
c            usv = uu*cos(wd*pi/180.)
                  wd1 = (wd-180.)
                  if(wd1 .lt. 0.) wd1 = wd1 + 360.
                  x = XL*sin(wd1*pi/180.)
                  y = XL*cos(wd1*pi/180.)
                  z = hs_x
                  elev = zcoord
      if( debug .gt. 0) then
                  write(iolog,*) 'x,y,z',x,y,z
      end if
                  if(MUNITS .eq. 1) then
                      x = xcoord + x
                      y = ycoord + y
                  else
                      x = xcoord+ x*0.001
                      y = ycoord + y*0.001
                  endif 
                  q(1) = qemit
                  sy = 0.
                  sz = 0.
              else
                  hs_x  = hs
                  de_x  = ds
                  ts_x  = tamb
                  usw = 0. 
                  usu = 0. 
                  usv = 0.
                  fb    = 0.
                  fmx   = 0.
                  fmy   = 0.
                  fmz   = 0.
                  x = xcoord
                  y = ycoord
                  z = hs_x
                  elev  = zcoord
                  q(1) = 0.
                  sy = 0.
                  sz = 0.
              end if

c append the static emissions to the list              
              if(nspec.gt.1) then
              do ie=2,nspec
                 q(ie) = qlst(ie)
              end do 
              end if 
c increment time
              iy1 = iyr
              ij1 = ijul
              ih1 = ihr
              is1 = isec
              call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
              iy2 = iyr
              ij2 = ijul
              ih2 = ihr
              is2 = isec

c write ABflare puff
            ivar = 1
            if(MDSPMOD .eq. 1) then
c write the CALPUFF file               
               ier = iwrFLEMARBi(ioab,ivar,sname,
     &        iy1,ij1,ih1,is1,
     &        iy2,ij2,ih2,is2,
     &        x,y,z,elev,de_x,ts_x,usu,usv,usw,
     &        fb,fmx,fmy,fmz,sy,sz,q,nspec)
            else
c future 
c write the variable aermod file               
c               ier = iwrFLEMARBi_aermod(ioab,ivar,sname,
c     &        iy1,ij1,ih1,is1,
c     &        iy2,ij2,ih2,is2,
c     &        x,y,z,elev,de_x,ts_x,usu,usv,usw,
c     &        fb,fmx,fmy,fmz,sy,sz,q,nspec)
            end if 

c increment time
c         call incrs(iolst,iyr,ijul,ihr,isec,1)
              ip       = ip + 1
              call MKDAT(iyr,ijul,ihr,idatecur)
c reset blowdown start time
              if(ip .eq. 1 ) then
                  call mkDAT(iyr,ijul,ihr,iblowstart)
              end if 
          end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
          close (iomet)
          close (ioab)   
      end do ! iloop=1,nloops   

      return
      end 

c ---------------------------------------------------------------------
c subroutine mkfilam()
c
c creates the series of output files for AERMOD formated variable 
c emissions file
c ---------------------------------------------------------------------
      subroutine mkfilam()
      include 'h_stddefs.for'
      include 'h_read1.for'
      include 'h_combustion.for'
      include 'h_met1.for'
      include 'h_file1.for'
      include 'h_isccalcs.for'
c --- DECLARATIONS
      real(8) :: fgri
      integer :: indx(mxmet)  ! index
      real(8) :: vrise(mxmet) ! plume rise, m
      real(8) :: vdex(mxmet)  ! source diameter, m
      real(8) :: vhsx(mxmet)  ! source height, m
      real(8) :: vx(mxmet)  ! source x-location, m
      real(8) :: vy(mxmet)  ! source y-location, m
      real(8) :: ve(mxmet)   ! source emission rate, g/s
      real(8) :: vusx(mxmet)   ! source exit velociy, m/s
      real(8) :: vtsx(mxmet)  ! source temperature,K
      integer :: ivs(mxmet)   ! source #

      real(8) :: vsTs(mxvsrc)  ! averae Ts
      real(8) :: vsUs(mxvsrc)  ! averae Us
      real(8) :: vsHs(mxvsrc)  ! averae hs
      real(8) :: vsDs(mxvsrc)  ! average ds
      real(8) :: vsX(mxvsrc)  ! average x
      real(8) :: vsY(mxvsrc)  ! average y
      integer :: ivsNs(mxvsrc) ! number of in group
      
      integer iloop, ip, ih0, imet
      integer ier
      character*255 :: FROOT
      character*255 :: FLOOP
      character*12 :: srcname
      character*12 :: srcname2
      character(len=5)    ::  ulcase(2)
      data ulcase/'UPPER','LOWER'/
      
      nloops = npuffs
      nsep   = 1
      ipuffdur  = 3600
      ipuffdurx = 3600

c ---------------------------------------------------------------------
c ----------------------------------------------------------------------
      do iloop=1,nloops                    

c ----------------------------------------------------------------------
c     cycle through all dates and create a plume rise list

c calculate start date stamp  
         iyr  = iyys
         ijul = ijjs
         ihr  = ihhs
         isec = isss
         call MKDAT(iyr,ijul,ihr,idatecur)

c Open METDAT
         if(MMET .eq. 1) then
            call methdr(iomet,iolog,METDAT,ws,wd,tamb,ipg)  
         else if(MMET .eq. 2) then
            call methdr_aermet(iomet,iolog,METDAT,ws,wd,tamb,ipg)  
         else            
            ws   = rwindsp
            if( rwinddr .ge. 0 ) then
               wd   = rwinddr
            end if 
            tamb     = rtamb
            ipg      = irpgclass
            ipgreal  = ipg
            zws      = rzref
         end if 
         tambreal = tamb
         wsreal   = ws
         ipgreal  = ipg
         imet     = 0 
c ----------------------------------------------------------------------
         do while (idatecur .le. idateend)

          write(6,8001) iloop,idatecur
8001      format('+',I4,1x,I10)               

         if( debug .gt. 0) then
            write(iolog,*) '---------------------------------'
            write(iolog,*) 'iloop: ',iloop
            write(iolog,*) '---------------------------------'
            write(iolog,*) idatecur
         end if           
            tamb   = tambreal 
            ws     = wsreal   
            ipg    = ipgreal
            fgri   = fgr

c get meteorology for datecur
            if(MMET .eq. 1) then
                call metget(iomet,iolst,idatecur,isec,ws,wd,tamb,ipg)
            else if(MMET .eq. 2) then
                call metget_aermet(iomet,iolst,idatecur,isec,ws,wd,tamb,
     &                       ipg)
            else
               ws   = rwindsp
               if( rwinddr .ge. 0 ) then
                     wd   = rwinddr
               end if 
               tamb = rtamb ! convert user entry from C to K
               ipg  = irpgclass
               ipgreal  = ipg
               zws  = rzref
            end if    
            tambreal = tamb
            wsreal   = ws
            ipgreal  = ipg

         if( debug .gt. 0) then
            write(iolog,*) 'tamb=',tambreal
            write(iolog,*) 'ws=',wsreal
            write(iolog,*) 'ipg=',ipgreal
         end if             
         
            
c --- vessel exponential blowdown 
c repeat blowdown calculations if vessel temperature is varying           
c with ambient temperature
          if( minput.eq. 1. .and. mblowdown .eq. 2) then
             tempinitx = tempinit
             if( tempinitx .lt. 0. ) then
                 tempinitx = tambreal
             end if               
             ierr= ivessel(gcraw_x,pressinit,tempinitx,
     &                 pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgas2   )
          end if       

          if( minput.eq. 1. .and. mblowdown .eq. 2) then
              tgas = tgas2 ! tgas2 is calculated in iVessel()
          else    
             if(tginit .lt. 0.) then
               tgas = tamb
             else
               tgas = tginit
             end if 
          end if 
          
c         tamb = 5
c         wd = 90
c         ws = 4.137 
c         tgas = tamb

c set puff 
         ip       = iloop
         iemitt   = 1
         qmax1    = pseq(1,ip)
         
c set puff duration based upon source input
c         ipuffdurx = nint(pseq(2,ip))
         ipuffdurx = 3600

         if( MINPUT .eq. 2 ) then
c set h2s to user input
            rh2s1 = rh2s
            if(pseq(3,ip) .ge. 0) then
               rh2s1 = pseq(3,ip)
            end if                       
c set fgr to user input
            if(pseq(4,ip) .ge. 0) then
               fgri = pseq(4,ip)
            end if                       
            ierr = inormalizeh2s(gcraw, rh2s1, gcraw_x)
         end if 

c calculate source conditions
c get xL, yL, zL
c get FB, FM
c get ds_x, hs_x, ts
         qtotal1   = qmax1*real(ipuffdurx)/24./3600.

c --- fuel gas is added proportional to current puff rate
         if( MFGR .eq. 1) then
               fgr1=fgri
c --- fuel gas is added proportional to qmax initial rate
c     need to back calculate effective FGR from qmax0
         else if( MFGR .eq. 2) then
               fgr1=(fgri*qmax0)/qmax1
         end if 

c for a blowdown, the fgr changes with time relative to the blowdown.
c that is, fuel gas rate is fixed
c must calculate the fgr based upon current qmax and the initial fuel 
c gas flow 
c            tamb = 4.
c            tgas = tamb
c            ws = 23.641

         if( debug .gt. 0) then
            write(iolog,*) 'qmax=',qmax1
            write(iolog,*) 'fgri=',fgr1
         end if             

         if(MINPUT .eq. 1 .or. MINPUT.eq. 2) then  
               call emissions(gcraw_x,qmax1,gcfuel,fgr1,gclift,
     &  qtotal1,hs,ds,ws,zws,ipgreal,
     &  MSTRIP,MFUELGAS,MFASSIST,MLIFTGAS,
     &  qliftgas,stm_dia,stm_press,stm_temp,stm_flow,
     &  stm_nports,air_dia,air_flow,air_nports)
         else if(MINPUT .eq. 3) then  
            qemit  = rqemit
            extemp = psts
            ue_x   = psus 
            de_x   = psds
            hs_x   = pshs
         end if 

         XH(1)   = 0.0
         DELH(1) = Hs
         XH(2)   = XL
         DELH(2) = hs_x

            call calcDELH(wsreal, tambreal, ipgreal, 
     &            extemp, ue_x, de_x, hs_x, xh,DELH, zws)

         if( debug .gt. 0) then
            write(iolog,*) 'qemit=',qemit
            write(iolog,*) 'extemp=',extemp
            write(iolog,*) 'ue_x=',ue_x
            write(iolog,*) 'de_x=',de_x
            write(iolog,*) 'hs_x=',hs_x
            write(iolog,*) 'ipg=',ipgreal
            write(iolog,*)'FBisc,FMisc',fbisc,fmisc
            write(iolog,*) 'DELH'
            write(iolog,*) 'Hs',XH(1),DELH(1)
            write(iolog,*) 'XL',XH(2),DELH(2)
            write(iolog,*) 'XF*1/3',XH(3),DELH(3)
            write(iolog,*) 'XF*2/3',XH(4),DELH(4)
            write(iolog,*) 'XF', XH(5),DELH(5)
         end if 
         ts_x  = extemp
         fmx   = 0.
         fmy   = 0.
         fmz   = fm
         usw   = ue_x
         usu = 0. 
         usv = 0.
c --- components
c            usw = uv
c            usu = uu*sin(wd*pi/180.)
c            usv = uu*cos(wd*pi/180.)
         wd1 = (wd-180.)
         if(wd1 .lt. 0.) wd1 = wd1 + 360.
         x = XL*sin(wd1*pi/180.)
         y = XL*cos(wd1*pi/180.)
         z = hs_x
         elev = zcoord
         if(MUNITS .eq. 1) then
               x = xcoord + x
               y = ycoord + y
         else
               x = xcoord + x*0.001
               y = ycoord + y*0.001
         endif 
         q(1) = qemit
         sy = 0.
         sz = 0.

c Save the information for later indexing         
         imet = imet + 1
         indx(imet) = imet
         vrise(imet) = DELH(5)
         vdex(imet) = de_x
         vhsx(imet) = hs_x
         vusx(imet) = ue_x
         vtsx(imet) = extemp
         ve(imet) = qemit
         vx(imet) = x
         vy(imet) = y
     
         
c increment time
         iy1 = iyr
         ij1 = ijul
         ih1 = ihr
         is1 = isec
         call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
         iy2 = iyr
         ij2 = ijul
         ih2 = ihr
         is2 = isec

         ip       = ip + 1
         call MKDAT(iyr,ijul,ihr,idatecur)
      end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
      close (iomet)

c =======================================
c determine virtual sources
      nmet = imet   
c Determine final rise heights limits
      Call HSORTrL(nmet, vrise, indx)

c determine average for each for the virtual sources
      do i=1,NSOURCES
         vsTs(i) = 0.
         vsUs(i) = 0.
         vsHs(i) = 0.
         vsDs(i) = 0.
         vsX(i) = 0.
         vsY(i) = 0.
         ivsNs(i) = 0
      end do    
      do imet = 1,nmet
         ix = indx(imet)
         j = 1 + int(real(imet)/(real(nmet)/ real(NSOURCES)))
         j = min(j,nsources)
         ivsNs(j) = ivsNs(j) + 1
         vsTs(j) = vsTs(j) + vtsx(ix)
         vsUs(j) = vsUs(j) + vusx(ix)
         vsDs(j) = vsDs(j) + vdex(ix)
         vsHs(j) = vsHs(j) + vhsx(ix)
         vsX(j) = vsX(j) + vx(ix)
         vsY(j) = vsY(j) + vy(ix)
         ivs(ix) = j 
      end do
      do i=1,NSOURCES
         If( ivsNs(i) .gt. 0 ) Then 
            vsTs(i) = vsTs(i) / ivsNs(i)
            vsUs(i) = vsUs(i) / ivsNs(i)
            vsDs(i) = vsDs(i) / ivsNs(i)
            vsHs(i) = vsHs(i) / ivsNs(i)
            vsX(i) = vsX(i) / ivsNs(i)
            vsY(i) = vsY(i) / ivsNs(i)
         end if             
      end do    
c =======================================

c create output path for this loop
      write(FROOT,"(A,'.','MOD','_',I2.2,'.INX')")
     &      OUTNAME(1:LEN_REAL(OUTNAME)),iloop
      call concat(FLOOP,OUTPATH,FROOT)
      call mk_case(iolst, ulcase(icase), FLOOP, ierr)
      open(ioab,file=FLOOP,iostat=ierr,status='unknown')

      write(ioab,"('CO STARTING')")
      write(ioab,"('CO TITLEONE ERCBflare ')")
      if(MSTRIP.eq.1) then
         write(ioab,"('CO TITLETWO H2S EMISSIONS')")
      else         
         write(ioab,"('CO TITLETWO SO2 EMISSIONS')")
      end if          
      write(ioab,"('CO MODELOPT NOCHKD CONC NOSTD ELEV')")
      write(ioab,"('CO AVERTIME 1')")
      if(MSTRIP.eq.1) then
         write(ioab,"('CO POLLUTID H2S')")
      else
         write(ioab,"('CO POLLUTID SO2')")
      end if          
      write(ioab,"('CO RUNORNOT RUN')")
      write(ioab,"('CO ERRORFIL AERMOD.err')")
      write(ioab,"('CO FINISHED')")
      
      write(ioab,"('SO STARTING')")
      write(ioab,"('SO ELEVUNIT METERS')")
      write(ioab,"('SO EMISUNIT    .100000E+07 ')") ! (GRAMS/SEC)          (MICROGRAMS/CUBIC-METER)')")
      write(ioab,"('** SRCPARAM SOURCE1  QS    HS    TS    US    DS')")

      do i=1,NSOURCES
         if(i .lt. 10) then
            write(srcname,"('SOURCE',i1)") i
         else
            write(srcname,"('SOURCE',i2)") i
         end if 
         write(ioab,"('SO LOCATION ',A,' POINT',3(F13.4,1x))")
     &                           srcname,vsx(i),vsy(i), zcoord
         e1 = 0.
         write(ioab,"('SO SRCPARAM ',5(F10.3,1x))")
     &                           e1,vshs(i),vsts(i),vsus(i),vsds(i)     
      end do
c --- output file
      write(FROOT,"(A,'.','MOD','_',I2.2,'.DAT')")
     &    OUTNAME(1:LEN_REAL(OUTNAME)),iloop
      call concat(FLOOP,OUTPATH,FROOT)
      call mk_case(iolst, ulcase(icase), FLOOP, ierr)
      n=LEN_REAL(FLOOP)
      i=1
         if(i .lt. 10) then
            write(srcname,"('SOURCE',i1)") i
         else
            write(srcname,"('SOURCE',i2)") i
         end if 
      i=NSOURCES         
         if(i .lt. 10) then
            write(srcname2,"('SOURCE',i1)") i
         else
            write(srcname2,"('SOURCE',i2)") i
         end if 
      write(ioab,"('SO HOUREMIS ',A,' ',A12,'-',A12)") 
     &       FLOOP(1:n),srcname,srcname2
      write(ioab,"('SO SRCGROUP  ALL')")
      write(ioab,"('SO FINISHED')")
      write(ioab,"('RE STARTING')")
      write(ioab,"('RE FINISHED')")
      write(ioab,"('ME STARTING')")
      n=LEN_REAL(METDAT)
      write(ioab,"('ME SURFFILE ',A)") METDAT(1:n)
      n=LEN_REAL(METDAT2)
      write(ioab,"('ME PROFFILE ',A)") METDAT2(1:n)
      write(ioab,"('ME SURFDATA  00011111  2010  ONSITE')")
      write(ioab,"('ME UAIRDATA  00022222  2010  ONSITE')")
      write(ioab,"('ME PROFBASE ',f10.1,' METERS')"), zcoord
      write(ioab,"('ME FINISHED')")
      write(ioab,"('OU STARTING')")
      write(ioab,"('OU RECTABLE ALLAVE FIRST')")
      write(ioab,"('OU MAXTABLE ALLAVE 10')")
      write(ioab,"('OU PLOTFILE 1 ALL FIRST aermod.dat')")
      write(ioab,"('OU POSTFILE 1 ALL UNFORM aermod.bin')")
      write(ioab,"('OU FINISHED')")

      close(ioab)
      
c ----------------------------------------------------------------------
c create output path for this loop
         write(FROOT,"(A,'.','MOD','_',I2.2,'.DAT')")
     &        OUTNAME(1:LEN_REAL(OUTNAME)),iloop
         call concat(FLOOP,OUTPATH,FROOT)
         call mk_case(iolst, ulcase(icase), FLOOP, ierr)
c write ABflare.dat header, open a unit
         if( MDSPMOD.eq.1) then
            write(iolst,*) 'Probable bad combination '
            write(6,*) 'Probable bad combination '
            stop
         else if( MDSPMOD.eq.2) then
c            ier = iwrFLEMARBhdr_aermod(ioab, iolst, FLOOP)
         endif
      if(DEBUG.gt.0) then
          write(iolog,*) '---------------------------------------------'
          write(iolog,*) 'loop=',iloop
          write(iolog,*) FLOOP
      end if                     
         open(ioab,file=FLOOP,iostat=ierr,status='unknown')
c calculate start date stamp  
          iyr  = iyys
          ijul = ijjs
          ihr  = ihhs
          isec = isss
          call MKDAT(iyr,ijul,ihr,idatecur)
          
c ----------------------------------------------------------------------
          imet = 0
          do while (idatecur .le. idateend)
c          write(6,8002) iloop,idatecur
c8002  format('+',I4,1x,I10)               
      
c increment time
            iy1 = iyr
            ij1 = ijul
            ih1 = ihr
            is1 = isec
            call incrs(iolst,iyr,ijul,ihr,isec,ipuffdurx)
            iy2 = iyr
            ij2 = ijul
            ih2 = ihr
            is2 = isec

            call grday(iolst,iyr1,ij1,imo1,iday1)
            
c write ABflare puff
c            ivar = 1
c            if(MDSPMOD .eq. 1) then
c               stop
c            else
c               ier = iwrFLEMARBi_aermod(ioab,ivar,sname,
c     &        iy1,ij1,ih1,is1,
c     &        iy2,ij2,ih2,is2,
c     &        x,y,z,elev,de_x,ts_x,usu,usv,usw,
c     &        fb,fmx,fmy,fmz,sy,sz,q,nspec)
c            end if 
         imet = imet + 1   
         do i=1,NSOURCES
            e1 =  ve(imet)
            if(ivs(imet) .ne. i) then
               e1 = 0.
            end if 
            if(i .lt. 10) then
               write(srcname,"('SOURCE',i1)") i
            else
               write(srcname,"('SOURCE',i2)") i
            end if
c adjust year to two digits            
            if(iy1.gt.2000) then
               iyout = iy1 -2000
            else if (iy1.gt.1900) then
               iyout = iy1 -1900
            else
               iyout = iy1
            end if 
            write(ioab,8003) 'SO HOUREMIS',
     &            iyout,imo1,iday1,(ih1+1),srcname,
     &            e1,vtsx(imet),vusx(imet)
8003     format(A,4(1x,i3),1x,A12,3(f10.3,1x))      
         end do
         
c increment time
c         call incrs(iolst,iyr,ijul,ihr,isec,1)
              ip       = ip + 1
              call MKDAT(iyr,ijul,ihr,idatecur)
          end do ! (datecur .le. idateend)
c ----------------------------------------------------------------------
          close (iomet)
          close (ioab)   
      end do ! iloop=1,nloops   

      return
      end 


      subroutine HSORTrL(n, RA, IB)
c ... INPUTS
      integer :: n, IB(*)
      real(8) :: RA(*)
c ... DECLARATIONS
      integer :: L 
      integer :: ir
      integer :: i
      integer :: j, IIB
      real(8) :: RRA
      L = n / 2 + 1
      ir = n
10    CONTINUE
      If (L.gt.1) Then
         L = L - 1
         RRA = RA(L)
         IIB = IB(L)
      Else
         RRA = RA(ir)
         IIB = IB(ir)
         RA(ir) = RA(1)
         IB(ir) = IB(1)
         ir = ir - 1
         If (ir.eq.1) Then
            RA(1) = RRA
            IB(1) = IIB
            return
         End If
      End If
      i = L
      j = L + L
20    continue
      If (j .le.ir) Then
         If (j .lt. ir) Then
            If (RA(j) .lt. RA(j + 1)) Then 
               j = j + 1
            end if    
         End If
         If (RRA .lt. RA(j)) Then
            RA(i) = RA(j)
            IB(i) = IB(j)
            i = j
            j = j + j
         Else
            j = ir + 1
         End If
         GoTo 20
      End If
      RA(i) = RRA
      IB(i) = IIB
      GoTo 10
      return
      end 

c ----------------------------------------------------------------------
c HEAP SORT ON FIVE R and one I ARRAYS
c ----------------------------------------------------------------------
      subroutine HSORT5rL(n, RA, RB, RC, RD, RE, IB)
c ... INPUTS
      integer :: n, IB(*)
      real(8) :: RA(*), RB(*), RC(*), RD(*), RE(*) 
c ... DECLARATIONS
      integer :: L 
      integer :: ir
      integer :: i
      integer :: j, IIB
      real(8) :: RRA, RRB, RRC, RRD, RRE
      L = n / 2 + 1
      ir = n
10    CONTINUE
      If (L.gt.1) Then
         L = L - 1
         RRA = RA(L)
         RRB = RB(L)
         RRC = RC(L)
         RRD = RD(L)
         RRE = RE(L)
         IIB = IB(L)
      Else
         RRA = RA(ir)
         RRB = RB(ir)
         RRC = RC(ir)
         RRD = RD(ir)
         RRE = RE(ir)
         IIB = IB(ir)
         RA(ir) = RA(1)
         RB(ir) = RB(1)
         RC(ir) = RC(1)
         RD(ir) = RD(1)
         RE(ir) = RE(1)
         IB(ir) = IB(1)
         ir = ir - 1
         If (ir.eq.1) Then
            RA(1) = RRA
            RB(1) = RRB
            RC(1) = RRC
            RD(1) = RRD
            RE(1) = RRE
            IB(1) = IIB
            return
         End If
      End If
      i = L
      j = L + L
20    continue
      If (j .le.ir) Then
         If (j .lt. ir) Then
            If (RA(j) .lt. RA(j + 1)) Then 
               j = j + 1
            end if    
         End If
         If (RRA .lt. RA(j)) Then
            RA(i) = RA(j)
            RB(i) = RB(j)
            RC(i) = RC(j)
            RD(i) = RD(j)
            RE(i) = RE(j)
            IB(i) = IB(j)
            i = j
            j = j + j
         Else
            j = ir + 1
         End If
         GoTo 20
      End If
      RA(i) = RRA
      RB(i) = RRB
      RC(i) = RRC
      RD(i) = RRD
      RE(i) = RRE
      IB(i) = IIB
      GoTo 10
      return
      end 

      

c----------------------------------------------------------------------
c irdcontrol
c----------------------------------------------------------------------
c --- Developed by B.Zelt - www.zeltpsi.com
c
c     Reads the control file and parses the lines into keywords
c     and user inputs
c
c UPDATES
c ---   Version: 1.002    Level: 121112
c     - added lift gas
c     - added aermod capacity
c
c ---   Version: 1.001    Level: 110706
c     
c INPUTS
c     FIN     : char(128) : path of the file to read
c
c OUTPUT
c     ierr: integer   : 0  no errors found
c                     : 1  errors were found
c
c----------------------------------------------------------------------
      function irdcontrol(FIN )
     &result(ierr)

c   Common blocks
      include 'h_stddefs.for'
      include 'h_read1.for'
c    INPUTS
      integer       :: ierr
      character*128 :: FIN    ! path for file input
c    INPUTs Declarations
      integer     :: iversion   ! version of the input file
      character*128 :: dummy
      character*16  :: imodel
      character*16  :: specname
      integer       :: ioin
      integer       :: io
c      include 'h_interface.for'
      
      io = ioin    
      ierr = 1   
      ier = 0
      open(unit=io,file=FIN,iostat=ier,status='old')
      if(ier .ne. 0) goto 9000

      read(io,*) imodel,iversion

c      write(io,*) ' Model:',mmodel,' Version:',mversion,' Level:',mlevel
      ier = irdparam_a(io, 'STITLE1', stitle)
      ier = irdparam_a(io, 'SNAME', sname)
      ier = irdparam_a(io, 'OUTNAME', OUTNAME)
      ier = irdparam_a(io, 'OUTPATH', OUTPATH)
      ier = irdparam_a(io, 'METDAT', METDAT)
      ier = irdparam_a(io, 'LSTDAT', LSTDAT)
      ier = irdparam_a(io, 'LOGDAT', LOGDAT)
      ier = irdparam_a(io, 'LCFILES', dummy)
      lcfiles = .FALSE.
      if( dummy(1:1) .eq. 't' .or. dummy(1:1) .eq. 'T' ) then
          lcfiles = .TRUE.
      end if   

      ier = irdparam_i(io, 'MDSPMOD', MDSPMOD)
      ier = irdparam_i(io, 'MINPUT', MINPUT)
      ier = irdparam_i(io, 'MBLOWDOWN',   mblowdown )  ! 2:Calculated from vessel pressure 1:user input
      ier = irdparam_i(io, 'MDIST',       mdist )  ! 1:equal duration; 2:equal mass
      ier = irdparam_i(io, 'MFGR', MFGR)
      ier = irdparam_i(io, 'MSTRIP', MSTRIP)
      ier = irdparam_i(io, 'MMET', MMET)
      ier = irdparam_i(io, 'MDEBUG', MDEBUG)
      ier = irdparam_i(io, 'NSEP', NSEP)  
      ier = irdparam_i(io, 'NSOURCES', NSOURCES)  

c read second met file for AERMOD files      
      if(MMET.eq.2) then
      ier = irdparam_a(io, 'METDAT2', METDAT2)
      end if 
      
      ier = irdparam_a(io, 'UTMHEM', UTMHEM)
      ier = irdparam_i(io, 'IUTMZN', IUTMZN)
      ier = irdparam_a(io, 'PMAP', PMAP)
      ier = irdparam_a(io, 'DATUM', DATUM)
      ier = irdparam_a(io, 'UNITS', UNITS)
      ier = irdparam_a(io, 'ABTZ', ABTZ)
      ier = irdparam_i(io, 'NSPEC', nspec)
      ier = irdparam_av(io, 'CHEMLST', nspec, chemlst)
      ier = irdparam_rv(io, 'MW', nspec, mmlst)
      ier = irdparam_rv(io, 'EMIT', nspec, qlst)

c      write(6,*) 'title=[',setitle,']'
c      ier = irdparam_i(io, 'idspl ',idspl  )
      ier = irdparam_i(io, 'YYS',     iyys  )
      ier = irdparam_i(io, 'MMS',     imms  )
      ier = irdparam_i(io, 'DDS',     idds  )
      ier = irdparam_i(io, 'HHS',     ihhs  )
      ier = irdparam_i(io, 'SSS',     isss  )
      ier = irdparam_i(io, 'YYE',     iyye  )
      ier = irdparam_i(io, 'MME',     imme  )
      ier = irdparam_i(io, 'DDE',     idde  )
      ier = irdparam_i(io, 'HHE',     ihhe  )
      ier = irdparam_i(io, 'SSE',     isse  )
      ier = irdparam_i(io, 'TIMEINC',  itiminc  )
      ier = irdparam_r(io, 'XCOORD',     xcoord   )
      ier = irdparam_r(io, 'YCOORD',     ycoord   )
      ier = irdparam_r(io, 'ZCOORD',     zcoord   )
      ier = irdparam_r(io, 'HS  ',     hs   )
      ier = irdparam_r(io, 'DS  ',     ds  )
      ier = irdparam_r(io, 'RH2S',     rh2s )
      ier = irdparam_r(io, 'TGINIT',   tginit )

c --- determine source configuration

***      ier = irdparam_r(io, 'frel',     frel )
***      ier = irdparam_r(io, 'duration', duration )

      if(MINPUT .eq. 1 ) then
          if( mblowdown .eq. 1) then
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'QTOTAL',   qtotal )
              ier = irdparam_r(io, 'FREL',     frel )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
          else if( mblowdown .eq. 2 ) then         
              ier = irdparam_r(io, 'QMAX',     qmax )
              ier = irdparam_r(io, 'PRESSINIT',       pressinit )
              ier = irdparam_r(io, 'TEMPINIT',        tempinit )
              ier = irdparam_r(io, 'PRESSFINAL',      pressfinal )
              ier = irdparam_r(io, 'VESSELDIA',       vesseldia )
              ier = irdparam_r(io, 'VESSELLEN',       vessellen )
              ier = irdparam_r(io, 'ORIFICEDIA',      orificedia )
              ier = irdparam_r(io, 'DISCHARGECOEF',   dischargecoef )

c              ier = irdparam_r(io, 'DURATION',    duration)      
              ier = irdparam_i(io, 'PUFFDUR',     ipuffdur )      ! one of 
              ier = irdparam_i(io, 'NPUFFS',     npuffs )
          end if 
      else if( MINPUT .eq. 2 ) then
c first value is flowrate, second is duration, 3rd is h2s
          ier = irdparam_rv2(io, 'NSEQ', nseq, 4, pseq )
          npuffs = nseq
      else if( MINPUT .eq. 3 ) then
          npuffs    = 1
          ipuffdur = 3600
c          ier = irdparam_i(io, 'puffdur',     ipuffdur )
          ier = irdparam_r(io, 'PSHS', pshs )
          ier = irdparam_r(io, 'PSDS', psds )
          ier = irdparam_r(io, 'PSTS', psts )
          ier = irdparam_r(io, 'PSUS', psus )
c          ier = irdparam_r(io, 'QEMIT', rqemit )
          rqemit = qlst(1)
      end if 

c --- Fuel gas      
      ier = irdparam_i(io, 'MFUELGAS',     mfuelgas )
      if(mfuelgas .eq. 0) then
          fgr = 0.
      else
          ier = irdparam_r(io, 'FGR',      fgr )
      end if           

c --- lift gas      
      ier = irdparam_i(io, 'MLIFTGAS',     mliftgas )
      if(mliftgas.eq. 0) then
          qliftgas = 0.
      else
          ier = irdparam_r(io, 'QLIFTGAS',      qliftgas )  ! e3m3/d
      end if           

      
c need QMAX for MFGR=2          
        if (MFGR .eq. 2) then
        ier = irdparam_r(io, 'QMAX',     qmax )
            if(ier .eq.0 ) then
        write(6,*)      '***FATAL ERROR*** Missing QMAX with MFGR=2'
                stop
            endif 
        end if           
      
      ier = irdparam_i(io, 'MFASSIST',     MFASSIST )
c --- group FLARE ASSIST
c     initialize
      stm_dia 	  = 0. 
      stm_press 	  = -1. 
      stm_temp 	  = -1. 
      stm_flow 	  = 0. 
      stm_nports 	  = 0 
      stm_imassvol  = 0 
      air_nports 	  = 0 
      air_dia 	  = 0. 
      air_flow 	  = 0. 
c --- steam assist
      if( MFASSIST .eq. 1 .or. MFASSIST .eq. 3) then
          ier = irdparam_i(io, 'STM_NPORTS  ',  stm_nports )
          ier = irdparam_r(io, 'STM_DIA     ',  stm_dia )
          ier = irdparam_i(io, 'STM_IMASSVOL',  stm_imassvol )
          ier = irdparam_r(io, 'STM_PRESS   ',  stm_press )
          ier = irdparam_r(io, 'STM_TEMP    ',  stm_temp )
          ier = irdparam_r(io, 'STM_FLOW    ',  stm_flow )
      end if 
c --- air assist
      if( MFASSIST .eq. 2 .or. MFASSIST .eq. 3) then
          ier = irdparam_i(io, 'AIR_NPORTS ',  air_nports )
          ier = irdparam_r(io, 'AIR_DIA    ',  air_dia )
          ier = irdparam_r(io, 'AIR_FLOW   ',  air_flow )
      end if 

      ier = irdparam_r(io, 'RAW_H2O     ',  gcraw(jph2o     ) )
      ier = irdparam_r(io, 'RAW_H2      ',  gcraw(jph2      ) )
      ier = irdparam_r(io, 'RAW_HE      ',  gcraw(jphe      ) )
      ier = irdparam_r(io, 'RAW_N2      ',  gcraw(jpn2      ) )
      ier = irdparam_r(io, 'RAW_CO2     ',  gcraw(jpco2     ) )
      ier = irdparam_r(io, 'RAW_H2S     ',  gcraw(jph2s     ) )
      ier = irdparam_r(io, 'RAW_C1      ',  gcraw(jpch4     ) )
      ier = irdparam_r(io, 'RAW_C2      ',  gcraw(jpc2h6    ) )
      ier = irdparam_r(io, 'RAW_C3      ',  gcraw(jpc3h8    ) )
      ier = irdparam_r(io, 'RAW_IC4     ',  gcraw(jpic4h10  ) )
      ier = irdparam_r(io, 'RAW_NC4     ',  gcraw(jpnc4h10  ) )
      ier = irdparam_r(io, 'RAW_IC5     ',  gcraw(jpic5h12  ) )
      ier = irdparam_r(io, 'RAW_NC5     ',  gcraw(jpnc5h12  ) )
      ier = irdparam_r(io, 'RAW_NC6     ',  gcraw(jpnc6h14  ) )
      ier = irdparam_r(io, 'RAW_C7      ',  gcraw(jpnc7h16x ) )
      ier = irdparam_r(io, 'RAW_CO      ',  gcraw(jpco      ) )
      ier = irdparam_r(io, 'RAW_NH3     ',  gcraw(jpnh3     ) )
      ier = irdparam_r(io, 'RAW_AR      ',  gcraw(jpar      ) )

      if(mfuelgas .eq. 0) then
         ier = initialize(gcfuel,0.0D0)
      else          
          ier = irdparam_r(io, 'FUEL_H2O    ',  gcfuel(jph2o     ) )
          ier = irdparam_r(io, 'FUEL_H2     ',  gcfuel(jph2      ) )
          ier = irdparam_r(io, 'FUEL_HE     ',  gcfuel(jphe      ) )
          ier = irdparam_r(io, 'FUEL_N2     ',  gcfuel(jpn2      ) )
          ier = irdparam_r(io, 'FUEL_CO2    ',  gcfuel(jpco2     ) )
          ier = irdparam_r(io, 'FUEL_H2S    ',  gcfuel(jph2s     ) )
          ier = irdparam_r(io, 'FUEL_C1     ',  gcfuel(jpch4     ) )
          ier = irdparam_r(io, 'FUEL_C2     ',  gcfuel(jpc2h6    ) )
          ier = irdparam_r(io, 'FUEL_C3     ',  gcfuel(jpc3h8    ) )
          ier = irdparam_r(io, 'FUEL_IC4    ',  gcfuel(jpic4h10  ) )
          ier = irdparam_r(io, 'FUEL_NC4    ',  gcfuel(jpnc4h10  ) )
          ier = irdparam_r(io, 'FUEL_IC5    ',  gcfuel(jpic5h12  ) )
          ier = irdparam_r(io, 'FUEL_NC5    ',  gcfuel(jpnc5h12  ) )
          ier = irdparam_r(io, 'FUEL_NC6    ',  gcfuel(jpnc6h14  ) )
          ier = irdparam_r(io, 'FUEL_C7     ',  gcfuel(jpnc7h16x ) )
          ier = irdparam_r(io, 'FUEL_CO     ',  gcfuel(jpco      ) )
          ier = irdparam_r(io, 'FUEL_NH3    ',  gcfuel(jpnh3     ) )
          ier = irdparam_r(io, 'FUEL_AR     ',  gcfuel(jpar      ) )
      end if

      if(mliftgas.eq. 0) then
         ier = initialize(gclift,0.0D0)
      else
          ier = irdparam_r(io, 'LIFT_H2O    ',  gclift(jph2o     ) )
          ier = irdparam_r(io, 'LIFT_H2     ',  gclift(jph2      ) )
          ier = irdparam_r(io, 'LIFT_HE     ',  gclift(jphe      ) )
          ier = irdparam_r(io, 'LIFT_N2     ',  gclift(jpn2      ) )
          ier = irdparam_r(io, 'LIFT_CO2    ',  gclift(jpco2     ) )
          ier = irdparam_r(io, 'LIFT_H2S    ',  gclift(jph2s     ) )
          ier = irdparam_r(io, 'LIFT_C1     ',  gclift(jpch4     ) )
          ier = irdparam_r(io, 'LIFT_C2     ',  gclift(jpc2h6    ) )
          ier = irdparam_r(io, 'LIFT_C3     ',  gclift(jpc3h8    ) )
          ier = irdparam_r(io, 'LIFT_IC4    ',  gclift(jpic4h10  ) )
          ier = irdparam_r(io, 'LIFT_NC4    ',  gclift(jpnc4h10  ) )
          ier = irdparam_r(io, 'LIFT_IC5    ',  gclift(jpic5h12  ) )
          ier = irdparam_r(io, 'LIFT_NC5    ',  gclift(jpnc5h12  ) )
          ier = irdparam_r(io, 'LIFT_NC6    ',  gclift(jpnc6h14  ) )
          ier = irdparam_r(io, 'LIFT_C7     ',  gclift(jpnc7h16x ) )
          ier = irdparam_r(io, 'LIFT_CO     ',  gclift(jpco      ) )
          ier = irdparam_r(io, 'LIFT_NH3    ',  gclift(jpnh3     ) )
          ier = irdparam_r(io, 'LIFT_AR     ',  gclift(jpar      ) )
      end if         
      
c --- group MET
      if(MMET .eq. 3 ) then
          ier = irdparam_r(io, 'WINDSP',  rWINDSP )
          ier = irdparam_r(io, 'WINDDR',  rWINDDR)
          ier = irdparam_r(io, 'TAMB',    rTAMB )  ! user entry is in C
          ier = irdparam_i(io, 'PGCLASS', irPGCLASS )
          ier = irdparam_r(io, 'ZREF', rZREF )
      else
          rWINDSP    = 0.
          rWINDDR    = 0.
          rTAMB      = 0.
          irPGCLASS  = 0
          rzref      = 0.
      end if

      ierr = 0
 9000 close(io)
      return
      end

     
c----------------------------------------------------------------------
c ichkinput
c----------------------------------------------------------------------
c --- Developed by B.Zelt - www.zeltpsi.com
c
c     Check the input values read for range of values or valid entries
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     io  :intger :  list file io number
c
c --- OUTPUTS
c     ierr: integer   : 0 no errors
c                     : 1 terminal errors were found
c
c----------------------------------------------------------------------
      function ichkinput(io )
     &result(ierr)
c   Common blocks
      include 'h_stddefs.for'
      include 'h_read1.for'
c    INPUTS
      integer     :: ierr
      integer     :: io
c    Declarations
c      include 'h_interface.for'

      ierr =0
      
      if( MINPUT .lt. 1 .or. MINPUT .gt. 3) then
          write(io,*) 'FATAL ERROR: Unknown setting for MINPUT=',MINPUT
          ierr = ierr + 1
      end if 
      if( MBLOWDOWN .lt. 1 .or. MBLOWDOWN .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MBLOWDOWN=',
     &          MBLOWDOWN
          ierr = ierr + 1
      end if 
      if( MDIST .lt. 1 .or. MDIST .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MDIST=',MDIST
          ierr = ierr + 1
      end if 
      if( MSTRIP .lt. 0 .or. MSTRIP .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MSTRIP=',MSTRIP
          ierr = ierr + 1
      end if 
      if( MMET .lt. 1 .or. MMET .gt. 3) then
          write(io,*) 'FATAL ERROR: Unknown setting for MMET=',MMET
          ierr = ierr + 1
      end if 
      if( MFUELGAS .lt. 0 .or. MFUELGAS .gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFUELGAS=',
     &          MFUELGAS
          ierr = ierr + 1
      end if 
      if( MLIFTGAS .lt. 0 .or. MLIFTGAS.gt. 1) then
          write(io,*) 'FATAL ERROR: Unknown setting for MLIFTGAS=',
     &          MLIFTGAS
          ierr = ierr + 1
      end if 
      if( MFGR .lt. 1 .or. MFGR .gt. 2) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFGR=',MFGR
          ierr = ierr + 1
      end if 
      if( MFASSIST .lt. 0 .or. MFASSIST .gt. 3) then
          write(io,*) 'FATAL ERROR: Unknown setting for MFASSIST=',
     &          MFASSIST
          ierr = ierr + 1
      end if 
      
      if( int(nsep/24)*24 .ne. nsep ) then
          write(io,*) 'FATAL ERROR: NSEP is not a multiple of 24'
          ierr = ierr + 1
      end if 
      if( nsep/24 .gt. 5) then
          write(io,*) 'FATAL ERROR: NSEP is greater than 5*24'
          ierr = ierr + 1
      end if 
      
c      if( nspec .gt. 2) then
c          write(io,*) 'FATAL ERROR: nspec is greater than 1'
c          ierr = ierr + 1
c      end if 

      if( leq(units,'m') .eq.  0 .and. 
     &     leq( units,'M') .eq. 0 .and. 
     &     leq(units,'km') .eq. 0 .and. 
     &     leq(units,'KM') .eq. 0) then
          write(io,*) 'FATAL ERROR: bad units (m or km):', units
          ierr = ierr + 1
      end if 

c --------------------     
      if( MINPUT.eq.1 .and. MBLOWDOWN .eq. 2) then
      if( pressinit .lt. 1. .or. pressinit .gt. 25000. ) then
          write(io,*) 'WARNING: initial pressure may be out of bounds'
      end if 
      
      if( pressinit .le. pressfinal ) then
          write(io,*) 'FATAL ERROR: bad final presssure:', pressfinal
          ierr = ierr + 1
      end if 

      if( tginit .gt. 0. ) then
         if( tginit .lt. rk0-50. .or. tginit .gt. rk0+50.) then
             write(io,*) 'WARNING: tginit may be out of bounds'
         end if 
      else
          write(io,*) 'WARNING: Using tginit = tamb'
      end if 
      
      if( tempinit .gt. 0. ) then
         if( tempinit .lt. rk0-50. .or. tempinit .gt. rk0+50.) then
             write(io,*) 'WARNING: tempinit may be out of bounds'
         end if 
      else
          write(io,*) 'WARNING: Using tempinit = tamb'
      end if 
      
      if( vesseldia .lt. 0.01 .or. vesseldia .gt. 10. ) then
          write(io,*) 'WARNING: Vessel Diameter may be out of bounds', 
     &          vesseldia
      end if 
      
      if(vessellen .lt. 1. .or. vessellen.gt. 1000000. ) then
          write(io,*) 'WARNING: Vessel length may be out of bounds',
     &           vessellen
      end if 
      
      if(orificedia*0.001.lt. 0.0001 .or. orificedia*0.001 .gt. 10. ) then
          write(io,*) 'WARNING: orifice diameter may be out of bounds',
     &            orificedia          
      end if 

      if(dischargecoef.lt.0.001 .or. dischargecoef.gt. 1 ) then
          write(io,*) 'WARNING: ',
     &          'Discharge coefficient may be out of bounds',
     &          dischargecoef
      end if 
      end if 

c --------------------     
      if( MINPUT.eq.1 .and. MBLOWDOWN .eq. 1) then
c      if( ) then
c          write(io,*) 'FATAL ERROR: '
c          write(io,*) 'WARNING: '
c          ierr = ierr + 1
c     end if 
      end if     

c check tau
c check pufdur
c check npuffs
      
      return 
      end
     
c----------------------------------------------------------------------
c iwrcontrol
c----------------------------------------------------------------------
c
c     writes the input parameters and variables as they were parsed 
c
c Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.002    Level: 121112
c     - added lift gas
c     - added aermod capacity
c
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     io  : integer   : list file io number
c
c --- OUTPUTS
c     ierr: integer:  0: no errors
c                     1: errors occured
c----------------------------------------------------------------------
      function iwrcontrol(io )
     &result(ierr)

c   Common blocks
      include 'h_stddefs.for'
      include 'h_read1.for'
c    INPUTS
      integer     :: io
c      character*128 :: FIN

c    INPUTs Declarations
c      include 'h_interface.for'

      ierr = 1   
      ier = 0

c      write(io,*) ' Model:',mmodel,' Version:',mver,' Level:',mlevel
      n=LEN_REAL(stitle)
      write(io,'(A,A,A)') '! STITLE1 =', stitle(1:n),'!'
      n=LEN_REAL(sname)
      write(io,'(A,A,A)') '! SNAME   =', sname(1:n),'!'

      n=LEN_REAL(OUTNAME)
      write(io,'(A,A,A)') '! OUTNAME=', OUTNAME(1:n), '!'
      n=LEN_REAL(OUTPATH)
      write(io,'(A,A,A)') '! OUTPATH=', OUTPATH(1:n), '!'
      n=LEN_REAL(METDAT)
      write(io,'(A,A,A)') '! METDAT=', METDAT(1:n), '!'
      n=LEN_REAL(METDAT2)
      write(io,'(A,A,A)') '! METDAT2=', METDAT2(1:n), '!'
      n=LEN_REAL(LSTDAT)
      write(io,'(A,A,A)') '! LSTDAT=', LSTDAT(1:n), '!'
      n=LEN_REAL(LOGDAT)
      write(io,'(A,A,A)') '! LOGDAT=', LOGDAT(1:n), '!'
      if( lcfiles ) then
          write(io,'(A,A,A)') '! LCFILES= T !'
      else          
          write(io,'(A,A,A)') '! LCFILES= F !'
      end if   

      write(io,'(A,I,A)') '! MDSPMOD   =', MDSPMOD,'!'
      write(io,'(A,I,A)') '! MINPUT    =', MINPUT,'!'
      write(io,'(A,I,A)') '! MBLOWDOWN =', MBLOWDOWN,'!'
      write(io,'(A,I,A)') '! MDIST     =',   MDIST,'!'
      write(io,'(A,I,A)') '! MSTRIP    =', MSTRIP,'!'
      write(io,'(A,I,A)') '! MMET      =', MMET,'!'
      write(io,'(A,I,A)') '! MFUELGAS  =', MFUELGAS ,'!'
      write(io,'(A,I,A)') '! MFGR      =', MFGR,'!'
      write(io,'(A,I,A)') '! MLIFTGAS  =', MLIFTGAS ,'!'
      write(io,'(A,I,A)') '! MFASSIST  =', MFASSIST ,'!'
      write(io,'(A,I,A)') '! MDEBUG    =', MDEBUG ,'!'
      write(io,'(A,I,A)') '! NSEP      =', NSEP ,'!'
      write(io,'(A,I,A)') '! NSOURCES  =', NSOURCES ,'!'

      write(io,'(A,I,A)') '! IUTMZN  =', IUTMZN,'!'
      n=LEN_REAL(UTMHEM)
      write(io,'(A,A,A)') '! UTMHEM  =', UTMHEM(1:n),'!'
      n=LEN_REAL(PMAP)
      write(io,'(A,A,A)') '! PMAP    =', PMAP(1:n),'!'
      n=LEN_REAL(DATUM)
      write(io,'(A,A,A)') '! DATUM   =', DATUM(1:n),'!'
      n=LEN_REAL(UNITS)
      write(io,'(A,A,A)') '! UNITS   =', UNITS(1:n),'!'
      n=LEN_REAL(ABTZ)
      write(io,'(A,A,A)') '! ABTZ    =', ABTZ(1:n),'!'

      write(io,'(A,I,A)') '! NSPEC   =', nspec,'!'
      write(io,'(A,10(A,A))') '! CHEMLST =', 
     &((chemlst(i)(1:3),','),i=1,nspec-1), chemlst(nspec)(1:3),'!'
      write(io,'(A,10(F,A))') '! MW =', 
     &((mmlst(i),','),i=1,nspec-1), mmlst(nspec),'!'
      write(io,'(A,10(F,A))') '! EMIT =', 
     &((qlst(i),','),i=1,nspec-1), qlst(nspec),'!'

      write(io,'(A,I,A)') '! YYS     =',     iyys ,'!'
      write(io,'(A,I,A)') '! MMS     =',     imms ,'!'
      write(io,'(A,I,A)') '! DDS     =',     idds  ,'!'
      write(io,'(A,I,A)') '! HHS     =',     ihhs  ,'!'
      write(io,'(A,I,A)') '! SSS     =',     isss  ,'!'
      write(io,'(A,I,A)') '! YYE     =',     iyye  ,'!'
      write(io,'(A,I,A)') '! MME     =',     imme  ,'!'
      write(io,'(A,I,A)') '! DDE     =',     idde  ,'!'
      write(io,'(A,I,A)') '! HHE     =',     ihhe  ,'!'
      write(io,'(A,I,A)') '! SSE     =',     isse  ,'!'
      write(io,'(A,I,A)') '! TIMEINC =',  itiminc  ,'!'
      write(io,'(A,F,A)') '! XCOORD  =',     xcoord   ,'!'
      write(io,'(A,F,A)') '! YCOORD  =',     ycoord   ,'!'
      write(io,'(A,F,A)') '! ZCOORD  =',     zcoord   ,'!'
      write(io,'(A,F,A)') '! HS      =',     hs   ,'!'
      write(io,'(A,F,A)') '! DS      =',     ds  ,'!'
      write(io,'(A,F,A)') '! RH2S    =',     rh2s ,'!'
      write(io,'(A,F,A)') '! TGINIT  =',     tginit ,'!'

      
      if(MINPUT .eq. 1 ) then
          if( mblowdown .eq. 1) then
              write(io,'(A,F,A)') '! QMAX        =',     qmax ,'!'
              write(io,'(A,F,A)') '! QTOTAL      =',   qtotal ,'!'
              write(io,'(A,F,A)') '! FREL        =',     frel ,'!'
              write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
              write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
              write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
              write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
              write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
              write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
              write(io,'(A,F,A)')'* DISCHARGECOEF =',dischargecoef  ,'*'
              write(io,'(A,I,A)') '! NPUFFS      =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR     =',     ipuffdur,'!' ! one of 
              write(io,'(A,F,A)') '! DURATION    =',     duration ,'!'
          else if( mblowdown .eq. 2 ) then         
              write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
              write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
              write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
              write(io,'(A,F,A)') '! PRESSINIT     =',   pressinit  ,'!'
              write(io,'(A,F,A)') '! TEMPINIT      =',    tempinit  ,'!'
              write(io,'(A,F,A)') '! PRESSFINAL    =',  pressfinal  ,'!'
              write(io,'(A,F,A)') '! VESSELDIA     =',   vesseldia  ,'!'
              write(io,'(A,F,A)') '! VESSELLEN     =',   vessellen  ,'!'
              write(io,'(A,F,A)') '! ORIFICEDIA    =',  orificedia  ,'!'
              write(io,'(A,F,A)')'! DISCHARGECOEF =',dischargecoef  ,'!'
              write(io,'(A,I,A)') '! NPUFFS      =',     npuffs  ,'!'  
              write(io,'(A,I,A)') '! PUFFDUR     =',     ipuffdur,'!' ! one of 
              write(io,'(A,F,A)') '! DURATION    =',     duration ,'!'
          end if 
          write(io,'(A,F,A)') '* PSHS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSDS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSTS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSUS      =',     0.  ,'*'
          write(io,'(A,I,A)') '* NSEQ     =',     NSEQ,'*'
      else if( MINPUT .eq. 2 ) then
          write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
          write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
          write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
          write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
          write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
          write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
          write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
          write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
          write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
          write(io,'(A,F,A)')'* DISCHARGECOEF =',dischargecoef  ,'*'
          write(io,'(A,I,A)') '* NPUFFS      =',     npuffs  ,'*'
          write(io,'(A,I,A)') '* PUFFDUR     =',     ipuffdur,'*'
          write(io,'(A,F,A)') '* DURATION    =',     duration ,'*'
          write(io,'(A,F,A)') '* PSHS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSDS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSTS      =',     0.  ,'*'
          write(io,'(A,F,A)') '* PSUS      =',     0.  ,'*'
          write(io,'(A,I,A)') '! NSEQ     =',     NSEQ,'!'
          do i=1,NSEQ
              write(io,'(I,A,F,F,F,F,A)') i,'!=',
     &              pseq(1,i),pseq(2,i),pseq(3,i),pseq(4,i),'!'
          end do
      else if( MINPUT .eq. 3 ) then
          write(io,'(A,F,A)') '* QMAX        =',     qmax ,'*'
          write(io,'(A,F,A)') '* QTOTAL      =',   qtotal ,'*'
          write(io,'(A,F,A)') '* FREL        =',     frel ,'*'
          write(io,'(A,F,A)') '* PRESSINIT     =',   pressinit  ,'*'
          write(io,'(A,F,A)') '* TEMPINIT      =',    tempinit  ,'*'
          write(io,'(A,F,A)') '* PRESSFINAL    =',  pressfinal  ,'*'
          write(io,'(A,F,A)') '* VESSELDIA     =',   vesseldia  ,'*'
          write(io,'(A,F,A)') '* VESSELLEN     =',   vessellen  ,'*'
          write(io,'(A,F,A)') '* ORIFICEDIA    =',  orificedia  ,'*'
          write(io,'(A,F,A)')'* DISCHARGECOEF =',dischargecoef  ,'*'
          write(io,'(A,I,A)') '* NPUFFS      =',     npuffs  ,'*'
          write(io,'(A,I,A)') '* PUFFDUR     =',     ipuffdur,'*'
          write(io,'(A,F,A)') '* DURATION    =',     duration,'*'
          write(io,'(A,F,A)') '! PSHS      =',     pshs  ,'!'
          write(io,'(A,F,A)') '! PSDS      =',     psds  ,'!'
          write(io,'(A,F,A)') '! PSTS      =',     psts  ,'!'
          write(io,'(A,F,A)') '! PSUS      =',     psus  ,'!'
          write(io,'(A,I,A)') '* NSEQ     =',     NSEQ,'*'
      end if 

      if(mfuelgas .eq. 1 ) then
          write(io,'(A,F,A)') '! FGR         =',     fgr     ,'!'
      else
          write(io,'(A,F,A)') '* FGR         =',     fgr     ,'*'
      end if
      if(mliftgas .eq. 1 ) then
          write(io,'(A,F,A)') '! QLIFTGAS    =',     qliftgas  ,'!'
      else
          write(io,'(A,F,A)') '* QLIFTGAS    =',     qliftgas     ,'*'
      end if


      if(MFASSIST .eq. 0) then
          write(io,'(A,I,A)') '* STM_NPORTS  =',  stm_nports,'*'
          write(io,'(A,F,A)') '* STM_DIA     =',  stm_dia,'*'
          write(io,'(A,I,A)') '* STM_IMASSVOL=',  stm_imassvol,'*'
          write(io,'(A,F,A)') '* STM_PRESS   =',  stm_press,'*'
          write(io,'(A,F,A)') '* STM_TEMP    =',  stm_temp,'*'
          write(io,'(A,F,A)') '* STM_FLOW    =',  stm_flow,'*'
          write(io,'(A,I,A)') '* AIR_NPORTS  =',  air_nports,'*'
          write(io,'(A,F,A)') '* AIR_DIA     =',  air_dia,'*'
          write(io,'(A,F,A)') '* AIR_FLOW    =',  air_flow,'*'
      else if(MFASSIST .eq. 1) then
          write(io,'(A,I,A)') '! STM_NPORTS  =',  stm_nports,'!'
          write(io,'(A,F,A)') '! STM_DIA     =',  stm_dia,'!'
          write(io,'(A,I,A)') '! STM_IMASSVOL=',  stm_imassvol,'!'
          write(io,'(A,F,A)') '! STM_PRESS   =',  stm_press,'!'
          write(io,'(A,F,A)') '! STM_TEMP    =',  stm_temp,'!'
          write(io,'(A,F,A)') '! STM_FLOW    =',  stm_flow,'!'
          write(io,'(A,I,A)') '* AIR_NPORTS  =',  air_nports,'*'
          write(io,'(A,F,A)') '* AIR_DIA     =',  air_dia,'*'
          write(io,'(A,F,A)') '* AIR_FLOW    =',  air_flow,'*'
      else if(MFASSIST .eq. 2) then
          write(io,'(A,I,A)') '* STM_NPORTS  =',  stm_nports,'*'
          write(io,'(A,F,A)') '* STM_DIA     =',  stm_dia,'*'
          write(io,'(A,I,A)') '* STM_IMASSVOL=',  stm_imassvol,'*'
          write(io,'(A,F,A)') '* STM_PRESS   =',  stm_press,'*'
          write(io,'(A,F,A)') '* STM_TEMP    =',  stm_temp,'*'
          write(io,'(A,F,A)') '* STM_FLOW    =',  stm_flow,'*'
          write(io,'(A,I,A)') '! AIR_NPORTS  =',  air_nports,'!'
          write(io,'(A,F,A)') '! AIR_DIA     =',  air_dia,'!'
          write(io,'(A,F,A)') '! AIR_FLOW    =',  air_flow,'!'
      else if(MFASSIST .eq. 3) then
          write(io,'(A,I,A)') '! STM_NPORTS  =',  stm_nports,'!'
          write(io,'(A,F,A)') '! STM_DIA     =',  stm_dia,'!'
          write(io,'(A,I,A)') '! STM_IMASSVOL=',  stm_imassvol,'!'
          write(io,'(A,F,A)') '! STM_PRESS   =',  stm_press,'!'
          write(io,'(A,F,A)') '! STM_TEMP    =',  stm_temp,'!'
          write(io,'(A,F,A)') '! STM_FLOW    =',  stm_flow,'!'
          write(io,'(A,I,A)') '! AIR_NPORTS  =',  air_nports,'!'
          write(io,'(A,F,A)') '! AIR_DIA     =',  air_dia,'!'
          write(io,'(A,F,A)') '! AIR_FLOW    =',  air_flow,'!'
      end if           

      write(io,'(A,F6.4,A)') '! RAW_H2O     =',  gcraw(jph2o     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_H2      =',  gcraw(jph2      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_HE      =',  gcraw(jphe      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_N2      =',  gcraw(jpn2      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_CO2     =',  gcraw(jpco2     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_H2S     =',  gcraw(jph2s     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C1      =',  gcraw(jpch4     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C2      =',  gcraw(jpc2h6    ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C3      =',  gcraw(jpc3h8    ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_IC4     =',  gcraw(jpic4h10  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC4     =',  gcraw(jpnc4h10  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_IC5     =',  gcraw(jpic5h12  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC5     =',  gcraw(jpnc5h12  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NC6     =',  gcraw(jpnc6h14  ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_C7      =',  gcraw(jpnc7h16x ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_CO      =',  gcraw(jpco      ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_NH3     =',  gcraw(jpnh3     ) ,'!'
      write(io,'(A,F6.4,A)') '! RAW_AR      =',  gcraw(jpar      ) ,'!'

      if(mfuelgas .eq. 0 ) then
          write(io,'(A,F6.4,A)') '* FUEL_H2O    =', gcfuel(jph2o     ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_H2     =', gcfuel(jph2      ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_HE     =', gcfuel(jphe      ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_N2     =', gcfuel(jpn2      ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_CO2    =', gcfuel(jpco2     ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_H2S    =', gcfuel(jph2s     ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_C1     =', gcfuel(jpch4     ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_C2     =', gcfuel(jpc2h6    ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_C3     =', gcfuel(jpc3h8    ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_IC4    =', gcfuel(jpic4h10  ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_NC4    =', gcfuel(jpnc4h10  ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_IC5    =', gcfuel(jpic5h12  ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_NC5    =', gcfuel(jpnc5h12  ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_NC6    =', gcfuel(jpnc6h14  ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_C7     =', gcfuel(jpnc7h16x ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_CO     =', gcfuel(jpco      ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_NH3    =', gcfuel(jpnh3     ) ,'*'
          write(io,'(A,F6.4,A)') '* FUEL_AR     =', gcfuel(jpar      ) ,'*'
      else
          write(io,'(A,F6.4,A)') '! FUEL_H2O    =', gcfuel(jph2o     ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_H2     =', gcfuel(jph2      ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_HE     =', gcfuel(jphe      ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_N2     =', gcfuel(jpn2      ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_CO2    =', gcfuel(jpco2     ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_H2S    =', gcfuel(jph2s     ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_C1     =', gcfuel(jpch4     ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_C2     =', gcfuel(jpc2h6    ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_C3     =', gcfuel(jpc3h8    ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_IC4    =', gcfuel(jpic4h10  ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_NC4    =', gcfuel(jpnc4h10  ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_IC5    =', gcfuel(jpic5h12  ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_NC5    =', gcfuel(jpnc5h12  ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_NC6    =', gcfuel(jpnc6h14  ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_C7     =', gcfuel(jpnc7h16x ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_CO     =', gcfuel(jpco      ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_NH3    =', gcfuel(jpnh3     ) ,'!'
          write(io,'(A,F6.4,A)') '! FUEL_AR     =', gcfuel(jpar      ) ,'!'
      end if 
      if(mliftgas .eq. 0 ) then
          write(io,'(A,F6.4,A)') '* LIFT_H2O    =', gclift(jph2o     ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_H2     =', gclift(jph2      ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_HE     =', gclift(jphe      ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_N2     =', gclift(jpn2      ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_CO2    =', gclift(jpco2     ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_H2S    =', gclift(jph2s     ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_C1     =', gclift(jpch4     ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_C2     =', gclift(jpc2h6    ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_C3     =', gclift(jpc3h8    ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_IC4    =', gclift(jpic4h10  ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_NC4    =', gclift(jpnc4h10  ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_IC5    =', gclift(jpic5h12  ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_NC5    =', gclift(jpnc5h12  ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_NC6    =', gclift(jpnc6h14  ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_C7     =', gclift(jpnc7h16x ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_CO     =', gclift(jpco      ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_NH3    =', gclift(jpnh3     ) ,'*'
          write(io,'(A,F6.4,A)') '* LIFT_AR     =', gclift(jpar      ) ,'*'
      else
          write(io,'(A,F6.4,A)') '! LIFT_H2O    =', gclift(jph2o     ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_H2     =', gclift(jph2      ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_HE     =', gclift(jphe      ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_N2     =', gclift(jpn2      ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_CO2    =', gclift(jpco2     ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_H2S    =', gclift(jph2s     ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_C1     =', gclift(jpch4     ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_C2     =', gclift(jpc2h6    ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_C3     =', gclift(jpc3h8    ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_IC4    =', gclift(jpic4h10  ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_NC4    =', gclift(jpnc4h10  ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_IC5    =', gclift(jpic5h12  ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_NC5    =', gclift(jpnc5h12  ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_NC6    =', gclift(jpnc6h14  ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_C7     =', gclift(jpnc7h16x ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_CO     =', gclift(jpco      ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_NH3    =', gclift(jpnh3     ) ,'!'
          write(io,'(A,F6.4,A)') '! LIFT_AR     =', gclift(jpar      ) ,'!'
      end if 

      
      if(MMET .eq. 3) then
          write(io,'(A,F,A)') '! WINDSP  =', rWINDSP   ,'!'
          write(io,'(A,F,A)') '! WINDDR  =', rWINDDR   ,'!'
          write(io,'(A,F,A)') '! TAMB    =', rTAMB     ,'!'
          write(io,'(A,I,A)') '! PGCLASS =', irPGCLASS ,'!'
          write(io,'(A,F,A)') '! ZREF    =', rZREF ,'!'
      else
          write(io,'(A,F,A)') '* WINDSP  =', 0.   ,'*'
          write(io,'(A,F,A)') '* WINDDR  =', 0.   ,'*'
          write(io,'(A,F,A)') '* TAMB    =', 0.   ,'*'
          write(io,'(A,I,A)') '* PGCLASS =', 0    ,'*'
          write(io,'(A,F,A)') '* ZREF    =', 0.    ,'*'
      end if 
      write(io,*) ' ---------------------------------' 
      write(io,*) 'end of input file'
      write(io,*) 
      write(io,*) 

      ierr = 0
 ! close(io)
 9000 return
      end


c----------------------------------------------------------------------
c Vessel initialize
c----------------------------------------------------------------------
c
c     Calculate the flow rate output of a pressurized vessel through
c     a specified orifice.
c
c
c Develped by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     mix           :real(mxcomp) :normalized raw gas combusted
c     pressinit     :real   :Expected Maximum Initial Pressure	kPa (gauge)
c     tempinit      :real   :Expected Minimum Initial Gas Temperature	K
c     pressfinal    :real   :Expected Minimum Final Pressure	kPa
c     vesseldia     :real   :Pipeline/Vessel Inside Diameter	m
c     vessellen     :real   :Pipeline/Vessel Length	m
c     orificedia    :real   :Minimum Orifice Diameter	mm
c     dischargecoef :real   :Discharge Coefficient
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c     frel          :real   :fraction of volume released
c     tgasout       :real   :exit temperature of gas, K
c
c --- OUTPUTS
c     ierr: integer:  0: no errors
c                     1: errors were found
c----------------------------------------------------------------------
      function ivessel(mix,pressinit,tempinit,pressfinal,vesseldia,
     &                 vessellen,orificedia,dischargecoef,
     &                 qmax,qtotal,frel,tgasout )
     &result(ierr)
c includes
      include 'h_stddefs.for'
c inputs      
      real(8):: mix(*)    ! normalized raw gas combusted
      real(8), intent (in) :: pressinit		! Expected Maximum Initial Pressure	kPa (gauge)
      real(8), intent (in) :: tempinit		! Expected Minimum Initial Gas Temperature	K 
      real(8), intent (in) :: pressfinal		! Expected Minimum Final Pressure	kPa
      real(8), intent (in) :: vesseldia		! Pipeline/Vessel Inside Diameter	m
      real(8), intent (in) :: vessellen		! Pipeline/Vessel Length	m
      real(8), intent (in) :: orificedia		! Minimum Orifice Diameter	mm
      real(8), intent (in) :: dischargecoef	! Discharge Coefficient	
      real(8), intent (out):: qmax           ! initial volume flow rate, e3m3/d (ideal gas)
      real(8), intent (out):: qtotal         ! total volume released, e3m3 (ideal gas)
      real(8), intent (out):: frel           ! fraction of volume released
      real(8), intent (out):: tgasout        ! exit temperature of gas, K
c declarations
      real(8) :: ap  ! area of vessel
      real(8) :: vp  ! volume of vessel
      real(8) :: ae  ! exit area
      real(8) :: rmm ! molar mass
      real(8) :: rs  ! gas constant 
      real(8) :: rhoref ! density at reference conditions
      real(8) :: cpi ! specific heat 
      real(8) :: pinit ! initial pressure
      real(8) :: tinit ! initial temperatue
      real(8) :: avek, ratiok1, ratiok2, ratiok3
      real(8) :: zo  ! compressibility, initial
      real(8) :: co  ! speed of sound, initial
      real(8) :: ze  ! compressibility, exit
      real(8) :: ce  ! speed of sound, exit
      real(8) :: rhoinit ! initial density
      real(8) :: rhoe    ! exit density
      real(8) :: teig_choked
      real(8) :: peig_choked
      real(8) :: teig
      real(8) :: peig
c      real(8) :: pratio 
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

c initializations
      ap      = 0.25*pi*vesseldia*vesseldia
      vp      = ap*vessellen
      ae      = 0.25*pi*orificedia*orificedia*.001*.001 ! m2
      rmm     = calc_molarmass(mix)       ! kg/kmole
      rs      = ru*1000./rmm              ! j/(kg.K)
      rhoref  = 1000.*pref/(tref*rs)      ! kg/m3
      cpi     = calc_byprop(mix, ipcpmole, ipcalc)/rmm*1000. ! j/(kg.K)
      pinit   = plocal + pressinit    ! kPa 
      tinit   = tempinit        ! K

c Thermodynamics      
      avek    = cpi/(cpi-rs)  
      ratiok1 = 2./(avek+1.)
      ratiok2 = (avek+1.)/(avek-1.)
      ratiok3 = avek/(avek-1.)
      zo      = 1.0
      co      = SQRT(avek*zo*rs*tinit)        ! m/s
      rhoinit = pinit*1000./(zo*rs*tinit)     ! kg/m3
      teig_choked    = tinit*ratiok1          ! K
      peig_choked    = pinit*ratiok1**ratiok3 ! kPa
      ze      = 1.0
      if( peig_choked .lt. plocal) then
          peig    = plocal
          teig    = tinit
      else          
          peig    = peig_choked
          teig    = teig_choked/ze*zo
      end if 
      rhoe    = peig*1000./(rs*ze*teig)   ! kg/m3
      ce      = SQRT(avek*ze*rs*teig)     ! m/s

c exponential blowdown model
      rqtotal = rhoinit*vp               ! total mass released, kg       
      qtotal  = rqtotal/(rhoref*1000.)   ! total volume released, e3m3
      rqemit  = dischargecoef*(ze/zo*ratiok1)**(0.5*ratiok2)
     &         *SQRT(avek/(zo*rs*tinit))*pinit*ae*1000.   ! ideal gas initial mass release rate, kg/s     
      qmax    = rqemit/rhoref*3600.*24./1000.   ! max volume flow released, e3m3/d
      frel    = 1.-(plocal+pressfinal)/(plocal+pinit)   ! fraction released
c      tgasout = teig - rk0    ! degrees C
      tgasout = teig    ! degrees K
      
      ierr = 0
      return
      end
      
      
c----------------------------------------------------------------------
c iblowdown
c----------------------------------------------------------------------
c
c     Calculates a sequence of flowrates based upon an exponential
c     blowdown curve
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c
c --- INPUTS
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c     frel          :real   :fraction of volume released
c     mblowdown   : integer:  
c                     1: user specified qmax
c                     2: vessel blowdown 
c     mdist   : integer   :puff distribution selection
c                     1: constant duration
c                     2: constant mass
c
c --- OUTPUTS
c     ipuffdur  : integer : puff duration, s
c     npuffs    : integer : number of puffs used to represent blowdown
c     pseq(4,mxpuff) : real :
c         pseq(1,*) : flowrate e3m3/d
c         pseq(2,*) : duration, s
c         pseq(3,*) : rH2S
c         pseq(4,*) : 
c
c     ierr:   integer: 0 : no errors
c                      1 : errors were found
c----------------------------------------------------------------------
      function iblowdown(qmax, qtotal, frel, ipuffdur, pseq, npuffs,
     &    mblowdown,mdist)
     &result(ierr)
c includes
      include 'h_stddefs.for'
c INPUTS
      real(8), intent (in)  :: qmax     ! intital, maximum flow rate, e3m3/d
      real(8), intent (in)  :: qtotal   ! total volume contained, e3m3
      real(8), intent (in)   :: frel     ! fraction of volum/mass released
      integer, intent (in out)   :: ipuffdur   ! puff duration, s
      real(8) :: pseq(4,*) ! puff average mass puffs in blowdown
      integer, intent (in out) :: npuffs  ! number of puffs used to represent blowdown
      integer, intent (in) :: mblowdown   ! 1:user qmax; 2:vessel exponential model selection
      integer, intent (in) :: mdist       ! puff distribution selection
c DECLARATIONS
      real(8)  :: frel1
      real(8)  :: tconst, tconsti 
      real(8)  :: r1,t0,t1,m0,m1,cum
      real(8) :: duration
      integer :: i
c      include 'h_interface.for'
      
c calculate the release time constant (seconds)
      tconst = timeconstant(qmax, qtotal)
      tconsti = -1.e30
      if(tconst .gt. 0 ) then
          tconsti = -1./tconst
      end if 


c limit the actual amount released 
      frel1    = min(frel,rmxfrel)  ! set limit

c actual duration      
      duration = -tconst*log(1.-frel1)    ! seconds

      if(npuffs .gt. mxpuff) then
          write(6,*) '***ERROR*** npuffs exceeds maximum puffs'
          write(6,*) 'npuffs    :',npuffs
          write(6,*) 'max puffs :',mxpuff
          STOP 'HALTED in routine BLOWDOWN()'
      end if 

c----------------------------------------------------------------------
c constant volume blow down
      if( MDIST .eq. 1)  then
          
c for user input determine npuffs and duration     
      if(npuffs.gt.1) then
          ipuffdur = nint(duration/npuffs)
      else
          npuffs   = nint(duration/ipuffdur)
      end if 
      duration = float(npuffs)*float(ipuffdur)
      
      r1    = qtotal*(24.*60.*60.)/float(ipuffdur   )
      cum   = 0.
      t0    = 0.
      t1    = t0 + ipuffdur
      m0    = 0
      m1    = exp(tconsti*t0) - exp(tconsti*t1)
      cum   = m1 + cum

      do 100 i=1,npuffs
c --- if mass in proposed puff is greater than limit
c     then truncate
          if(cum .gt. frel1) then
              m1 = frel1 - (cum-m1)
          end if 
          pseq(1,i)  = r1*(m1)
          pseq(2,i)  = ipuffdur
          
          t0       = t1
          m0       = m1
          t1       = t0 + ipuffdur
          m1       = exp(tconsti*t0) - exp(tconsti*t1)
          cum      = m1 + cum
 100  end do
c --- ensure that puffs selection is not missing some mass
c     else add it to the last puff
c     typically this shouldn't be required since NINT(...) above adds
c     an extra puff if required.
      if (cum .lt. frel1) then
          pseq(1,npuffs) = pseq(1,npuffs) + r1*(frel1-cum)
      end if

c----------------------------------------------------------------------
      else if( MDIST .eq. 2) then
c --- constant mass puffs           
c mass fraction released on each puff step      
      r1    = frel/float(npuffs)
      t0    = 0.0
      do 200 i=1,npuffs
          t1         = -tconst*log(1.-float(i)*r1)    ! second
          pseq(2,i)  = t1  -t0                           ! seconds duration
          vol  = qmax/(3600.*24.)*
     &          (EXP(-t0/tconst)-EXP(-t1/tconst))*(tconst)*1000.
          pseq(1,i)  = vol/(t1-t0)*3600.*24./1000. ! e3m3/d
          pseq(3,i)  =  -1.   ! use default
          pseq(4,i)  =  -1.   ! use default
          t0 = t1
 200  end do

      else
          write(6,*) 'MDIST=',MDIST 
          STOP 'HALTED in routine BLOWDOWN()'
      end if 
      
      ierr   = 0
      return
      end





c----------------------------------------------------------------------
c timeconstant
c----------------------------------------------------------------------
c
c     Returns calculated time constant (seconds)
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c ---   Version: 1.001    Level: 110706
c
c --- INPUTS
c     qmax          :real   :initial volume flow rate, e3m3/d (ideal gas)
c     qtotal        :real   :total volume released, e3m3 (ideal gas)
c
c --- OUTPUTS
c
c----------------------------------------------------------------------
      function timeconstant(qmax, qtotal)
c INPUTS
      real(8), intent (in)  :: qmax     ! intital, maximum flow rate, e3m3/d
      real(8), intent (in)  :: qtotal   ! total volume contained, e3m3
      real(8) :: timeconstant  ! time constant
c DECLARATION
c      include 'h_interface.for'

      timeconstant = 0.
      if(qtotal .gt. 0. ) then
          timeconstant = qtotal/qmax  *24. * 60. * 60.
      end if 
c      timeconstant = tconst
      return
      end


c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c See PROPERTIES.xlsx
c      Note a6,a7 (or a1*), b6,b7 (or b1*)
c      are provided in the properties list from the spreadsheet, but are not 
c      used.  The a6 and a1* properties are calculated
c      within this code.
c         
c  ref: references conditions at 15 C
c  -Gas Producers Suppliers Association (GPSA), 2012, Engineering Data Book, 13th Edition, SI, Vol 1 & 2 
c  -Gordon,S.,  McBride, B.J., 1971, Computer Program for Calculation of Complex Chemical Equilibrium Composition, 
c  Rocket Performance, Incident and Reflected Shocks and Chapman-Jouguet Detonations, NASA SP-273.
c  -Burcat,A., Ruscic,B., 2005, Third Millennium Ideal Gas and Condensed Phase Thermochecmical Database for 
c  Combustion with Updates from Active Thermochemical Tables.  HTTP://www.osti.gov.bridge.
c  -Zabetakis,M.G.. 1965. Flammability Characteristics of Combustible Gases and Vapors. U.S. Dept. 
c  of the Interior, Bureau of Mines, Bulletin 627."									
c         
c         
c Updates:
c     1311029: Updated with addition of a6          
c     121029: Updated to GPSA 13th Ed
c
c----------------------------------------------------------------------
c  Initialize chemo-phsyical constants
c
c   see stddefs.h for i,j constants
c
c   i=1,mxprop
c  Properties from GPSA
c     1:  ipmmass:  kg/kmol:  "Molar Mass"
c     2:  ipnhv:  "MJ/m (15C,101.325 kPa)":  "Net Heating Value"
c     3:  ipghv:  "MJ/m (15C,101.325 kPa)":  "Gross Heating Value"
c     4:  ipcpmass:  "kJ/(kgK) (15C,101.325 kPa)":  "Mass Specific Heat Cp"
c     5:  ipcpmole:  "kJ/(kmolK) (15C,101.325 kPa)":  "Molar Specific Heat Cp"
c  Combustion Properties
c     6:  ipstoico2:  O2 required:  Stoichiometric Combustion (moles/mole fuel)
c     7:  ipstoicc:  Carbons:  
c     8:  ipstoicp:  Products:  
c     9:  ipstoich20:  H20:  
c     10:  iplfl:  %:  Lower Flammable Limit
c     11:  ipcomb:  :  Combustible Fraction
c     12:  ipa1:  a1:  Table D.1 Temperature-dependent Molar Heat Capacities of Gases at Zero Pressure (Low Temp 200-1000K)
c     13:  ipa2:  a2:  
c     14:  ipa3:  a3:  
c     15:  ipa4:  a4:  
c     16:  ipa5:  a5:  
c     17:  ipa6:  a6:  
c     18:  ipa7:  a7:  
c     19:  ipb1:  b1:  Table D.1 Temperature-dependent Molar Heat Capacities of Gases at Zero Pressure (Hi Temp 1000-6000K)
c     20:  ipb2:  b2:  
c     21:  ipb3:  b3:  
c     22:  ipb4:  b4:  
c     23:  ipb5:  b5:  
c     24:  ipb6:  b6:  
c     25:  ipb7:  b7:  
c     26:  ipcalc:  :  
c     27:  iph:  :  

c----------------------------------------------------------------------
      include 'h_stddefs.for'

       data  (prop(i,jph2o),i=1,mxprop)/
     &   18.0153,
     &   0.000,
     &   0.000,
     &   1.8620,
     &   33.5445,
     &   0.000,
     &   0.000,
     &   1,
     &   1,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   4.1986352,
     &   -0.002036402,
     &   6.52034E-06,
     &   -5.48793E-09,
     &   1.77197E-12,
     &   -1168.629395,
     &   0.000307536,
     &   2.6770389,
     &   0.002973182,
     &   -7.73769E-07,
     &   9.44335E-11,
     &   -4.269E-15,
     &   -760.7973604,
     &   0.000307611,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jph2),i=1,mxprop)/
     &   2.0159,
     &   10.223,
     &   12.102,
     &   14.2450,
     &   28.7165,
     &   0.5,
     &   0.000,
     &   1,
     &   1,
     &   4.0,
     &   1.0,
     &   400.0,
     &   2.34433112,
     &   0.007980521,
     &   -1.94782E-05,
     &   2.01572E-08,
     &   -7.37612E-12,
     &   -882.0730893,
     &   -0.004268315,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jphe),i=1,mxprop)/
     &   4.0026,
     &   0.000,
     &   0.000,
     &   5.1931,
     &   20.7859,
     &   0.000,
     &   0.000,
     &   1,
     &   0.000,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   2.499958153,
     &   0,
     &   0,
     &   0,
     &   0,
     &   -720.3629417,
     &   0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   1.000,
     &   0.0000/
       data  (prop(i,jpn2),i=1,mxprop)/
     &   28.0135,
     &   0.000,
     &   0.000,
     &   1.0395,
     &   29.1200,
     &   0.000,
     &   0.000,
     &   1,
     &   0.000,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   3.53100528,
     &   -0.000123661,
     &   -5.02999E-07,
     &   2.43531E-09,
     &   -1.40881E-12,
     &   -1011.996356,
     &   0.000155819,
     &   2.95257637,
     &   0.0013969,
     &   -4.92632E-07,
     &   7.8601E-11,
     &   -4.60755E-15,
     &   -888.9687624,
     &   0.000155813,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   1.000,
     &   0.7812/
       data  (prop(i,jpco2),i=1,mxprop)/
     &   44.0100,
     &   0.000,
     &   0.000,
     &   0.8316,
     &   36.5987,
     &   0.000,
     &   1,
     &   1,
     &   0.000,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   2.356813,
     &   0.00898413,
     &   -7.12206E-06,
     &   2.4573E-09,
     &   -1.42885E-13,
     &   -996.5183259,
     &   -0.010254533,
     &   4.6365111,
     &   0.002741457,
     &   -9.95898E-07,
     &   1.60387E-10,
     &   -9.16199E-15,
     &   -1649.451229,
     &   -0.010254597,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jph2s),i=1,mxprop)/
     &   34.0820,
     &   21.905,
     &   23.784,
     &   0.9967,
     &   33.9695,
     &   1.5,
     &   0.000,
     &   2,
     &   1,
     &   4.3,
     &   1.0,
     &   260.0,
     &   4.12024455,
     &   -0.001879074,
     &   8.21427E-06,
     &   -7.06426E-09,
     &   2.14235E-12,
     &   -1157.37178,
     &   -0.021001965,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpch4),i=1,mxprop)/
     &   16.0420,
     &   33.949,
     &   37.707,
     &   2.2062,
     &   35.3919,
     &   2,
     &   1,
     &   3,
     &   2,
     &   5.0,
     &   1.0,
     &   537.0,
     &   5.14911468,
     &   -0.013662201,
     &   4.91454E-05,
     &   -4.84247E-08,
     &   1.66603E-11,
     &   -1233.769573,
     &   0.0074427,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpc2h6),i=1,mxprop)/
     &   30.0690,
     &   60.429,
     &   66.067,
     &   1.7094,
     &   51.3999,
     &   3.5,
     &   2,
     &   5,
     &   3,
     &   2.9,
     &   1.0,
     &   515.0,
     &   4.29142572,
     &   -0.005501549,
     &   5.99438E-05,
     &   -7.08466E-08,
     &   2.68686E-11,
     &   -1377.233523,
     &   0.008435817,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpc3h8),i=1,mxprop)/
     &   44.0960,
     &   86.419,
     &   93.936,
     &   1.6292,
     &   71.8412,
     &   5,
     &   3,
     &   7,
     &   4,
     &   2.0,
     &   1.0,
     &   466.0,
     &   4.21093013,
     &   0.001708865,
     &   7.0653E-05,
     &   -9.20061E-08,
     &   3.64618E-11,
     &   -1709.65734,
     &   0.020677431,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpic4h10),i=1,mxprop)/
     &   58.1220,
     &   112.008,
     &   121.404,
     &   1.6218,
     &   94.2623,
     &   6.5,
     &   4,
     &   9,
     &   5,
     &   1.8,
     &   1.0,
     &   405.0,
     &   4.4547914,
     &   0.008260589,
     &   8.29886E-05,
     &   -1.14648E-07,
     &   4.6457E-11,
     &   -2119.122088,
     &   0.034122928,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpnc4h10),i=1,mxprop)/
     &   58.1220,
     &   112.396,
     &   121.792,
     &   1.6577,
     &   96.3488,
     &   6.5,
     &   4,
     &   9,
     &   5,
     &   1.5,
     &   1.0,
     &   405.0,
     &   6.14474013,
     &   0.0001645,
     &   9.67849E-05,
     &   -1.25486E-07,
     &   4.97846E-11,
     &   -2358.247181,
     &   0.018886369,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpic5h12),i=1,mxprop)/
     &   72.1490,
     &   138.088,
     &   149.363,
     &   1.6014,
     &   115.5394,
     &   8,
     &   5,
     &   11,
     &   6,
     &   1.3,
     &   1.0,
     &   258.0,
     &   2.19440218,
     &   0.036171717,
     &   3.01587E-05,
     &   -5.87986E-08,
     &   2.4706E-11,
     &   -2286.217278,
     &   0.011212352,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpnc5h12),i=1,mxprop)/
     &   72.1490,
     &   138.381,
     &   149.656,
     &   1.6256,
     &   117.2854,
     &   8,
     &   5,
     &   11,
     &   6,
     &   1.4,
     &   1.0,
     &   258.0,
     &   10.0497596,
     &   -0.019351883,
     &   0.000147548,
     &   -1.69026E-07,
     &   6.2897E-11,
     &   -3288.671858,
     &   0.992045541,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpnc6h14),i=1,mxprop)/
     &   86.1750,
     &   164.399,
     &   177.554,
     &   1.6351,
     &   140.9047,
     &   9.5,
     &   6,
     &   13,
     &   7,
     &   1.1,
     &   1.0,
     &   223.0,
     &   9.87121167,
     &   -0.00936699,
     &   0.000169888,
     &   -2.1502E-07,
     &   8.45407E-11,
     &   -3539.7901,
     &   0.230454867,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpnc7h16x),i=1,mxprop)/
     &   100.2020,
     &   190.388,
     &   205.422,
     &   1.6123,
     &   161.5557,
     &   11,
     &   7,
     &   15,
     &   8,
     &   1.0,
     &   1.0,
     &   223.0,
     &   11.1532994,
     &   -0.009494198,
     &   0.000195572,
     &   -2.49754E-07,
     &   9.84878E-11,
     &   -4008.519336,
     &   0.071047602,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpco),i=1,mxprop)/
     &   28.0100,
     &   11.965,
     &   11.965,
     &   1.0357,
     &   29.0100,
     &   0.5,
     &   1,
     &   1,
     &   0.000,
     &   12.5,
     &   1.0,
     &   609.0,
     &   3.5795335,
     &   -0.000610354,
     &   1.01681E-06,
     &   9.07006E-10,
     &   -9.04424E-13,
     &   -1011.246694,
     &   -0.014471607,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpnh3),i=1,mxprop)/
     &   17.0306,
     &   13.391,
     &   16.210,
     &   2.0792,
     &   35.4100,
     &   0.75,
     &   0.000,
     &   2,
     &   1.5,
     &   15.5,
     &   1.0,
     &   630.0,
     &   4.30177808,
     &   -0.004771273,
     &   2.19342E-05,
     &   -2.29856E-08,
     &   8.28992E-12,
     &   -1181.082292,
     &   0.003474835,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   1.000,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpar),i=1,mxprop)/
     &   39.9480,
     &   0.000,
     &   0.000,
     &   0.5245,
     &   20.9534,
     &   0.000,
     &   0.000,
     &   1,
     &   0.000,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   2.500,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   -726.1691923,
     &   0.02010825,
     &   2.5,
     &   0,
     &   0,
     &   0,
     &   0,
     &   -726.1691923,
     &   0.02010825,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   1.000,
     &   0.0092/
       data  (prop(i,jpo2),i=1,mxprop)/
     &   31.9988,
     &   0.000,
     &   0.000,
     &   0.9166,
     &   29.3301,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   1.0E+06,
     &   0.0,
     &   0.0,
     &   3.78245636,
     &   -0.002996734,
     &   9.8473E-06,
     &   -9.6813E-09,
     &   3.24373E-12,
     &   -1028.720115,
     &   0.000275942,
     &   3.66096065,
     &   0.000656366,
     &   -1.4115E-07,
     &   2.05798E-11,
     &   -1.29913E-15,
     &   -1180.753729,
     &   0.000275938,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   -9999.000,
     &   0.2096/
       data  (prop(i,jpso2),i=1,mxprop)/
     &   64.0650,
     &   0.000,
     &   0.000,
     &   0.6166,
     &   39.5025,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.0,
     &   0.0,
     &   3.67480752,
     &   0.002283021,
     &   8.46893E-06,
     &   -1.36562E-08,
     &   5.76272E-12,
     &   -1200.601635,
     &   0.002195055,
     &   5.38423482,
     &   0.001679306,
     &   -6.32063E-07,
     &   1.08465E-10,
     &   -6.6689E-15,
     &   -1861.796497,
     &   0.002195044,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jps),i=1,mxprop)/
     &   32.0660,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.0,
     &   0.0,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   0.0,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   -9999.000,
     &   0.0000/
       data  (prop(i,jpair),i=1,mxprop)/
     &   28.95861628,
     &   0.000,
     &   0.000,
     &   1.0045,
     &   29.0889,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.000,
     &   0.0,
     &   0.0,
     &   3.574224178,
     &   -0.000724719,
     &   1.67105E-06,
     &   -1.26738E-10,
     &   -4.20679E-13,
     &   -1012.872046,
     &   0.000364559,
     &   3.096890012,
     &   0.001228833,
     &   -4.14429E-07,
     &   6.57166E-11,
     &   -3.87172E-15,
     &   -948.6291353,
     &   0.000364554,
     &   1.000,
     &   -9999.000,
     &   1.000,
     &   -9999.000,
     &   0.0000/

      end


c----------------------------------------------------------------------
c  setup1
c----------------------------------------------------------------------
c
c     hard coding of several variables
c
c     DEVELOPED by B.Zelt  www.zeltpsi.com
c
c --- UPDATES
c
c
c --- INPUTS
c     none
c
c --- OUTPUTS
c     none
c----------------------------------------------------------------------
      subroutine setup1()
      include 'h_stddefs.for'
c standard constants
      effminERCB   = .98 ! Minimum Conversion and Combustion Efficiency ERCB
      effmin   = .50    ! Minimum Conversion and Combustion Efficiency
      radloss  = .25    ! Flame Radiation Loss (RAD fraction)
      tgas  = 278.15    ! Raw Gas and Fuel Gas Temperature before Combustion
      tamb  = 278.15   ! Average Ambient Air Temperature
      nhvrec   = 20.    ! Minimum Recommended Net Heating Value, MJ/m (15C,101.325 kPa)
      nhvmin   = 12.    ! Minimum Allowed Net Heating Value MJ/m (15C,101.325 kPa)
      tradbkg  = 1.04   ! Background Thermal Radiation, KW/m2
      tradtot  = 4.73   ! Maximum Recommended Total Thermal Radiation
      u10min   = 1.     ! Minimum Wind Speed at 10 m reference height, m/s
      ustabf   = 1.     ! Stability Class F Plume Rise Criterion Wind Speed
      tgrad    = .035   ! Stability Class F Vertical Potential Temperature Gradient, K/m
      slopemx  = .05    ! Maximum Slope Criterion
      uofaa    = 133.3  ! UofA Flare Research Project constant A, (MJ/kg)^3
      uofab    = .317   ! UofA Flare Research Project constant B
      tref     = 288.15   ! K
      pref     = 101.325  ! kPa (absolute)
      rlapse    = 0.0065
c      fn2air   = 0.791 ! now using input gas composition of air

      cpv      = ru*tref/pref

      rk1      = 1000./24./3600.*1000000./4.1868
      rk2      = 1000./24./3600./cpv*prop(ipmmass,jpso2)*1000./100.
      rk4      = 1000./24./3600.*1000000. ! ??? check this

c reset properties for molar specific heat
      do i=1,mxcomp
         prop(ipcpmole,i) = prop(ipmmass,i)*prop(ipcpmass,i)
      end do 
      
      return
      end

c----------------------------------------------------------------------
c InitLocal
c----------------------------------------------------------------------
c
c  Initialize local/site variables
c
c     DEVELOPED B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     elev    : real  : site elevation (m) above sea level
c
c --- OUTPUTS
c     plocal : real: (common block)  : site pressure
c 
c----------------------------------------------------------------------
      subroutine InitLocal(elev)
      real(8) :: elev
      include 'h_stddefs.for'

c initialize local pressure from the elevation
      plocal   = calc_plocal(elev)

      return
      end

c----------------------------------------------------------------------
c calc_totalrate()
c----------------------------------------------------------------------
c
c     Calculates the total rate
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     rate    : real  : raw gas flow rate
c     fgr     : real  : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     calc_totalrate :    real : total flow rate =(raw gas + fuel gas)
c
c----------------------------------------------------------------------
      function calc_totalrate(rate, fgr)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: rate
      real(8)  :: fgr
      real(8) :: calc_totalrate 
      calc_totalrate = rate *(1.+fgr)
      return
      end 

c----------------------------------------------------------------------
c calc_fuelrate()
c----------------------------------------------------------------------
c
c     calculates the fuel gas flow rate from the total rate
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     rate    : real :  total flare rate (=raw gas + fuel gas rate)
c     fgr     : real  : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     calc_fuelrate   : real : calculated fuel gas rate
c
c
c----------------------------------------------------------------------
      function calc_fuelrate(rate, fgr)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: rate
      real(8)  :: fgr
      real(8)   :: calc_fuelrate 
      calc_fuelrate = rate *(fgr)
      return
      end 

      
c----------------------------------------------------------------------
c initialize
c----------------------------------------------------------------------
c
c     initializes a gas composition to a value = rval
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp) : real  : gas composition
c     rval        : real  : initialization value
c
c --- OUTPUTS
c     returns   0: no errors
c               1: errors were found
c----------------------------------------------------------------------
      function initialize(mix,rval)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8)  :: rval
      integer :: initialize
c DECLARATIONS
      integer   :: j
      do j=1,mxcomp
          mix(j) =   rval
      end do
      initialize = 0
      return 
      end

c----------------------------------------------------------------------
c inormalize
c----------------------------------------------------------------------
c 
c     Creates a normlized gas composition from raw gas, fuel gas 
c     and fgr 
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     raw(mxcomp)     : real : raw gas composition
c     fuel(mxcomp)    : real : fuel gas composition
c     fgr             : real : fuel gas to raw gas ratio
c
c --- OUTPUTS
c     mix(mxcomp) : real : mixed gas composition
c
c --- RETURNS
c     inormalize = 0: no errors 
c                = 1: errors were found
c----------------------------------------------------------------------
      function inormalize(raw, fuel, fgr, mix)
      include 'h_stddefs.for'
c INPUTS
      real(8) :: raw(mxcomp)
      real(8) :: fuel(mxcomp)
      real(8) :: mix(mxcomp)
      real(8)  :: fgr
      integer :: inormalize
c DECLARATIONS
      integer   :: j
      do j=1,mxcomp
          if( prop(ipcalc,j) .gt. 0) then
              mix(j) =   (raw(j)+fgr*fuel(j))/(1.+fgr)
          end if 
      end do

      inormalize = 0
      return 
      end

c----------------------------------------------------------------------
c inormalizeh2s
c----------------------------------------------------------------------
c
c     Normalizes a gas composition with the addition of a new H2S 
c     mole fraction
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     raw(mxcomp)     : real : raw gas composition
c     fh2s            : real : mole fraction of H2S
c
c --- OUTPUTS
c     mix(mxcomp) : real : mixed gas composition
c
c --- RETURNS
c     inormalizeh2s = 0: no errors 
c                   = 1: errors were found
c
c
c----------------------------------------------------------------------
      function inormalizeh2s(raw, fh2s, mix)
      include 'h_stddefs.for'
c INPUTS
      real(8) :: raw(mxcomp)
      real(8) :: fh2s
      real(8) :: mix(mxcomp)
      integer :: inormalizeh2s
c DECLARATIONS
      real(8)      :: fnorm
      integer   :: j
      fnorm    = (1.-fh2s)/(1.-raw(jph2s))
      do j=1,mxcomp
          if( prop(ipcalc,j) .gt. 0) then
              mix(j) =   raw(j)*fnorm
          end if 
      end do
      mix(jph2s) = fh2s
      inormalizeh2s = 0
      return 
      end

c----------------------------------------------------------------------
c calc_molarmass()
c----------------------------------------------------------------------
c
c     Calculates the molar mass from a gas mixture
c
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c     
c --- OUTPUTS
c
c --- RETURNS
c     calc_molarmass  : real : molar mass  (kg/kmol)
c
c----------------------------------------------------------------------
c --- kg/kmol
      function calc_molarmass(mix)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(*)  ! mxcomp
      real(8) :: calc_molarmass      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_molarmass = calc_byprop(mix,ipmmass,ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_nhv
c----------------------------------------------------------------------
c
c     Calculates the net heating value (lower heating value) from a 
c     gas composition mixture
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_nhv    :real   : net heating value  (MJ/m3)
c
c----------------------------------------------------------------------
c --- MJ/m3
      function calc_nhv(mix)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_nhv      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_nhv = calc_byprop(mix, ipnhv, ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_ghv()
c----------------------------------------------------------------------
c
c     Calculates the gross heating value (higher heating value) of a 
c     gas composition mixture
c
c     DEVELOPED B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_ghv : real : gross heating value  (MJ/m3)
c----------------------------------------------------------------------
c --- MJ/m3
      function calc_ghv(mix)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_ghv      
c      include 'h_interface.for'
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

      calc_ghv = calc_byprop(mix, ipghv, ipcalc)
      return 
      end

c----------------------------------------------------------------------
c calc_nhvmass
c----------------------------------------------------------------------
c
c     Calculates the net heating value in mass units
c
c     Developed by B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c     mmass   : real : molar mass
c     rnhv    : real : net heating value  (MJ/m3)
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_nhvmass : real: net heat value (MJ/kg)
c
c --- uses
c     cpv : real: specific volume
c----------------------------------------------------------------------
      function calc_nhvmass(mmass, rnhv)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mmass, rnhv
      real(8) :: calc_nhvmass
      
      calc_nhvmass= rnhv/mmass * cpv
      return 
      end


c----------------------------------------------------------------------
c calc_byprop()
c----------------------------------------------------------------------
c
c     Calculates a property from a gas composition based upon
c     the requested inprop and incalc. inprop2
c     allows more complex calculations.
c
c     DEVELOPED by B.Zelt - www.zeltpsi.com
c
c     mix      : mole fraction of gas mix
c     inprop   : property to sumproduct (see stddefs.h)
c     incalc   : conditional property
c
c --- UPDATES
c
c --- INPUTS
c     mix(mxcomp)     : real : gas composition
c     inprop  : integer : property to be calculated
c     incalc  : integer : conditional property.  If the incalc property
c                     is set, then the component is included in the calculation
c     inprop2 : integer : OPTIONAL
c                     if inprop2 is set then this property
c                     is added to the product
c
c --- OUTPUTS
c
c --- RETURNS
c     calc_byprop : real : sumproduct calculation
c----------------------------------------------------------------------
      function calc_byprop(mix, inprop, incalc, inprop2)
      include 'h_stddefs.for'
c INPUTS
      real(8) :: calc_byprop      
      real(8) :: mix(*) ! (mxcomp)
      integer :: inprop
      integer :: incalc
      integer,optional :: inprop2
c DECLARATIONS
      integer  :: j
      real(8)  :: rinc

      calc_byprop =0.
      do j=1,mxcomp
          if( prop(incalc,j) .gt. 0) then
              rinc = mix(j)*prop(inprop,j)
              if(present(inprop2)) then
                  rinc = rinc * prop(inprop2,j)
              end if 
              calc_byprop= calc_byprop+  rinc
          end if 
      end do
      return 
      end


c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hydrocarbons(mix)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_hydrocarbons      
c DECLARATIONS
      integer  :: j

      calc_hydrocarbons = 0
      do j=1,mxcomp
          if( prop(iph,j) .gt. 0) then
              calc_hydrocarbons  = calc_hydrocarbons +  mix(j)
          end if 
      end do
      return 
      end

c --- 
c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_lfl(mix )
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_lfl      
c DECLARATIONS
      integer  :: j
      real(8)     :: sum_comb

      sum_comb =0.
      calc_lfl =0.
      do j=1,mxcomp
          if( prop(iplfl,j) .gt. 0.) then
              calc_lfl= calc_lfl+ mix(j)*1./prop(iplfl,j)*prop(ipcomb,j)
              sum_comb = sum_comb + mix(j)*prop(ipcomb,j)
          end if 
      end do
      if(calc_lfl .gt. 0.) then
          calc_lfl = 0.01/(calc_lfl/sum_comb)
      end if
      return 
      end


c----------------------------------------------------------------------
c
c B.Zelt - www.zeltpsi.com
c
c Calculations the autoignition temperature, K 
c v130113
c 
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_autoig(mix )
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8)  :: calc_autoig      
c DECLARATIONS
      integer  :: j
      real(8)  :: sum_comb

      sum_comb =0.
      calc_autoig =0.
      do j=1,mxcomp
          if( prop(ipautoig,j) .gt. 0.) then
              calc_autoig= calc_autoig+ mix(j)*prop(ipautoig,j)*
     &              prop(ipcomb,j)
              sum_comb = sum_comb + mix(j)*prop(ipcomb,j)
          end if 
      end do
      if(sum_comb .gt. 0.) then
          calc_autoig= calc_autoig/sum_comb
      end if
      calc_autoig = calc_autoig + rk0
      return 
      end
      
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c old
c    Hf = aT + b/2T^2 + c/3T^3 + d/4T^4 
c new 
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c     131128 - change temperature default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hf(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf      
c DECLARATIONS
      real(8) :: cp_tref, cp_lo, cp_hi,a1corr,b1corr
      real(8) :: t1000

      cp_tref = calc_cp_ref(jcomp,tref)
      a1corr = (prop(ipcpmole,jcomp)-cp_tref)/ru

      t1000 = 1000.   ! default match temperature K         
      if( tempK .le. t1000) then
          calc_hf = ((prop(ipa1,jcomp)+a1corr)+
     &    tempk*(prop(ipa2,jcomp)/2.+
     &    tempk*(prop(ipa3,jcomp)/3.+
     &    tempk*(prop(ipa4,jcomp)/4.+
     &    tempk*prop(ipa5,jcomp)/5. ))))
          calc_hf = calc_hf* tempk*ru
c calc a6 correction for <1000K
          hf_lo = calc_hf_ref(jcomp,tref) +a1corr*1000.*ru
          calc_hf = calc_hf + hf_lo
      else
          cp_lo = calc_cp(jcomp,t1000)
          cp_hi = calc_cp_hi(jcomp,t1000)
          b1corr = (cp_lo - cp_hi)/ ru
          calc_hf = ((prop(ipb1,jcomp)+b1corr)+
     &    tempk*(prop(ipb2,jcomp)/2.+
     &    tempk*(prop(ipb3,jcomp)/3.+
     &    tempk*(prop(ipb4,jcomp)/4.+
     &    tempk*prop(ipb5,jcomp)/5. ))))
          calc_hf = calc_hf* tempk*ru
c calc a6 correction for 1000K
          hf_lo = calc_hf_ref(jcomp,t1000) +a1corr*1000.*ru
          hf_hi = calc_hf_hi(jcomp,t1000)  +b1corr*1000.*ru
          calc_hf = calc_hf + (hf_lo-hf_hi)
      end if

      return 
      end

c
c
c B.Zelt - www.zeltpsi.com
c
c
c --- UPDATES
c
c --- INPUTS
c       tempk : K
c
c --- OUTPUTS
c
c --- RETURNS
      function calc_hf_ref(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf_ref      
c DECLARATIONS

      calc_hf_ref= tempk*((prop(ipa1,jcomp))+
     &tempk*(prop(ipa2,jcomp)/2.+
     &tempk*(prop(ipa3,jcomp)/3.+
     &tempk*(prop(ipa4,jcomp)/4.+
     &tempk*prop(ipa5,jcomp)/5. )))) *ru

      return 
      end

c
c
c B.Zelt - www.zeltpsi.com
c
c
c --- UPDATES
c
c --- INPUTS
c     temp : K
c
c --- OUTPUTS
c
c --- RETURNS
      function calc_hf_hi(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: tempk
      integer :: jcomp
      real(8) :: calc_hf_hi      
c DECLARATIONS

      calc_hf_hi= tempk*((prop(ipb1,jcomp))+
     &tempk*(prop(ipb2,jcomp)/2.+
     &tempk*(prop(ipb3,jcomp)/3.+
     &tempk*(prop(ipb4,jcomp)/4.+
     &tempk*prop(ipb5,jcomp)/5. )))) *ru

      return 
      end



c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c calculates the molar average enthalpy
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hfbar(mix,temp)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: calc_hfbar            
c DECLARATIONS
      integer  :: j
c      include 'h_interface.for'

      calc_hfbar=0.
      do j=1,mxcomp
          calc_hfbar= calc_hfbar+  calc_hf(j,temp)* mix(j)
      end do

      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c calculates the mix enthalpy
c    Eq. 91: H0_T/(RT)=a1 +a2/2*T +a3/3*T^2 +a4/4*T^3 +a5/5*T^4 +a6/T
c
c --- UPDATES
c     131128 - changed default temperature to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_hfmix(mix,tempk)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_hfmix      
c DECLARATIONS
      integer  :: j
      real(8) :: a(7)
      real(8) :: b(7)
      real(8) :: t1000
c      include 'h_interface.for'
      interface
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      end interface

      
c calc MIX
      do i=1,7
          a(i) = 0.
          b(i) = 0.
      end do
      do j=1,mxcomp
          do i=1,5
              a(i) = a(i) + prop(ipa1+i-1,j) * mix(j) * ru
              b(i) = b(i) + prop(ipb1+i-1,j) * mix(j) * ru
          end do
      end do 
      a(6) = 0.0
      b(6) = 0.0

      t1000 = 1000. ! match temperature
      
      cp_ref= (a(1)+
     &tref*(a(2)+
     &tref*(a(3)+
     &tref*(a(4)+
     &tref*a(5) ))))
      acorr = calc_cpbar(mix,tref) - cp_ref

      if( tempK .gt. t1000) then
          cp_lo= (a(1)+ acorr+
     &    t1000*(a(2)+
     &    t1000*(a(3)+
     &    t1000*(a(4)+
     &    t1000*a(5) ))))

          cp_hi= (b(1)+
     &    t1000*(b(2)+
     &    t1000*(b(3)+
     &    t1000*(b(4)+
     &    t1000*b(5) ))))
          bcorr = (cp_lo - cp_hi)
c calculate b6 correction
          hf_lo= 0.
          hf_hi= 0.
          do i=5,1,-1
              hf_lo= t1000*(a(i)/float(i) + hf_lo)
              hf_hi= t1000*(b(i)/float(i) + hf_hi)
          end do
          hf_lo = hf_lo + acorr*t1000*ru
          hf_hi = hf_hi + bcorr*t1000*ru
          b(6) = (hf_lo-hf_hi)
          b(1) = b(1) + bcorr

      end if
      a(1) = a(1) + acorr

      if(tempK .le. t1000) then
          do i=1,6
              b(i) = a(i)
          end do
      end if 

c calc h
      calc_hfmix = 0.
      do i=5,1,-1
          calc_hfmix= tempK*(b(i)/float(i) + calc_hfmix)
      end do
      calc_hfmix= calc_hfmix+ b(6)

      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c calculates the mix enthalpy
c    Eq. 90: cp0/R=a1 +a2*T +a3*T^2 +a4*T^3 +a5*T^4 
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cpmix(mix,tempk)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_cpmix      
c DECLARATIONS
      integer  :: j
      real(8) :: a(7)
      real(8) :: b(7)
      real(8) :: t1000
c      include 'h_interface.for'
      interface
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      end interface

c calc MIX
      do i=1,7
          a(i) = 0.
          b(i) = 0.
      end do
      do j=1,mxcomp
          do i=1,5
              a(i) = a(i) + prop(ipa1+i-1,j) * mix(j) * ru
              b(i) = b(i) + prop(ipb1+i-1,j) * mix(j) * ru
          end do
      end do 
      a(6) = 0.0
      b(6) = 0.0

      t1000 = 1000. ! default match temperature K
      
      cp_ref= (a(1)+
     &tref*(a(2)+
     &tref*(a(3)+
     &tref*(a(4)+
     &tref*a(5) ))))
      acorr = calc_cpbar(mix,tref) - cp_ref

      if( tempK .gt. t1000) then
          cp_lo= (a(1)+ acorr+
     &    t1000*(a(2)+
     &    t1000*(a(3)+
     &    t1000*(a(4)+
     &    t1000*a(5) ))))

          cp_hi= (b(1)+
     &    t1000*(b(2)+
     &    t1000*(b(3)+
     &    t1000*(b(4)+
     &    t1000*b(5) ))))
          bcorr = (cp_lo - cp_hi)
c calculate b6 correction
          hf_lo= 0.
          hf_hi= 0.
          do i=5,1,-1
              hf_lo= t1000 *(a(i)/float(i) + hf_lo)
              hf_hi= t1000 *(b(i)/float(i) + hf_hi)
          end do
          hf_lo = hf_lo + acorr*t1000*ru
          hf_hi = hf_hi + bcorr*t1000*ru
          b(6) = (hf_lo-hf_hi)
          b(1) = b(1) + bcorr

      end if
      a(1) = a(1) + acorr

      if(tempK .le. t1000) then
          do i=1,6
              b(i) = a(i)
          end do
      end if 

c calc cp
      calc_cpmix = 0.
      do i=5,1,-1
          calc_cpmix = tempK*(b(i) + calc_cpmix)
      end do
      calc_cpmix = calc_cpmix/tempK

      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c   Eq. 90: cp0/R=a1 +a2T +a3*T^2 +a4*T^3 +a5*T^4
c
c --- UPDATES
c     131128
c        changed temperatue to default K 
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      integer :: jcomp
      real(8)     :: tempk
      real(8) :: calc_cp      
c DECLARATIONS
      real(8)      :: cp_tref
      real(8) :: t1000
c      include 'h_interface.for'

      cp_tref = calc_cp_ref(jcomp,tref)
      a1corr  = (prop(ipcpmole,jcomp)-cp_tref)/ru

      t1000 = 1000.  ! match point at 1000.K
      if( tempK .le. t1000 ) then
          cp_lo   = calc_cp_ref(jcomp,tempk)
          calc_cp = cp_lo + a1corr*ru
      else
          cp_lo = calc_cp_ref(jcomp,t1000) + a1corr*ru
          cp_hi = calc_cp_hi(jcomp,t1000)
          b1corr = (cp_lo - cp_hi)/ ru
          calc_cp = calc_cp_hi(jcomp,tempk)+ ru*b1corr
      end if
      return 
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_ref(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      integer :: jcomp
      real(8) :: tempk
      real(8) :: calc_cp_ref      
c DECLARATIONS

      calc_cp_ref= (prop(ipa1,jcomp)+
     &tempk*(prop(ipa2,jcomp)+
     &tempk*(prop(ipa3,jcomp)+
     &tempk*(prop(ipa4,jcomp)+
     &tempk*prop(ipa5,jcomp) )))) * ru
      return
      end



c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c   Eq. 90: cp0/R=a1 +a2T +a3*T^2 +a4*T^3 +a5*T^4
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cp_hi(jcomp,tempk)
      include 'h_stddefs.for'
c INPUTS
      integer :: jcomp
      real(8)     :: tempk
      real(8) :: calc_cp_hi      
c DECLARATIONS

      calc_cp_hi= (prop(ipb1,jcomp)+
     &tempk*(prop(ipb2,jcomp)+
     &tempk*(prop(ipb3,jcomp)+
     &tempk*(prop(ipb4,jcomp)+
     &tempk*prop(ipb5,jcomp) )))) * ru

      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c pass T in C
c
c --- UPDATES
c     131128
c        changed temperature to K default
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_cpbar(mix,tempk,inprop2)
      include 'h_stddefs.for'
c INPUTS
      real(8) :: mix(mxcomp)
      real(8) :: tempk
      real(8) :: calc_cpbar      
      integer,optional :: inprop2
c DECLARATIONS
      integer :: j
      real(8) :: rinc
c      include 'h_interface.for'

      calc_cpbar=0.
      do j=1,mxcomp
         rinc = mix(j)*calc_cp(j,tempk)
         if(present(inprop2)) then
            if(prop(inprop2,j).gt.0) then
               rinc = rinc * prop(inprop2,j)
            else
               rinc = 0.
            end if                
         end if 
         calc_cpbar= calc_cpbar+  rinc
      end do

      return 
      end

c----------------------------------------------------------------------
c wsstkht
c
c Calculate the wind speed at stack height
c
c B.Zelt - www.zeltpsi.com
c
c
c
c      
      subroutine wsstkht(wref, zref, ipg, hs, ws)      
c --- INPUTS
      real(8) :: wref   
      real(8) :: zref   
      real(8) :: hs   
      real(8) :: ws   
      integer :: ipg
c --- DECLARATIONS
      real(8) :: plx0(6)  ! wind profile power law exponents (6) 

c --- ISC rural defaults for wind profile power law exponents (6)
      data plx0/.07,.07,.10,.15,.35,.55/ 

      ws = wref
      
c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      If (hs .ge. 10.) Then
         ws = ws * (hs / Zref)**plx0(ipg)
      ElseIf (Zref .gt. 10.) Then
         ws = ws * (10. / Zref)**plx0(ipg)
      End If

      return
      end

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c     totalvolume = total volume to be flared (10^3m3)
c     duration    = flaring duration (s)
c     rate        = mix raw gas flare rate (10^3m3/d)
c     fgr         = mix fuel gas ratio
c     mix         = mix gas composition
c     MSTRIP      = 0 normal;  1=stripped gases source 
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      subroutine emissions(gcraw,rate,gcfuel,fgr,gclift,totalvolume,
     &   hs,dia,wref,zref,iPG,
     &  MSTRIP,MFUELGAS,MFASSIST,MLIFTGAS,
     &  qliftgas,stm_dia,stm_press,tstm,stm_flow,
     &  stm_nports,air_dia,air_flow,air_nports)
      include 'h_stddefs.for'
      include 'h_combustion.for'
c INPUTS
      real(8)  :: gcraw(mxcomp)  ! raw gas mix
      real(8)  :: rate        ! raw gas flare rate (10^3m3/d)
      real(8)  :: gcfuel(mxcomp) ! fuel gas mix 
      real(8)  :: fgr         ! fuel gas ratio
      real(8)  :: gclift(mxcomp) ! liftgas mix 
      real(8)  :: totalvolume ! total volume to be flared (10^3m3)
      real(8)  :: hs          ! stack height (m)
      real(8)  :: dia         ! stack inside diameter (m)
      real(8)  :: wref          ! reference wind speed (m/s)
      real(8)  :: zref        ! windspeed reference height (m)
      integer :: iPG       ! PG stability class
	integer :: MSTRIP	   ! 
	integer :: MFASSIST	! iflareassist:0 no assist; 1=steam; 2=air; 3=both
      integer :: MLIFTGAS  !0=no;1=yes lift gas
      real    :: qliftgas  ! lift gas flow rate
      real(8) :: stm_dia			! steam assist individual port diameter [m]
      real(8) :: stm_press		! steam assist pressure of steam or -1 if unknown
      real(8) :: tstm     		! K, steam assist temperature of steam or -1 if unknown
      real(8) :: stm_flow		! steam assist mass flow rate kg/s or flow rate e3m3/d
	integer :: stm_nports	! number of steam assist injection ports
	integer :: air_nports	! number of air assist injection ports
      real(8) :: air_dia			! air assist individual port diameter[m]
      real(8) :: air_flow		! air assist flow rate e3m3/d
c DECLARATIONS
      real(8)  :: ws             ! stack top wind speed (m/s)
      real(8)  :: mix0(mxcomp)   ! raw + fuel + lift gasmix
      real(8)  :: mixa(mxcomp)   ! raw + fuel gas + air mix
      real(8)  :: mixb(mxcomp)   ! raw + fuel gas + air + steam mix 
      real(8)  :: mix(mxcomp)    ! final (raw + fuel + air + steam + lift) mix
      real(8)  :: gcstm(mxcomp)
      real(8)  :: gcair(mxcomp)
      real(8)  :: mix0_m     ! fuel + raw + lift gas mass flow
      real(8)  :: mix0_v     ! fuel + raw + lift gas volume flow
      real(8)  :: lift_v     ! liftgas volume flow
      real(8)  :: lift_m     ! liftgas mass flow 
      real(8)  :: finert  ! fraction of inerts in air
      
c DECLARATIONS      
      real(8)  :: rmm,rnhv,rghv
      real(8)  :: exhaustmix(mxcomp)
      real(8)  :: rk3     ! Buoyancy Flux-Heat Release Rate Constant (K3),m4/(s2J)
      real(8)  :: tambk ! ambient temperature in K

      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      
      end interface
      
c force or correct entries
      if(MLIFTGAS.eq.0) then
         qliftgas = 0.
      end if 
      if(MFASSIST.eq.0) then
         stm_flow = 0.   
         air_flow = 0.   
         stm_dia_eff = 0.
         air_dia_eff = 0.
      else if(MFASSIST.eq.1) then
         stm_dia_eff = sqrt(stm_nports*stm_dia*stm_dia)
         air_flow = 0.   
      else if(MFASSIST.eq.2) then
         air_dia_eff = sqrt(air_nports*air_dia*air_dia)
         stm_flow = 0.   
      else         
         stm_dia_eff = sqrt(stm_nports*stm_dia*stm_dia)
         air_dia_eff = sqrt(air_nports*air_dia*air_dia)
      end if 
      
c set standard steam gas compositions      
      ier = initialize(gcstm,0.0D0)
      gcstm(jph2o) = 1.0
c set standard air gas compositions      
      ier = initialize(gcair,0.0D0)
      finert = 0.0
      do i=1,mxcomp
         gcair(i) = prop(ipair,i)
         if(prop(ipinert,i)>0.) then
            finert = finert + gcair(i)
         end if 
      end do

c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      call wsstkht(wref, zref, ipg, hs, ws)      

c --- 131128, changed ambient temperature to K default      
c      tambk = tamb + rk0
      if(debug .gt. 0 ) then
      write(iolog,*) 'wref = ',wref,'  tamb=',tamb,'  zref=',zref
      write(iolog,*) 'ws = ',ws,'   hs=',hs
      end if 
      
c Calculate flare assisted final mix      
      if( MFUELGAS .eq. 1) then   
         ierr = inormalize(gcraw, gcfuel, fgr, mixb)
         totalflaringrate_ve3m3 = calc_totalrate(rate, fgr)
         rmmfuel  = calc_molarmass(gcfuel)   ! kg/kmole
      else
         ierr = inormalize(gcraw, gcraw, 1.0D0, mixb)
         totalflaringrate_ve3m3 = rate
         rmmfuel  = 0.d0   ! kg/kmole
      end if 

      if( MLIFTGAS .eq. 1) then   
         ierr = inormalize(mixb, gclift, 
     &        qliftgas/totalflaringrate_ve3m3, mix0)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + qliftgas
         rmmlift  = calc_molarmass(gclift)   ! kg/kmole
      else
         ierr = inormalize(mixb, mixb, 
     &        1.0D0, mix0)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
         rmmlift  = 0.D0   ! kg/kmole
      end if 
      
      if( MFASSIST .eq. 2 .or. MFASSIST.eq.3) then 
         ierr = inormalize(mix0, gcair, 
     &        air_flow/totalflaringrate_ve3m3, mixa)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + air_flow
      else
         ierr = inormalize(mix0, mix0, 
     &        1.0D0, mixa)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 
      
      if( MFASSIST .eq. 1 .or. MFASSIST.eq.3) then 
         ierr = inormalize(mixa, gcstm, 
     &        stm_flow/totalflaringrate_ve3m3, mix)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3 + stm_flow
      else
         ierr = inormalize(mixa, mixa, 
     &        1.0D0, mix)
         totalflaringrate_ve3m3 = totalflaringrate_ve3m3
      end if 

c Basic properities
      rmm      = calc_molarmass(mix)      ! kg/kmole
      rmmraw   = calc_molarmass(gcraw)    ! kg/kmole
      rmmflare = calc_molarmass(mix0)     ! kg/kmole
      rmmstm   = calc_molarmass(gcstm)    ! prop(ipmmass,jph2o)      ! kg/kmole
      rmmair  = calc_molarmass(gcair)    ! prop(ipmmass,jpair)      ! kg/kmole
c      rmmair   = prop(ipmmass,jpair)      ! kg/kmole
      
      if(MFUELGAS .eq. 1) then
         mix0_v = (qliftgas+calc_totalrate(rate, fgr))* 1000./(24.*3600.)  ! volume flow rate, m3/s
         mix0_m = mix0_v/cpv*rmmflare              ! mass flow rate, kg/s
      else
         mix0_v = (qliftgas+calc_totalrate(rate, 0.D0))* 1000./(24.*3600.)  ! volume flow rate, m3/s
         mix0_m = mix0_v/cpv*rmmflare              ! mass flow rate, kg/s
      end if 
      raw_v  = rate* 1000./(24.*3600.)          ! volume flow rate, m3/s
      raw_m  = raw_v/cpv*rmmraw                 ! mass flow rate, kg/s
      fuel_v = fgr/(1.+fgr)*calc_totalrate(rate, fgr)* 1000./(24.*3600.)  ! volume flow rate, m3/s
      fuel_m = fuel_v/cpv*rmmfuel               ! mass flow rate, kg/s
      stm_v  = stm_flow* 1000./(24.*3600.)      ! volume flow rate, m3/s
      stm_m  = stm_v/cpv*rmmstm                 ! mass flow rate, kg/s
      air_v  = air_flow* 1000./(24.*3600.)      ! volume flow rate, m3/s
      air_m  = air_v/cpv*rmmair                 ! mass flow rate, kg/s
      lift_v  = qliftgas* 1000./(24.*3600.)     ! volume flow rate, m3/s
      lift_m  = lift_v/cpv*rmmlift              ! mass flow rate, kg/s
      
c EMISSIONS
      rnhv = calc_nhv(mix)             ! MJ/m3
      rghv = calc_ghv(mix)             ! MJ/m3
      cpmol_mix  = calc_byprop(mix,     ipcpmole, ipcalc)
      cpmol_raw  = calc_byprop(gcraw,   ipcpmole, ipcalc)
      cpmol_fuel = calc_byprop(gcfuel,  ipcpmole, ipcalc)
      cpmol_stm  = calc_byprop(gcstm,   ipcpmole, ipcalc)
      cpmol_air  = calc_byprop(gcair,   ipcpmole, ipcalc)
      cpmol_lift = calc_byprop(gclift,  ipcpmole, ipcalc)
      cpmass_mix  = 0.
      cpmass_raw  = 0.
      cpmass_fuel = 0.
      cpmass_stm  = 0.
      cpmass_air  = 0.
      cpmass_lift = 0.
      if(rmm .gt.0)  cpmass_mix  = cpmol_mix/ rmm
      if(rmmraw .gt. 0.)  cpmass_raw   = cpmol_raw/ rmmraw
      if(rmmfuel .gt. 0.) cpmass_fuel  = cpmol_fuel/ rmmfuel
      if(rmmstm .gt. 0.)  cpmass_stm   = cpmol_stm/ rmmstm
      if(rmmair .gt. 0.)  cpmass_air   = cpmol_air/ rmmair
      if(rmmlift.gt. 0.)  cpmass_lift  = cpmol_lift/ rmmlift
      
      totalflaringrate_v = totalflaringrate_ve3m3* 1000./(24.*3600.) ! m3/s at ref
      totalflaringrate_m = totalflaringrate_v*rmm/cpv                ! kg/s
      
      rnetheatreleased_a = totalflaringrate_ve3m3 * rnhv * rk1     ! cal/s
c n/r      rnetheatreleased_b   = totalflaringrate_v* rnhv*3600./1000.  ! GJ/h
      rnetheatreleased_c   = totalflaringrate_ve3m3 * rnhv * rk4     ! J/s
c H2S Release Rate at 0% conversion
      e_h2s_v   = totalflaringrate_v  * mix(jph2s)     ! m3/s
      e_h2s_m   = e_h2s_v * prop(ipmmass,jph2s)/cpv * 1000. !g/s
c SO2 Emission Rate at 100% conversion
      e_so2_m1  = totalflaringrate_ve3m3*mix(jph2s) /
     &cpv*prop(ipmmass,jpso2)   ! t/d
      e_so2_m   = e_so2_m1 * 1000.*1000./ (24.*3600.)                            ! g/s
      e_s_m1    = e_so2_m1 * prop(ipmmass,jps)/ prop(ipmmass,jpso2)    ! t/d
      e_s_mtotal= e_s_m1 * totalvolume/ rate !  t
      fuel_carbons = calc_byprop(mix, ipstoicc, iph)
     &/ calc_hydrocarbons(mix)
      fuel_hydrogen= fuel_carbons*2. + 2.   
c Process CO2 Emission Rate, t/d      
      e_co2_process    = mix(jpco2)*prop(ipmmass,jpco2)/cpv*
     &        totalflaringrate_ve3m3                            
c Combustion CO2 Emission Rate at 100% conversion, t/d      
      e_co2_combustion = (calc_byprop(mix, ipstoicc, iph) + 
     &        mix(jpco)*prop(ipstoicc,jpco))* 
     &        prop(ipmmass,jpco2)/ cpv * totalflaringrate_ve3m3 
      e_co2_m1    = (e_co2_process+e_co2_combustion)
      e_co2_total = e_co2_m1  * totalvolume/ rate  ! t

c      ue_epa = 
c      usmax  = ???
c      diamin=1000.*sqrt(4./pi*totalflaringrate_v*(pref/plocal)*(tgas/tref)/ue_epa)
c      diamax=1000.*sqrt(4./pi*averageflaringrate_v *(pref/plocal)*(tgas/tref)/usmax)

      if(debug .gt. 0 ) then
         write(iolog,*) 'NHV (MJ/m3)=',rnhv
         write(iolog,*) 'GHV (MJ/m3)=',rghv
      end if                  


c --- Average Cp, Air
      cpair_tamb      = calc_cp(jpair, tamb)    ! fneq90
      cpair_tref      = calc_cp(jpair, tref)    ! fneq90
      hf_air_tamb     = calc_hf(jpair, tamb)
      hf_air_tref     = calc_hf(jpair, tref)
      z_air_tamb      = 1.
      z_air_tref      = 1.
      rho_air_tamb    = plocal*rmmair/ru/tamb/z_air_tamb
      rho_air_tref    = plocal*rmmair/ru/tref/z_air_tref
      c_air_tamb  = sqrt((cpair_tamb/(cpair_tamb-ru))*
     &            ru/rmmair*tamb*1000.*z_air_tamb)
      c_air_tref  = sqrt((cpair_tref/(cpair_tref-ru))*
     &            ru/rmmair*tref*1000.*z_air_tref)
      if(tamb .eq. tref ) then 
          cpair_ave_mol   =  cpair_tref
      else
          cpair_ave_mol   = (hf_air_tref-hf_air_tamb)/(tref-tamb)
      end if       
      cpair_ave_mass  = cpair_ave_mol / rmmair
      
c --- Average Cp, Steam
      cpstm_tstm      = calc_cp(jph2o, tstm)
      cpstm_tref      = calc_cp(jph2o, tref)
      hf_stm_tstm     = calc_hf(jph2o, tstm)
      hf_stm_tref     = calc_hf(jph2o, tref)
      z_stm_tstm      = fnz(tstm,stm_press,tch2o,pch2o)
      z_stm_tref      = fnz(tref,plocal,tch2o,pch2o)
      rho_stm_tstm    = stm_press*rmmstm/ru/tstm/ z_stm_tstm
      rho_stm_tref    = plocal*rmmstm/ru/tref   / z_stm_tref
      c_stm_tstm  = sqrt((cpstm_tstm/(cpstm_tstm-ru))*
     &            ru/rmmstm*tstm*1000.*z_stm_tstm)
      c_stm_tref  = sqrt((cpstm_tref/(cpstm_tref-ru))*
     &            ru/rmmstm*tref*1000.*z_stm_tref)
      if(tstm .eq. tref ) then 
          cpstm_ave_mol   =  cpstm_tref
      else
          cpstm_ave_mol   = (hf_stm_tref-hf_stm_tstm)/(tref-tstm)
      end if       
      cpstm_ave_mass  = cpstm_ave_mol / rmmstm

c Exit Velocity
c --- MOMENTUM
c Exit Velocity (VE) at release temperature and site pressure,m/s
      ueflare = mix0_v*4./pi/(dia*dia)*tgas/tref
     &        *pref/plocal
      uestm   =0.0
      if( stm_v .gt. 0.) then
          uestm   = min(stm_v/pi*4./(stm_dia_eff*stm_dia_eff)*
     &             tstm/tref*pref/plocal,c_stm_tstm)
         uestm = uestm !* 1.d6
      end if 
      ueair   = 0.
      if(air_v .gt. 0.) then
          ueair   = min(air_v/pi*4./(air_dia_eff*air_dia_eff)*
     &             tamb/tref*pref/plocal,c_air_tamb)
         ueair= ueair  !* 1.d6
      end if 

c Steam Density at stm temperature and site pressure,kg/m
      den_stm     =plocal*rmmstm  /(ru*tstm)
c Air Density at air temperature and site pressure,kg/m
      den_air     =plocal*rmmair  /(ru*tamb)

      
c --- MOMENTUM  
c Note that momentum flux is relative to air
c Flared Incremental Momentum Flux FM,m4/s2
      FM_flare= ueflare*mix0_m/den_air/pi
c Steam Incremental Momentum Flux FM,m4/s2
      FM_stm  = uestm*stm_m/den_air/pi
c Air Incremental Momentum Flux FM,m4/s2
      FM_air  = ueair*air_m/den_air/pi

      

      
c --- COMBUSTION CALCULATIONS

c combustible fraction of mixture
      combfrac = calc_byprop(mix, ipcomb, ipcalc) 
c Lower Flammability Limit (LFL) of combustible flared gas
      rLFLcomb    = calc_lfl(mix)
c Lower Flammability Limit (LFL) of flared mixture
      rLFLflared  = 0.
      if(combfrac .gt.0.)then
          rLFLflared  = rLFLcomb/combfrac
      end if 
c Minimum Oxygen Concentration to support Combustion of combustibles
      rminoxyg    = rLFLcomb*calc_byprop(mix, ipstoico2, ipcalc, ipcomb)
     &                 / combfrac
c Maximum Inert Concentration of flared mixture
      rmaxinert   = 1. - rLFLcomb - rminoxyg
c Is proposed mixture flammable? (but we assume it is for now)
      isFlamable = 1
      if(1.-combfrac .gt. rmaxinert ) then
          isFlamable = 0
      end if 
      
      if(debug .gt. 0 ) then
      write(iolog,*) 'combfrac=',combfrac
      write(iolog,*) 'rLFLcomb=',rLFLcomb
      write(iolog,*) 'rLFLflared=',rLFLflared
      write(iolog,*) 'isFlammable(1=y,0=n)=',isFlammable
      end if       
      
c  mole air / mole fuel for MOC
      rmamf_moc = (rminoxyg-mix(jpo2))/(1.-finert-rminoxyg)
c mole air/ mole fuel to give LFL
      rmamf = (1.-rLFLflared)/rLFLflared
c stoichiometric mole O2 required / mole fuel
      stoic_o2 =calc_byprop(mix, ipstoico2, ipcalc)- mix(jpo2)

c stoichiometric mole air required / mole fuel
      stoic_air=stoic_o2/(1.-finert)
c Flame Excess Air for LFL or MOC,% 
      fealfl   =(max(rmamf_moc,rmamf)-stoic_air)/stoic_air    
c mole O2 supplied / mole fuel
      rfsupo2    =max(rmamf_moc,rmamf)*(1.-finert) + mix(jpo2)
c mole wet exhaust / mole fuel
      exwet    = calc_byprop(mix, ipstoicp, ipcalc) + 
     &        max(rmamf_moc,rmamf)/(1.+fealfl)*(finert+fealfl)
c mole dry exhaust / mole fuel
      exdry    =exwet-calc_byprop(mix, ipstoich2o, ipcalc)
c mole H2O / mole fuel
      rfh2o    =exwet-exdry
c mole O2 / mole fuel
      rfo2     =rfsupo2-stoic_o2
c mole Ar / mole fuel
      rfar     =mix(jpar)+MAX(rmamf_moc,rmamf)*gcair(jpar)
c mole CO2 / mole fuel
      rfco2    =calc_byprop(mix, ipstoicc, ipcalc)
c mole SO2 / mole fuel
      rfso2    = mix(jph2s)
c mole N2 / mole fuel
      rfn2     = exwet-rfh2o-rfo2-rfar-rfco2-rfso2
c Exhaust flow at Pamb and Tamb,m/s
      exrate_amb =totalflaringrate_v*exwet*pref/plocal*
     &(tgas/tref)*(tamb/tgas)
     &*(carb+(1.-carb)*(1.-fs))
c air required at Pamb and Tamb,m/s      
      air_amb = exrate_amb *rmamf/exwet

c--- initialize exhaust mixture
      ier = initialize(exhaustmix,0.0D0)
      exhaustmix(jpair) = 0
      exhaustmix(jph2o) = rfh2o/ exwet
      exhaustmix(jpo2)  = rfo2/ exwet
      exhaustmix(jpar)  = rfar/ exwet
      exhaustmix(jpn2)  = rfn2/ exwet
      exhaustmix(jpco2) = rfco2/ exwet
      exhaustmix(jpso2) = rfso2/ exwet

c mole Excess Air / mole fuel
      excessair=rfo2/(1.-finert)
c Stoichiometric Exhaust Molar Mass,kg/kmol
      sexmm = (rfh2o*prop(ipmmass,jph2o)+
     &stoic_air*calc_byprop(gcair,ipmmass,ipinert)+
     &rfco2*prop(ipmmass,jpco2)+rfso2*prop(ipmmass,jpso2))
     &/(rfh2o+stoic_air*finert+rfco2+rfso2)
c Stoichiometric Exhaust Mass Specific Heat,kJ/(kgK)
      sexcp = (rfh2o*calc_cp(jph2o,tref)+
     &stoic_air*calc_cpbar(gcair,tref,ipinert)+
     &rfco2*calc_cp(jpco2,tref)+rfso2*calc_cp(jpso2,tref))
     &/(rfh2o+stoic_air*finert+rfco2+rfso2)/sexmm


      
c --- Temperature with Assist
c Flared Gas Enthalpy, kJ/s      
      hf_raw = raw_m/rmmraw   * 
     &         (calc_hfmix(gcraw, tgas) - calc_hfmix(gcraw, tref))  
      hf_lift= 0.
      hf_fuel= 0.
      hf_stm = 0.
      hf_air = 0.
      if(MLIFTGAS .eq. 1) then
         hf_lift= lift_m/rmmlift * 
     &         (calc_hfmix(gclift, tgas) -calc_hfmix(gclift, tref))
      end if 
      if(MFUELGAS .eq. 1) then
         hf_fuel= fuel_m/rmmfuel * 
     &         (calc_hfmix(gcfuel, tgas) -calc_hfmix(gcfuel, tref)) 
      end if 
      if( MFASSIST .eq. 1 .or. MFASSIST.eq.3) then 
         hf_stm = stm_m/rmmstm   * 
     &         (calc_hfmix(gcstm, tstm) -calc_hfmix(gcstm, tref)) 
      end if          
      if( MFASSIST .eq. 2 .or. MFASSIST.eq.3) then 
         hf_air = air_m/rmmair   * 
     &         (calc_hfmix(gcair, tamb) -calc_hfmix(gcair, tref)) 
      end if          
      hf_mix = hf_raw + hf_lift + hf_fuel + hf_stm + hf_air 
c Estimated Temperature of Flared Gas, K
      tmix = hf_mix/ (cpmass_mix*totalflaringrate_m) + tref
c      rmdotcp_raw = raw_m *cpmass_raw *tgas
c      rmdotcp_lift= lift_m*cpmass_lift*tgas
c      rmdotcp_fuel= fuel_m*cpmass_fuel*tgas
c      rmdotcp_stm = stm_m *(cpstm_tref/rmmstm)  *tstm 
c      rmdotcp_air = air_m *(cpair_tref/rmmair)  *tamb 
c      rmdotcp_mix = rmdotcp_raw + rmdotcp_lift + 
c     &              rmdotcp_fuel +rmdotcp_stm +rmdotcp_air
c Estimated Temperature of Flared Gas, K
c      tmix  = rmdotcp_mix/totalflaringrate_m/cpmass_mix 
c goal enthalpy
c      egoal = rmdotcp_mix/totalflaringrate_m*rmm - cpmol_mix*tref

c goal enthalpy
      egoal = hf_mix/totalflaringrate_m*rmm 
c Temperature of Flared Gas, K
      extemp   = calc_flametemp(tmix, mix, egoal)  
c      extemp   = MAX(extemp, tamb+rk0)
      tflared = extemp   ! check : this should be same as tgas

c now use below 'Fuel Molar Average Specific Heat'      
c Fuel Molar Specific Heat at 15C,kJ/(kmolK)
      fcp   = calc_byprop(mix, ipcpmole, ipcalc) 
c Fuel Mass Average Specific Heat at 15C kJ/(kgK)
      fcp_m = fcp/rmm
      
c Fuel Molar Average Specific Heat at Fuel Temperature,kJ/(kmolK)
      temp_hf = tflared
      temp_h0 = tref
      cpbarf  = fcp_m
      if(temp_hf .ne. temp_h0 ) then                                    
          cpbarf = (calc_hfbar(mix,temp_hf)-calc_hfbar(mix,temp_h0))/
     &    (temp_hf-temp_h0)
      end if
c Fuel Mass Average Specific Heat at Fuel Temperature,kJ/(kgK)
      cpbarf_m = cpbarf/rmm
      
      cpair       = prop(ipcpmole,jpair)   ! calc_cp(jpair, tref)
      cpair_m     = cpair/rmmair
c      cpair_tamb  = calc_cp(jpair, tamb)
      hf_air =  calc_hf(jpair, tref) - 
     &            calc_hf(jpair, tamb)
      cpair_tamb  = cpair
      if( hf_air.gt. 0. .and. tref.ne.tamb) then
          cpair_tamb    = hf_air/(tref-tamb)
      end if               
      cpair_m_tamb= cpair_tamb /rmmair

      
c******************************************************************************      
      
      
c mixed flared gas Density at release temperature and site pressure,kg/m
      den_flare   =plocal*rmm/(ru*tflared)

c -------------------------------------------------------------     
c --- combustion efficiency  
c --- Exit Velocity
c Flare Tip Momentum, m/s.kg/s
c Steam Flare Assist Orifice(s) Momentum, m/s.kg/s
c Air Flare Assist Orifice(s) Momentum, m/s.kg/s
c Effective Release Exit Velocity, m/s
c flared gas exit velocity, m/s
      ue = (ueflare*mix0_m + uestm*stm_m + ueair*air_m)/
     &            totalflaringrate_m

      eff_max = max(1.-uofaa/(rnhv/rmm*cpv)**3.,0.)
c Modelled Conversion efficiency (CONV fraction)
      carb  = carbeff(rnhv/rmm*cpv, ws, dia, ue  )
      carb  = max(effmin,carb)    ! set minimum
c Vj/Ua - Jet to Air velocity ratio 
      Vj_Ua = min(110.,ue/ws)
c R - Jet to Air momentum flux ratio
      rjet  = den_flare/den_air*Vj_Ua*Vj_Ua
c Fraction of flared gas stripped and unconverted, remaining portion combusted (fs)
      fs    = min(0.9,max(0.1,0.5-0.0869*log(rjet)))
c Modelled Combustion Efficiency (COMB fraction)     
      comb  = carb/(carb+(1.-fs)*(1.-carb)) 
      
      if(debug .gt. 0 ) then
      write(iolog,*) 'conv eff=',carb
      write(iolog,*) 'comb eff=',comb
      write(iolog,*) 'cold fraction,fs=',fs
      end if 
c -------------------------------------------------------------     

      
c Stripped Gas Momentum Flux (loss)	m4/s2
      FM_strip= -(FM_flare + FM_stm + FM_air)*(1.-carb)*fs
c Total momemtum      
      FM = FM_flare + FM_stm + FM_air + FM_strip
      
      
c Emissions at combustion efficiency      
      e_co2 = (e_co2_process+e_co2_combustion *carb) + 
     &(1.-carb)*mix(jpch4)*
     &totalflaringrate_ve3m3 *prop(ipmmass,jpch4)/cpv*25.  ! t/d
      e_h2s = (1.-carb)*fs*e_h2s_m                         ! g/s
      e_so2 = e_so2_m*(carb + (1.-carb)*(1.-fs))           ! g/s

      if(debug .gt. 0 ) then
      write(iolog,*) 'e_co2=',e_co2,' t/d'
      write(iolog,*) 'e_so2=',e_so2, e_so2_m,' g/s'
      write(iolog,*) 'e_h2s=',e_h2s,' g/s as unoxydized raw H2S'
      end if 
      
c --- Combustion Products

c Exhaust Molar Mass ,kg/kmol
      exmm  = calc_byprop(exhaustmix, ipmmass, ipcalc)
c Exhaust Molar Specific Heat at 15C,kJ/(kmolK)
      excp  = calc_cpbar(exhaustmix, tref)
c Exhaust Mass Specific Heat at 15C,kJ/(kgK)
      excp_m = excp/exmm 
c Exhaust to Fuel Mass Ratio,kg/kg
      exfmr=exwet*exmm/rmm
c Excess Air to Fuel Mass Ratio,kg/kg
      rexcessair=excessair*prop(ipmmass,jpair)/rmm
c Stoichiometric Exhaust to Fuel Mass Ratio, kg/kg
      sexfmr = exfmr-rexcessair

c Total Air to Fuel Mass Ratio,kg/kg
      tafmr = rmamf*prop(ipmmass,jpair)/rmm

      
c --- STRIPPED GAS STREAM
c uses an assumed temperature that is constant (ie, not a function
c of wind speed nor stripped mass flow      
      stripped_m = totalflaringrate_m * (1.-carb)*fs
c Air flow for Stripped Mass at LFL, kg/s      
      stripped_air = stripped_m*tafmr
c Stripped Mass plus air flow rate, kg/s      
      stripped_tot =  stripped_m + stripped_air
c Auto-ignition temperature of combustibles, K      
      tautoig = calc_autoig(mix)
c Assumed temperature of stripped mass, K      
      tstrip = 0.5*(tautoig + tgas)
c stripped mass cp,kJ/kmol/K
c Specific heat of stripped mass at T, kJ/(kgK)
      hf_sm = calc_hfmix(mix, tstrip) - 
     &            calc_hfmix(mix, tamb)
      smcp     = cpmass_mix
      if( hf_sm.gt. 0. .and. tstrip .ne. tamb) then
          smcp    = hf_sm/(tstrip-tamb)
      end if               
c stripped mass cp,kJ/kg/K
      smcp    = smcp/rmm
c Specific heat of air at T, kJ/(kgK)
c      smaircp= calc_cp(jpair, tstrip)/rmmair    
      hf_smair =  calc_hf(jpair, tstrip) - 
     &            calc_hf(jpair, tamb)
      smaircp= cpmass_mix
      if( hf_smair.gt. 0. .and. tstrip.ne.tamb) then
          smaircp    = hf_smair/(tstrip-tamb)
      end if               
      smaircp    = smaircp/rmmair
c Specific heat of stripped mass and air at T, kJ/(kgK)
      smtotcp=(stripped_m*smcp+stripped_air*smaircp)/stripped_tot
c Molar mass at air and fuel,kg/kmol
      rmmsm   = (rmm+rmmair*rmamf)/(1.+rmamf)
c Buoyancy Flux if unconverted and air is heated up,m4/s3
      FB_strip1   =grav*ru/(pi*plocal*rmmair)*
     &     (smtotcp/cpair_m_tamb *
     &     (tstrip-tamb)-(1.-rmmair/rmmsm)*tamb)
     &     *stripped_tot
c Stripped Gas heat     ,m4/s3
      FB_strip2  =grav*ru/(pi*plocal*rmmair)*
     &     (smcp/cpair_m_tamb*(tstrip-tamb))*stripped_m
c Air heat     , m4/s3
      FB_strip3  =grav*ru/(pi*plocal*rmmair)*
     &     (smaircp/cpair_m_tamb*(tstrip-tamb))*stripped_air
c Total heat     , m4/s3
      htot =grav*ru/(pi*plocal*rmmair)*
     &     (smtotcp/cpair_m_tamb*(tstrip-tamb))*stripped_tot
c Stripped gas and air heat     , kW
      qstrip =htot*pi*plocal*cpair_m_tamb*rmmair/grav/ru
     
      
c --- Adjusted Heating Value

c Complete combustion heat release with no losses,MJ/m
      h1 = rnhv
c In-complete combustion loss (combustion efficiency calculated),MJ/m
      h2 = -(1.-comb)*h1
c Process heat removed and heat losses before exit (0% assumed),MJ/m
      h3 = 0.
c Fuel sensible heat,MJ/m
      h4 = cpbarf*(tflared-tref)/cpv/1000.
c Air sensible heat,MJ/m
      h5 = rmamf * cpair_ave_mol *(tamb-tref)/cpv/1000.
c Radiation heat losses from combustion heat released (25% assumed),MJ/m
      h6 =-radloss*(h1+ h2)
c Heat transferred to stripped fraction,MJ/m
      h7 =-qstrip/(totalflaringrate_v*1000.*(carb+(1.-carb)*(1.-fs))) 
c Total heat to plume (sum),MJ/m
      heatp=(h1 + h2 + h3 + h4 + h5 + h6 + h7)

c Temperature with losses using air properties,K
c      t1 =heatp*cpv/rmm*1000./(exwet*rmmair/rmm)/cpair_m+(tref)
c Complete combustion heat release with no losses,K
      t2 =(rnhv/rmm*cpv)*1000./exfmr/excp_m
c Heat loss due to incomplete combustion, K
      t3 =-t2*(1.-comb)
c Fuel sensible heat,K
      t4 =fcp_m/excp_m*(tflared-tref)/exfmr
c Air sensible heat,K
      t5 =cpair_m *(tamb-tref)*tafmr/exfmr/excp_m
c Radiation heat losses from combustion heat released (25% assumed),K
      t6 =-radloss*(t2+t3)
c Heat transferred to stripped fraction, K
      t7 = t2*h7/h1
c Reference temperature,K
      t8 =tref
c Temperature with constant specific heat at 15C sum,K
      tguess   =MAX(t2+t3+t4+t5+t6+t7+t8,tamb) 

c enthalpy goal, kJ/kmol
      egoal = heatp * 1000. * cpv / exwet
c Estimated Flame Temperature with specific heat function of temperature,K
c Estimated Flame Temperature with Cp function of temperature,K
      extemp   = calc_flametemp(tguess,exhaustmix, egoal)
      if(heatp .lt. 0) then
          extemp   = tamb
      end if 
      extemp = MAX(extemp, tamb)
c tflame, flame temperature in K      
      tflame = extemp
      
c specific heat of flared gas, kJ/kmol/K
      cpflared = calc_cpmix(exhaustmix, tflame) 
      
c enthalpy of flared gas, kJ/kmol
      hf_flared = calc_hfmix(exhaustmix, tflame) - 
     &            calc_hfmix(exhaustmix, tref)
c flared gas average cp,kJ/kmol/K
      cpflared_ave = 0.
      if( hf_flared .gt. 0.) then
          cpflared_ave = hf_flared/(tflame-tref)
      end if               
c flared gas exhaust flow, kmol/s
      totalflaringrate_mol =totalflaringrate_v/cpv*exwet*
     &     (carb + (1.-carb)*(1.-fs))
c heat to plume, kW
      heat_p  = totalflaringrate_mol * egoal   


c Exhaust Molar Average Specific Heat at Estimated Temperature,kJ/(kmolK)
      excp_extemp = (calc_hfmix(exhaustmix, tflame)-
     &calc_hfmix(exhaustmix, tref))/
     &(tflame-tref)  ! calc_cpbar(exhaustmix, tflame) 
c Exhaust Mass Average Specific Heat at Estimated Temperature,kJ/(kgK)
      excp_m_extemp=excp_extemp/exmm

c
c Exhaust Molar Specific Heat at Estimated Temperature, kJ/(kmolK)
c      excpest  = calc_cpbar(exhaustmix, tflame)
c Exhaust Mass Specific Heat at Estimated Temperature,kJ/(kgK)
c      excpest_m = excpest/exmm 
c Stoichiometric Exhaust Mass Specific Heat at Estimated Temperature, kJ/(kgK)
c      sexcp_test =cpair_ave_mass*
c     &             ((excpest_m/cpair_ave_mass-1.)*exfmr/sexfmr+1.)

c
      if(tref .eq. tamb) then
          cpair_ave = calc_cp(jpair, tref)
      else
          cpair_ave = (calc_hf(jpair, tamb)-calc_hf(jpair, tref))
     &    /(tamb-tref)
      end if 
      cpair_ave_m = cpair_ave/rmmair
      
c Stoichiometric Exhaust Mass Specific Heat,kJ/(kgK) at estimated Temperature  (C293)
      sexcp_tex  = cpair_ave_m*((excp_m_extemp/cpair_ave_m-1.)*
     &            exfmr/sexfmr + 1.)

      
      
      
c --- BUOYANCY
c --- Buoyancy Flux at Site Conditions

c Flared gas with assist less stripped gas rate unit factor	m4/(s3 K)
      fruf =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m*
     &   (carb + (1.-carb)*(1.-fs))
c Combustion heat release with losses,m4/s3
c      chre =(rnhv/rmm*cpv)*max(1.-radloss,0.)/cpair_ave_m*1000.*fruf
      chre =(rnhv/rmm*cpv)*max((h1+h2+h6+h7)/h1,0.)/cpair_ave_m*1000.*fruf
c Fuel sensible heat relative to reference temperature,m4/s3
      fshre =cpbarf_m/cpair_ave_m*(tflared-tref)*fruf
c Difference between molar mass of air and source at air temperature,m4/s3
      hre1  =sexfmr*(rmmair/sexmm-1.)*tamb*fruf
c Difference between specific heat of air and source,m4/s3
      hre2  =-sexfmr*(sexcp_tex/cpair_ave_m-1.)*(tamb-tref)*fruf
c Difference between ambient and reference temperature,m4/s3
      hre3  =-(tamb-tref)*fruf
c --- "Buoyancy Flux with constant specific heat at 15C sum (RIGOROUS USED)",m4/s3
      FB_tref    =chre + fshre + hre1 + hre2 + hre3
c --- use FB below at estimated temperature
      FB_tref    =MAX(FB_tref,0.)

c --- "Alternate Buoyancy Flux, FB infinity, based on estimated temperature and properties",m4/s3
      FB_tex =grav*ru/(rmmair*pi*plocal)*totalflaringrate_m*exfmr*
     &         (excp_m_extemp/cpair_ave_m*tflame-
     &         Tamb*(1.-rmmair/exmm+excp_m_extemp/cpair_ave_m))
     &         *(carb + (1.-carb)*(1.-fs))
      FB  =MAX(FB_tex,0.)

c --- Alternate Buoyancy Flux based on simple heat release rate to plume,m4/s3
      rk3      = grav*ru/
     &                (rmmair*pi*plocal*(cpair_tref/rmmair*1000.))
      FB_shre =rk3*rnetheatreleased_c*carb*(1.-radloss)

c Buoyancy Flux at reference pressure (previously used),m4/s3
      FB_old   = 0.000037*rnetheatreleased_a*(effmin-radloss)
      FB_old  =max(FB_old,0.)   

c Steam Assisted buoyancy flux      
      FB_stm =grav*ru/(rmmair*pi*plocal)*stm_m*
     & (cpstm_ave_mass/cpair_m_tamb*tstm-
     & Tamb*(1.-rmmair/rmmstm+cpstm_ave_mass/cpair_m_tamb))
c Air Assisted buoyancy flux      
      FB_air =grav*ru/(rmmair*pi*plocal)*air_m*
     & (cpair_ave_mass/cpair_m_tamb*tamb-
     & tamb*(1.-rmmair/rmmair+cpair_ave_mass/cpair_m_tamb))
      
      
c Adjustment factor for minimum fuel gas,previous / current
c Sensible Heat Release Rate to Rise (ES) for Best Fit,cal/s
      ES =FB/0.000037

c     Source Parameters for Ambient Wind Speed
      call brzflare(mix, dia, ue, ws, den_flare, den_air, XL, zL, FA)

c Vertical velocity based on flared gas momentum,m/s
      uv = ue/exfmr
c Horizontal velocity based on average wind speed momentum,m/s
      uu = tafmr*ws/exfmr
c Angle of plume from horizontal,degrees
      dangle = ATAN2(uv, uu)
      plumeangle =DEGREES(dangle)
c Total velocity,m/s
      us = max(sqrt(uv*uv + uu*uu),0.001)
c Exhaust flow at Pamb and Texh,m/s
      exrate   =exrate_amb*tflame/tamb
c Air required at Pamb and Tamb, m/s
c      rair     =exrate*rmamf/exwet
c Diameter of plume,m
      dia2 =sqrt(4.*exrate/(pi*us))
c Estimated source temperature,K
c      = extemp
c Converted SO2,ppm wet
      conv_so2 =exhaustmix(jpso2)*1000000.*carb
c Residual so2 or trs or H2S, if mixed in plume,ppm wet
      resid_h2s = exhaustmix(jpso2)*1000000.*(1.-carb) 
c     &            *prop(ipmmass,jph2s)/prop(ipmmass,jpso2)
c Conversion In-efficiency,
      eta=(resid_h2s/(conv_so2+resid_h2s))
c Downwind distance from flare tip,m
c      xf = XL
c Elevation  above flare tip,m
      hf = max(hs+2.*min(dia*(ue/ws-1.5),0.)+zL,0.)-hs

      if(debug .gt. 0) then
         write(iolog,*) 'XL=',XL
         write(iolog,*) 'ZL=',ZL
         write(iolog,*) 'FA=',FA
         write(iolog,*) 'uu,uv=',uu,uv
         write(iolog,*) 'plume angle (deg)=',plumeangle
         write(iolog,*) 'exhaust rate (m3/s)=',exrate
      end if 
      
c --- PSEUDO-STACK PARAMETERS
      if( MSTRIP .eq. 0 .or. MSTRIP.eq.2) then
c set output emissions
          if( MSTRIP.eq.0 ) then
              qemit = e_so2_m
c calculate actual SO2 emitted using combustion efficiency
          else if( MSTRIP.eq.2 ) then
              e_so2_ma= e_so2_m *(carb)
              qemit   = e_so2_ma
          end if 

          tdiff = tflame-tamb
c Effective Pseudo Stack Height (hS) Adjusted for Flare Tip Downwash,m
c note spreadsheet uses ws_max for ws here, ws is hourly
          hs_x = max(hs+2.*min(dia*(ue/ws-1.5),0.)+zL,0.)
c Pseudo Stack Gas Exit Velocity (VE),m/s
          ue_x =min(max(ue,0.1),max(grav*FM/FB*tdiff/tamb,0.1))
c Pseudo Stack Inside Diameter (DE),m
          de_x =2.*sqrt(FB*tflame/(tdiff*grav*ue_x))
c Estimated Stack Gas Exit Temperature (TE),K
c      =C363
c Assumed Ambient Air Temperature (TA),K
c      =tamb+rk0
c Correct Momentum Flux,m4/s2
          FM_check = 0.25*tamb/tflame *ue_x*ue_x*de_x*de_x
c Correct  Buoyancy Flux (FB infinity) at site conditions,m4/s3
          FB_check = 0.25*grav*tdiff/tflame*ue_x*de_x*de_x

c --- STRIPPED GAS STREAM -----------------------------------------------
c --- specify stripped gases source parameters
      else if( MSTRIP .eq. 1) then
c Stripped gas source temperature, K
         ts = tstrip
c Stripped gas source temperature, K
         extemp = tstrip 

c These are re-adjusted for site conditions below      
c Buoyancy Flux Simplified Method	m4/s3
         FB_strip = FB_strip1   
c Momentum Flux	m4/s2
         FM_strip = -FM_strip

c set temperature equal to ambient
c      tambk   = tamb
         tdiff   = tstrip-tamb
      
c Pseudo Stack Gas Exit Velocity (VE)	m/s
         ue_x = ue
         if(FB_strip.ne.0) then
            ue_x = grav*FM_strip/FB_strip*tdiff/tamb
         end if   
         ue_x =max(min(ue,ue_x),0.1)
      
c Effective Pseudo Stack Height (hS) Adjusted for Flare Tip Downwash	m
         hs_x = max(hs+2.*min(dia*(ue/ws-1.5),0.)+zL*0.5,0.)
      
c Pseudo Stack Inside Diameter (DE)	m
         if(FB_strip.eq.0. .or. tstrip .eq. tamb) then
            de_x = dia
         else
            de_x = 2.*sqrt(FB_strip1*tstrip/(tdiff*grav*ue_x))
         end if 

c calculate H2S emitted using combustion efficiency
          qemit = e_h2s

          FB = 0.25*grav*tdiff/tstrip*ue_x*de_x*de_x
          FM = 0.25*tamb/tstrip*ue_x*ue_x*de_x*de_x
      end if

      if( FM .ne. 0. .and. MSTRIP.eq.0) then
          if( abs((FM_check-FM)/FM) .gt. 0.00001 ) then
c     CALCULATION ERROR
              write(6,*) '*** ERROR *** bad calc in FM'
              write(6,*) 'FM_check :', FM_check
              write(6,*) '      FM :', FM
          end if 
      end if    
      if( FB .ne. 0. .and. MSTRIP.eq.0) then
          if( abs((FB_check-FB)/FB) .gt. 0.00001 ) then
c     CALCULATION ERROR
              write(6,*) '*** ERROR *** bad calc in FB'
              write(6,*) 'FB_check :', FB_check
              write(6,*) '      FB :', FB
          end if 
      end if    
      return
      end                               

c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c Carbon efficiency
c from UofA Flare Research Project 
c (note molar efficiency same as mass efficiency for C and S)
c     
c     common :  rk13, grav
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
c
      function carbeff(rnhv, ws, dia, ue  )
      include 'h_stddefs.for'
c INPTUS
      real(8)  :: rnhv  ! net heating value of flared gas, MJ/kg
      real(8)  :: ws    ! m/s, wind speed at reference height
      real(8)  :: dia   ! m, flare stack tip inside diameter
      real(8)  :: ue    ! m/s, maximum exit speed at flare tip
      real(8)  :: carbeff 
c DECLARATIONS

c  Maximum Possible Conversion Efficiency 
      carbeff= (uofaa/rnhv**3.)
      carbeff= carbeff*exp(uofab*ws/(grav*ue*dia)**(rk13))
      carbeff = 1. - carbeff
      carbeff= max(0., carbeff )
      return
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c calc_plocal
c
c COMMON
c     pref; rlapse; tref;rk0;grav;ru;
c     plocal      
c Calculate site pressure
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_plocal(elev)
      include 'h_stddefs.for'
c INPUTS
      real(8):: elev
      real(8):: calc_plocal
c DECLARATIONS

      calc_plocal = (1.-rlapse*elev/tref)
      calc_plocal = calc_plocal**
     &(grav/(ru*1000.)*prop(ipmmass,jpair)/rlapse)
      calc_plocal = calc_plocal *pref
c spreadsheet rounds to 3
c ROUND(psite,3)
      return 
      end


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c calc_trad()
c Calculate the thermal radiation
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_trad(rate, rnhv, hs, zL)
      include 'h_stddefs.for'
c INPUTS
      real(8) :: rate   ! flaring rate , m3/s
      real(8) :: rnhv   ! net heating value, MJ/m3
      real(8) :: hs     ! effective stack height, m
      real(8) :: zL     ! Brzustowski Flare Model, height, m
      real(8) :: calc_trad      
c DECLARATIONS
      real(8) :: heatreleased
      real(8) :: hheat
      real(8) :: radgnd

c total heat released
      heatreleased = rate*rnhv*radloss *1000000./1000.   ! with 25% loss
c Fireball Height at half of flame
      hheat        = hs -zL*(1.-0.82)
c radiation at the ground
      radgnd       = heatreleased /(4.*pi*hheat*hheat)
      calc_trad    = radgnd + tradbkg      
      return 
      end
c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
c Brzustowski Flare Model
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      subroutine brzflare(mix, dia, uexit, ws, rhoe, rhoa, XL, zL, FA)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(*)
      real(8)  :: dia   ! exit diamter,m
      real(8)  :: uexit ! exit velocity, m/s
      real(8)  :: ws    ! windspeed at stack top, m/s
      real(8)  :: rhoe  ! release density at ambient temperature & local pressure
      real(8)  :: rhoa  ! air density
      real(8)  :: XL
      real(8)  :: ZL
      real(8)  :: FA
c DECLARATIONS
c      real(8)  :: rLFL
      interface
      function calc_byprop(mix, inprop, incalc, inprop2)
         real(8) :: calc_byprop      
         real(8) :: mix(*)
         integer :: inprop
         integer :: incalc
         integer,optional :: inprop2
      end function
      end interface

c combustible fraction of mixture
      combfrac = calc_byprop(mix, ipcomb, ipcalc) 
c Lower Flammability Limit (LFL) of combustible flared gas
      rLFLcomb    = calc_lfl(mix)
c Lower Flammability Limit (LFL) of flared mixture
      rLFLflared  = 0.
      if(combfrac .gt.0.)then
          rLFLflared  = rLFLcomb/combfrac
      end if 
      rmm  = calc_molarmass(mix)       ! kg/kmole
      rmmair =  prop(ipmmass,jpair)  ! calc_molarmass(gcair) very similar to  ! kg/kmole
      rnhv = calc_nhv(mix)             ! MJ/m3

c Vj/Ua - Jet to Air velocity ratio 
      Vj_Ua = min(110.,uexit/ws)
c R - Jet to Air momentum flux ratio
      rjet = rhoe/rhoa*Vj_Ua*Vj_Ua
c CL - dimensionless lean limit concentration of fuel
      CL   = rLFLflared *Vj_Ua*(rmm/rmmair)
c SL - dimensionless coordinate along flame
      IF(CL .lt. 0.5) then 
          SL = 2.04*(CL**-1.03)
      else   
          SL = 2.51*(CL**-0.625)
      end if   
c XL - dimensionless horizontal coordinate along flame
      IF(CL .lt. 0.5) then
          XLL=SL-1.65
      else
          IF(SL .gt. 2.35) then
              XLL=SL-1.65
          else
              xguess = (SL/2.05)**(1/0.28)
              XLL=calc_XLL(xguess, SL)
          end if
      end if
c ZL - dimensionless vertical coordinate along flame
      ZLL   = 2.05*XLL**0.28
c zL - vertical coordinate upward from flare tip (used)
      ZL    = ZLL*dia*sqrt(rjet)
c xL - horizontal coordinate downwind from flare tip
      XL    = XLL*dia*sqrt(rjet)
c Angle from flare tip to flame tip, from horizontal
      FA    = DEGREES(ATAN2(ZL,XL))
      return
      end

c----------------------------------------------------------------------
c calc_XLL
c B.Zelt - www.zeltpsi.com
c
c
c
c --- UPDATES
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_XLL(xinit, SL)
c INPUTS
      real(8) :: xinit
      real(8) :: SL
      real(8) :: calc_XLL 
c DECLARATIONS
      real(8) :: xguess 
      real(8) :: diff
      real(8) :: slope  

      xguess = xinit
      slope = 2.*1.04*xguess + 0.28*2.05*xguess**(0.28-1.)

      maxloops = 100
      do i=1,maxloops
          SLnew  = 1.04*xguess*xguess + 2.05*xguess**0.28
          slope  = 2.*1.04*xguess + 0.28*2.05*xguess**(0.28-1.)
          diff   = SLnew-SL
          xguess = xguess-diff/slope
          if (abs(diff) .lt. 0.00001) Then 
              goto 100
          end if
      end do 

c 100  xguess = (eGoal - Intercept) / slope
 100  calc_XLL = xguess
      return 
      end

c----------------------------------------------------------------------
c calc_flametemp(tguess,mix, nhv, exit_fuel_ratio)
c B.Zelt - www.zeltpsi.com
c 
c MIX CONTAINS
c air
c h2o
c o2
c N2
c CO2
c SO2
c
c --- UPDATES
c     131128 - change tref default to K
c
c --- INPUTS
c
c --- OUTPUTS
c
c --- RETURNS
c----------------------------------------------------------------------
      function calc_flametemp(tinit, mix, eGoal)
      include 'h_stddefs.for'
c INPUTS
      real(8)  :: mix(mxcomp)
      real(8) :: tinit
      real(8) :: eGoal
      real(8)  :: calc_flametemp
c DECLARATIONS
      real(8) :: tguess
      real(8) :: Intercept, difference, cp, enthalpy_tg
      real(8) :: enthalpy_tr
      integer :: i, maxloops
      real(8) :: a(7), b(7)  ! thermo properties for CP lo and hi temp

      interface
      function calc_cpbar(mix,tempk,inprop2)
         real(8) :: mix(*)
         real(8) :: tempk
         real(8) :: calc_cpbar      
         integer,optional :: inprop2
      end function         
      end interface

c Flame temperature is approximately 1000 C
      tguess = tinit

c calc MIX
      do i=1,7
          a(i) = 0.
          b(i) = 0.
      end do
      do j=1,mxcomp
          do i=1,5
              a(i) = a(i) + prop(ipa1+i-1,j) * mix(j) * ru
              b(i) = b(i) + prop(ipb1+i-1,j) * mix(j) * ru
          end do
      end do 
      a(6) = 0.0
      b(6) = 0.0

      cp_ref= (a(1)+
     &tref*(a(2)+
     &tref*(a(3)+
     &tref*(a(4)+
     &tref*a(5) ))))
      acorr = calc_cpbar(mix,tref) - cp_ref

      tempK = tguess
      if( tempK .gt. 1000.) then
          tempK = 1000.
          cp_lo= (a(1)+ acorr+
     &    tempk*(a(2)+
     &    tempk*(a(3)+
     &    tempk*(a(4)+
     &    tempk*a(5) ))))

          cp_hi= (b(1)+
     &    tempk*(b(2)+
     &    tempk*(b(3)+
     &    tempk*(b(4)+
     &    tempk*b(5) ))))
          bcorr = (cp_lo - cp_hi)
c calculate b6 correction
          hf_lo= 0.
          hf_hi= 0.
          do i=5,1,-1
              hf_lo= tempK *(a(i)/float(i) + hf_lo)
              hf_hi= tempK *(b(i)/float(i) + hf_hi)
          end do
          hf_lo = hf_lo + acorr*tempK*ru
          hf_hi = hf_hi + bcorr*tempK*ru
          b(6) = (hf_lo-hf_hi)

      end if
      a(1) = a(1) + acorr
      b(1) = b(1) + bcorr

c use the lower a coefficients for enthalpy_tr
c use b coeeficients for enthalpy_tg
      tempK = tguess
      if(tempK .le. 1000.) then
          do i=1,6
              b(i) = a(i)
          end do
      end if 

c CP at tguess
      cp          = 0.
      enthalpy_tg = 0.
      enthalpy_tr = 0.
      do i=5,2,-1
          cp          = tguess*(b(i) + cp)
      end do
      cp = cp + b(1)
      do i=5,1,-1
          enthalpy_tg = tguess*(b(i)/float(i) + enthalpy_tg)
          enthalpy_tr =  tref*(a(i)/float(i) + enthalpy_tr)
      end do
      enthalpy_tg = enthalpy_tg + b(6)
      enthalpy_tr = enthalpy_tr + a(6)

      difference = enthalpy_tg - enthalpy_tr
      Intercept = difference - cp * tguess

      maxloops = 10000
      do j=1,maxloops
          tguess = (eGoal - Intercept) / cp

          cp          = 0.
          enthalpy_tg = 0.
          do i=5,2,-1
              cp          = tguess*(b(i) + cp)
          end do
          cp = cp + a(1)
          do i=5,1,-1
              enthalpy_tg = tguess*(b(i)/i + enthalpy_tg)
          end do
          enthalpy_tg = enthalpy_tg + b(6)

          enthalpy_tg = enthalpy_tg - enthalpy_tr
          Intercept = enthalpy_tg - cp * tguess
          difference = enthalpy_tg - eGoal
          if (abs(difference) .lt. 0.1) Then 
              goto 100
          end if
      end do 

 100  tguess = (eGoal - Intercept) / cp
      calc_flametemp = tguess
      return
      end 


c----------------------------------------------------------------------
c B.Zelt - www.zeltpsi.com
c
      function iwrFLEMARBhdr(io,iodb,FIN)
c----------------------------------------------------------------------
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  writes the header for the FLEMARB.DAT
c               arbitrary data file
c
c --- UPDATES:
c --- INPUT:
cio        - integer    - io unit
ciodb        - integer    - io debug unit
cFIN       - characeter - path for file output
csname     - character  - source name
ciutmzn     - integer    - utm zone  11,12 are ALBERTA
cABTZ    - character  - time zone in UTC format
cUTMHEM      - character  - N or S, for hemisphere
cDATUM    - character  - projection used for data
cUNITS     - character - m or km used to reference location
cPMAP
cnspec     - integer    - number of chemical species emitted
cchemlst() - character  - names of the chemicals emitted
cmmlst()   - real       - molar weights of chemicals emitted
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      include 'h_stddefs.for'
      include 'h_read1.for'
c INPUTS
      CHARACTER *128 :: FIN
      CHARACTER *8 :: dummy
      integer :: io
      integer :: iodb
      integer :: iwrFLEMARBhdr
c DECLARATIONS
      integer :: ier
      real(8)  :: params(8)
      data params/1.,0.,0.,0.,0.,0.,0.,0./

      open(unit=io,file=FIN,iostat=ier,status='unknown')
      if(ier .ne. 0) goto 9000

      nsrc   = 1              ! number of sources 

      write(io,101) 
 101  format('FLEMARB.DAT     1.0             Initial configuration')
      ncom = 7
      write(io,102) ncom
 102  format(i4)
      write(io,*) "Time-invariant (8 variables) var1=stack 
     &input type  var2-var8=not used"
      write(io,*) "     var1=1:  T(K), U, V, W (m/s)"
      write(io,*) "     var1=2:  Fb, Fmx, Fmy, Fmz  
     &(i.e., buoyancy and momentum fluxes)"
      write(io,*) "Time-variant (if var1=1)"
      write(io,*) "  X(km),Y(km),Z(mAGL),Elev(mMSL),
     &D(m),T(K),U,V,W,Sigy(m),Sigz(m),Q(g/s)..."
      write(io,*) "Time-variant (if var1=2)"
      write(io,*) "  X(km),Y(km),Z(mAGL),Elev(mMSL),
     &D(m),Fb,Fmx,Fmy,Fmz,Sigy(m),Sigz(m),Q(g/s)..."
      write(io,103) PMAP
 103  format(A3)
      write(io,104) iutmzn, UTMHEM
 104  format(i4,A1)
c      NAS-C   02-21-2003  
      write(io,105) DATUM
 105  format(A5)      
      if( MUNITS.eq.1 ) then
          write(io,106) '   M'
      else
          write(io,106) '  KM'
      end if 
 106  format(a4)     
      write(io,107) ABTZ
 107  format(a8)     

c 1990 009  04  0000  1990 009  10 0000
c    1   5
c 'SO2'   'NO'  'NO2'  'PM10'  'H2S'
c       64.000      30.000      46.000      30.000      34.000      
c 'Flare1'      1.0    0.0    0.0    0.0    0.0    0.0    0.0    0.0
      call julday(iodb,iyys,imms,idds,jjds)
      call julday(iodb,iyye,imme,idde,jjde)

      write(io,108) iyys,jjds,ihhs,isss, iyye,jjde,ihhe,isse
 108  format(3i4,1x,i4,2x,3i4,1x,i4)
      write(io,109) nsrc, nspec
 109  format(2i4)

c 110    format(("'",A,"'",3x))
      do i=1,nspec
          dummy = chemlst(i)
          n = LEN_REAL(dummy)
          write(io,110) chemlst(i)(1:n)
      end do

      write(io,*)

 110  format(("'",A,"'   "),$)


      write(io,111) (mmlst(i),i=1,nspec)
 111  format(10F12.3)
      
      n = LEN_REAL(sname)
      write(io,112) sname(1:n), (params(i),i=1,8)
 112  format("'",A,"'",8F7.1)       
      iwrFLEMARBhdr  = 0

      return 
 9000 iwrFLEMARBhdr = ier
      return 
      end

c----------------------------------------------------------------------
c     iwrFLEMARBi
c     
c --- Version: 1.001.0001    Level: 110706
c     B.Zelt - www.zeltpsi.com
c
c --- PURPOSE:  
c   
c  Time-invariant (8 variables) var1=stack input type  var2-var8=not used
c       var1=1:  T(K), U, V, W (m/s)
c       var1=2:  Fb, Fmx, Fmy, Fmz  (i.e., buoyancy and momentum fluxes)
c  Time-variant (if ivar1=1)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), T(K), U, V, W, Sigy(m), Sigz(m), Q(g/s)...
c  Time-variant (if ivar1=2)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), Fb, Fmx, Fmy, Fmz, Sigy(m), Sigz(m), Q(g/s)...
c            
c            
c
c --- UPDATES:
c --- INPUT:
c           iy1,ij1,ih1,is1,iy2,ij2,ih2,is2
c           x
c           y
c           z
c           elev
c           ds
c           ts
c           us
c           vs
c           ws
c           fb
c           fmx
c           fmy
c           fmz
c           sy
c           sz
c           q
c           nchem  (integer) :: number of chemical species emitted
c --- OUTPUT:
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function iwrFLEMARBi(io,ivar,sname,
     &iy1,ij1,ih1,is1,iy2,ij2,ih2,is2,
     &x,y,z,elev,ds,ts,us,vs,ws,
     &fb,fmx,fmy,fmz,sy,sz,q,nchem)
      include 'h_stddefs.for'
c INPUTS
      integer :: io
      integer :: ivar
      character*32 :: sname
      real(8) :: x,y,z,elev,ds,ts,us,vs,ws
      real(8) :: fb,fmx,fmy,fmz,sy,sz
      real(8)  :: q(mxchem)
      integer  :: nchem
c DECLARATIONS
      iwrFLEMARBi = 0

c use 0-23 clock
      write(io,111) iy1,ij1,ih1,is1,iy2,ij2,ih2,is2
 111  format(2(3x,i04,1x,i03,1x,i02,1x,i04))
c   1990  009  04 0000    1990  009  05 0000
c'Flare1'  340.011  4870.443   92.513  160.4 40.33 1122.17 0.0   0.0   0.725 0.0   0.0  9746.0  0.0  0.0  0.0  105.81

c  Time-invariant (8 variables) var1=stack input type  var2-var8=not used
c       var1=1:  T(K), U, V, W (m/s)
c       var1=2:  Fb, Fmx, Fmy, Fmz  (i.e., buoyancy and momentum fluxes)
      if ( ivar .eq. 1) then
c  Time-variant (if ivar1=1)
c       X(km), Y(km), Z(mAGL), Elev(mMSL), D(m), T(K), U, V, W, Sigy(m), Sigz(m), Q(g/s)...
          n = LEN_REAL(sname)
          if( MUNITS.eq.1 ) then
              write(io,112) sname(1:n),x,y,
     &        z,elev,ds,ts,us,vs,ws,sy,sz,
     &        (q(i),i=1,nchem)
          else
              write(io,114) sname(1:n),x,y,
     &        z,elev,ds,ts,us,vs,ws,sy,sz,
     &        (q(i),i=1,nchem)
          end if
 112      format("'",A,"'",2(1x,F10.2),1x,F10.3,1x,F10.3,1x,F11.3,1x,
     &    F7.1,3(1x,F7.3),2(1x,F15.3),
     &    10(1x,G15.8))
 114      format("'",A,"'",2(1x,F10.5),1x,F10.3,1x,F10.3,1x,F11.3,1x,
     &    F7.1,3(1x,F7.3),2(1x,F15.3),
     &    10(1x,G15.8))

      else if (ivar .eq. 2) then
c  Time-variant (if ivar1=2)
c       X(km), Y(km), hs(mAGL), Elev(mMSL), D(m), Fb, Fmx, Fmy, Fmz, Sigy(m), Sigz(m), Q(g/s)...
          n = LEN_REAL(sname)
          if( MUNITS.eq.1 ) then
              write(io,113) sname(1:n),x,y,
     &        z,elev,ds,fb,fmx,fmy,fmz,sy,sz,
     &        (q(i),i=1,nchem)
          else
              write(io,115) sname(1:n),x,y,
     &        z,elev,ds,fb,fmx,fmy,fmz,sy,sz,
     &        (q(i),i=1,nchem)
          end if 
 113      format("'",A,"'",2(1x,F10.2),1x,F10.3,1x,F10.3,4(1x,F15.3),
     &    2(1x,F15.3),10(1x,G15.8))
 115      format("'",A,"'",2(1x,F10.5),1x,F10.3,1x,F10.3,4(1x,F15.3),
     &    2(1x,F15.3),10(1x,G15.8))

      else
c  write error 
          write(6,*) 
     &    ' *** BAD PARAMETER *** in iwrFLEMARBi, ivar must be 1 or 2'
          stop
      end if
      iwrFLEMARBi = 1
      return 
      end

      subroutine calcDELH(Ua, Ta, Kst, Ts, Vs, Ds, Hs, XH, DELH, zref)
c***********************************************************************
c                 DELH Module of ISC2 Model
c
c        PURPOSE: To Calculate Final Plume Rise
c
c        PROGRAMMER: Roger Brode, Jeff Wang
c                    updates, B.Zelt - www.zeltpsi.com
c
c Date:           March 2, 1992
c
c        MODIFIED:   To use calling argument
c                    R. W. Brode, PES, Inc. - 9/30/94
c
c                    B.Zelt
c                    Sept, 14, 2011 
c                    To calculate array of gradual rise heights 
c                    to final rise
c        OUTPUTS: Final Plume Rise, DHFOUT (m)
c***********************************************************************
      include 'h_isccalcs.for'
      real(8) :: Ua        ! amb wind speed (m/s)
      real(8) :: Ta        ! amb temperature (k)
      integer :: Kst    ! PG stability class
      real(8) :: Ts        ! source temperature (K)
      real(8) :: Vs        ! source exit velocity (m/s)
      real(8) :: Ds        ! source diameter (m)
      real(8) :: Hs        ! source stack height (m)
      real(8) :: XH(5)        ! distance to final rise (m)
      real(8) :: DELH(5)      ! plume rise (m)
      real(8) :: zref      ! reference windspeed height (m)
c --- DECLARATIONS
      real(8) :: TS1       ! source temperature
      real(8) :: Uref      ! amb wind speed (m/s) at reference height
      real(8) :: Us        ! amb wind speed (m/s) at stack tip
      real(8) :: DtDz      ! potental temperature gradient (K/m)
      real(8) :: PI
      real(8) :: ptg0(2)

c --- ISC defaults for array of potential temperatures (2)
      data ptg0/0.020,0.035/
      data PI/3.141593/

c     Variable Declarations
c      INCLUDE 'MAIN1.INC'
c      INCLUDE 'MAIN2.INC'
c      INCLUDE 'MAIN3.INC'


c ---       Use default potential temperature lapse rate with the
c ---       surface temperature from the nearest station
      if(KST.LE.4) then
          dtdz=0.0
      elseif(KST.EQ.5) then
          dtdz=ptg0(1)
      else
          dtdz=ptg0(2)
      endif

c      Zref = 10.  !m, reference windspeed height
      Uref = Ua   !m/s, wind speed
      G = 9.80665 !m/s2

      If (DtDz .gt. 0. .and. Ta .gt. 0.) Then
          S = G * DtDz / Ta
          RTOFS = sqrt(S)
      Else
          S = 0.0000000001
          RTOFS = 0.0000000001
      End If


c --- Set Stability Category Logical Flags and Adjust for KST > 6
      UNSTAB = .false.
      NEUTRL = .false.
      STABLE = .false.
      If (Kst .gt. 6) Then Kst = 6
      If (Kst .lt. 4) Then
         UNSTAB = .true. 
      ElseIf (Kst .eq. 4) Then
         NEUTRL = .true. 
      ElseIf (Kst .gt. 4) Then
         STABLE = .true. 
      End If

      Ts1 = Ts  
      If (Ts1 .lt. Ta) Then Ts1 = Ta
         FBISC = (0.25 / Ts1) * (Vs * Ds * Ds) * G * (Ts1 - Ta)
         FMISC = (0.25 / Ts1) * (Vs * Ds * Ds) * Vs * Ta

c Wind speed is entered at stack tip height, so 
c don't need to adjust
c              US = Uref
c --- Adjust Wind Speed -- Assume Wind Speed Constant Below 10 meters
      call wsstkht(uref, zref, kst, hs, us)      

c --- Do Not Allow Stack Height Wind Speed < 1.0 m/s
      If (US .lt. 1.) Then
         US = 1.
      End If

c --- Calculate Delta-T
      DELT = Ts1 - Ta

      If(UNSTAB .or. NEUTRL) Then
         If (FBISC .gt. 55.) Then
            DTCRIT = 0.00575 * Ts1 * ((Vs * Vs) / Ds)**0.333333
         Else
            DTCRIT = 0.0297 * Ts1 * (Vs / (Ds * Ds))**0.333333
         End If
         If (DELT .ge. DTCRIT) Then
            If (FBISC .ge. 55.) Then
               DHFOUT = 38.71 * (FBISC**0.6) / US
            Else
               DHFOUT = 21.425 * (FBISC**0.75) / US
            End If
         Else
            DHFOUT = 3. * Ds * Vs / US
         End If

      ElseIf (STABLE) Then
        DTCRIT = 0.019582 * Vs * Ta * RTOFS
        If (DELT .ge. DTCRIT) Then
            DHFOUT = 2.6 * (FBISC / (US * S))**0.333333
c            Compare to Final Plume Rise for Calm Winds, DHCLM
          DHCLM = 4. * (FBISC**0.25) / (S**0.375)
          If (DHCLM .lt. DHFOUT) Then DHFOUT = DHCLM
          Else
            DHFOUT = 1.5 * (FMISC / (US * RTOFS))**0.333333
c           Compare to Maximum Momentum Rise for UNSTABLE/NEUTRAL, DHCHK
              DHCHK = 3. * Ds * Vs / US
          If (DHCHK .lt. DHFOUT) Then DHFOUT = DHCHK
          End If
      End If

C***********************************************************************
C        DISTF Module of ISC2 Short Term Model - ISCST2
C        PURPOSE: Calculates Distance to Final Plume Rise
C        PROGRAMMER: Roger Brode, Jeff Wang
C        DATE:    March 2, 1992
C        OUTPUTS: Distance to Final Plume Rise, XF (m), and Distance
C                 to Final Buoyant Rise (XFB) and Final Momentum Rise (XFM)
C***********************************************************************
      IF (UNSTAB .OR. NEUTRL) THEN
          IF (FBISC .GE. 55.) THEN
              XFB = 119. * FBISC**0.4
          ELSE IF (FBISC .GT. 0.) THEN
              XFB = 49. * FBISC**0.625
          ELSE
              XFB = 4.*DS*(VS+3.*US)*(VS+3.*US)/(VS*US)
          END IF
          XFM = 4.*DS*(VS+3.*US)*(VS+3.*US)/(VS*US)
          XF = XFB
          if(XFM.gt.XFB) then
              XF = XFM
          end if 
      ELSE IF (STABLE) THEN
          XFB = 2.0715*US/RTOFS
          XFM = 0.5*PI*US/RTOFS
          XF = XFB
          if(XFM.gt.XFB) then
              XF = XFM
          end if 
      END IF

      XH(3)   = XF*.333
      XH(4)   = XF*.667
      XH(5)   = XF

      XARG = XH(3)
      call calcDHPHS(XARG,DHFOUT,US,VS,DELH(3))
      XARG = XH(4)
      call calcDHPHS(XARG,DHFOUT,US,VS,DELH(4))

c add offset of flare flame tip location      
      XH(3) = XH(3) + XH(2)
      XH(4) = XH(4) + XH(2)
      XH(5) = XH(5) + XH(2)
c add offset of flare flame tip height
      DELH(3) = HS + DELH(3)
      DELH(4) = HS + DELH(4)
      DELH(5) = HS + DHFOUT

      return 
      end 


                  SUBROUTINE calcDHPHS(XARG,DHFINAL,US,VS,DHPOUT)
C***********************************************************************
C                 DHPHS Module of ISC2 Model
C
C        PURPOSE: Calculates Distance-dependent Plume Rise for
C                 Huber-Snyder Downwash Algorithm and for BID
C
C        PROGRAMMER: Roger Brode, Jeff Wang
C
C        DATE:    March 2, 1992
C
C        MODIFIED:   To use calling arguments
C                    R. W. Brode, PES, Inc. - 9/30/94
C
C        INPUTS:  Arrays of Source Parameters
C                 Buoyancy and Momentum Fluxes
C                 Meteorological Variables for One Hour
C                 Wind Speed Adjusted to Stack Height
C                 Downwind Distance
C
C        OUTPUTS: Distance-dependent Plume Rise, DHPOUT (m)
C
C        CALLED FROM:   PHEFF
C                       BID
C***********************************************************************
      include 'h_isccalcs.for'
c --- INPUTS
      real(8) :: XARG        ! distance (m)
      real(8) :: DHFINAL     ! final rise height (m)
      real(8) :: US          ! windspeed at stack tip height (m/s)
      real(8) :: VS          ! source vertical speed (m/s)
      real(8) :: DHPOUT      ! gradual rise at distacnce XARG
C     Variable Declarations
c      USE MAIN1
c      IMPLICIT NONE
c      CHARACTER MODNAM*12
c      SAVE

c --- Declarations
      real(8) :: XP

C     Variable Initializations
c      MODNAM = 'DHPHS'

C     Calculate Distance-dependent Buoyant Plume Rise (Eqn. 1-22)
      XP = MIN(XARG,XFB)
      IF (XP .LT. 1.0)  XP = 1.0
      IF (FBISC .LT. 1.0E-10)  FBISC = 1.0E-10
      DHPB = 1.60 * (FBISC*XP*XP)**0.333333 / US

C     Calculate Dist-dependent Momentum Plume Rise          ---   CALL DHPMOM
      CALL calcDHPMOM(XARG,US,VS,DHPM)

C     Select Maximum of Buoyant or Momentum Rise for Gradual Rise
      DHPOUT = MAX(DHPB, DHPM)
C     Compare to Final Rise and Select Smaller Value for Gradual Rise
      DHPOUT = MIN(DHPOUT, DHFINAL)

      RETURN
      END

      SUBROUTINE calcDHPMOM(XARG,US,VS,DHPM)
C***********************************************************************
C                 DHPMOM Module of ISC2 Model
C
C        PURPOSE: Calculates Distance-dependent Momentum Plume Rise
C
C        PROGRAMMER: Roger Brode, Jeff Wang
C
C        DATE:    March 2, 1992
C
C        INPUTS:  Arrays of Source Parameters
C                 Buoyancy and Momentum Fluxes
C                 Meteorological Variables for One Hour
C                 Wind Speed Adjusted to Stack Height
C                 Downwind Distance
C
C        OUTPUTS: Distance-dependent Momentum Plume Rise, DHPM (m)
C
C        CALLED FROM:   WAKFLG
C                       DHPHS
C***********************************************************************
      include 'h_isccalcs.for'
C     Variable Declarations
c      USE MAIN1
c      IMPLICIT NONE
c      CHARACTER MODNAM*12
c      SAVE
      real(8) :: XARG, XP, BETAJ
C     Variable Initializations
c      MODNAM = 'DHPMOM'

C     Calculate BETAJ Parameter (Entrainment Coefficient)
      BETAJ = 0.333333 + US/VS

      IF (UNSTAB .OR. NEUTRL) THEN
          XP = MIN(XARG,XFM)
          DHPM = (3.*FMISC*XP/(BETAJ*BETAJ*US*US))**0.333333
      ELSE IF (STABLE) THEN
          XP = MIN(XARG,XFM)
          DHPM = 3.*FMISC*SIN(RTOFS*XP/US) / (BETAJ*BETAJ*US*RTOFS)
C        Set Lower Limit for DHPM to Avoid Negative Arg for Cube Root
          DHPM = MAX(1.0E-10, DHPM)
          DHPM = DHPM ** 0.333333
      END IF

C     Do Not Let Gradual Rise Exceed Final Momentum Rise
      DHPM = MIN(DHPM, 3.*DS*VS/US)

      RETURN
      END


c ref: The International Association for the Properties of Water and Steam, 2007	
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam	
c Dimensionless steam saturation equation coefficients	
c i	ni
c 1 :	1167.052145
c 2 :	-724213.167
c 3 :	-17.07384694
c 4 :	12020.8247
c 5 :	-3232555.032
c 6 :	14.91510861
c 7 :	-4823.265736
c 8 :	405113.4054
c 9 :	-0.238555576
c 10:	650.1753484


c----------------------------------------------------------------------
c  p_sat()
c----------------------------------------------------------------------
c     
c     Calculates saturation pressure for steam
c
c
c B.Zelt - www.zeltpsi.com
c ref: The International Association for the Properties of Water and Steam, 2007
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam
c Dimensionless saturation equations
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- UPDATES
c --- INPUT
c     T   : real :    (K) temperature
c
c --- OUTPUT
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function p_sat(T)
c --- inputs      
      real(8) :: T
      real(8) :: p_sat      
c --- declarations      
c      real(8) :: sum
      real(8) :: gibbs(10)
      real(8) :: A
      real(8) :: b
      real(8) :: C
      real(8) :: V

      data gibbs/1167.052145,-724213.167,-17.07384694,12020.8247,-
     &           3232555.032,14.91510861,-4823.265736,405113.4054,
     &           -0.238555576,650.1753484/
      
      V = T + gibbs(9) / (T - gibbs(10))
   
      A = V * V + gibbs(1) * V + gibbs(2)
      b = gibbs(3) * V * V + gibbs(4) * V + gibbs(5)
      C = gibbs(6) * V * V + gibbs(7) * V + gibbs(8)
c Psat in MPa
      p_sat = 2. * C / (-b + sqrt(b * b - 4. * A * C))
      p_sat = p_sat * p_sat
      p_sat = p_sat * p_sat
      p_sat = p_sat * 1000. ! return kPa
      return
      end 

c----------------------------------------------------------------------
c t_sat()
c----------------------------------------------------------------------
c     
c     Calculates the saturated temperature for steam
c
c B.Zelt - www.zeltpsi.com
c ref: The International Association for the Properties of Water and Steam, 2007
c Revised Release on the IAPWS Industrial Formulation 1997 for the Thermodynamic Properties of Water and Steam
c Dimensionless saturation equations
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- PURPOSE:  
c               
c
c --- UPDATES
c --- INPUT
c     p   : real :  pressure kPa
c
c
c --- OUTPUT
c     t   : saturation temerpature, K
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function t_sat(p)
c INPUTS
      real(8) :: p       ! pressure kPa
      real(8) :: t_sat      
c DECLARATIONS
c      real(8) :: sum
c      real(8) :: ni
      real(8) :: gibbs(10)
      real(8) :: D 
      real(8) :: E 
      real(8) :: F 
      real(8) :: G 
      real(8) :: b 
   
      data gibbs/1167.052145,-724213.167,-17.07384694,12020.8247,-
     &           3232555.032,14.91510861,-4823.265736,405113.4054,
     &           -0.238555576,650.1753484/

c p in MPA
      b = (p * 0.001)**0.25
   
      E = b * b + gibbs(3) * b + gibbs(6)
      F = gibbs(1) * b * b + gibbs(4) * b + gibbs(7)
      G = gibbs(2) * b * b + gibbs(5) * b + gibbs(8)
   
      D = 2. * G / (-F - sqrt(F*F - 4. * E * G))
c t_sat in K
      t_sat = gibbs(10) + D - sqrt((gibbs(10) + D)**2. - 
     &        4.*(gibbs(9) + gibbs(10) * D))
      t_sat = t_sat * 0.5
      return
      end 


c----------------------------------------------------------------------
c fnZ()
c----------------------------------------------------------------------
c  
c     Calculates the compressibility factor steam
c        
c B.Zelt - www.zeltpsi.com
c http://www.chem.mtu.edu/~tbco/cm3450/Compressibility_from_Redlich_Kwong.pdf
c Redlich-Kwong equation
c based on Cutlip and Shacham, 2008, pp. 101-103 (see Seader, Henley & Roper, 3rd Ed, 2011)
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c
c --- UPDATES
c --- INPUTS
c     T   : real : temperature (K)
c     P   : real : pressure (kPa)
c     Tc  : real : critical temperature (K)
c     Pc  : real : critical pressure (kPa)
c
c --- OUTPUTS
c
c --- RETURNS
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      Function fnZ(T, P, Tc, Pc)
c INPUTS      
      real(8) :: T
      real(8) :: P
      real(8) :: Tc
      real(8) :: Pc
      real(8) ::fnZ      
c DECLARATIONS      
      real(8) :: Tr
      real(8) :: Pr
      real(8) :: A
      real(8) :: b
      real(8) :: q
      real(8) :: r
      real(8) :: rone
   
      Tr = T / Tc
      Pr = p / Pc
   
      A = 0.42747 * Pr / (Tr**(5. / 2.))
      b = 0.08664 * Pr / Tr
      r = A * b
      q = b * b + b - A
      rone = 1.
   
      fnZ = rmcroot(rone, -rone, -q, -r)
      return
      end 


c----------------------------------------------------------------------
c rmcroot()
c----------------------------------------------------------------------
c     
c http://www.chem.mtu.edu/~tbco/cm3450/Compressibility_from_Redlich_Kwong.pdf
c Redlich-Kwong equation
c based on Cutlip and Shacham, 2008, pp. 101-103 (see Seader, Henley & Roper, 3rd Ed, 2011)
c
c Computes the maximum real root of the cubic equation
c a3 x^3 + a2 x^2 + a1 x + a0 = 0
c
c     
c --- Version: 1.001.0001    Level: 110706
c --- B.Zelt, Zelt PSI
c
c --- UPDATES
c --- INPUT
c     a3   : real :
c     a2   : real :
c     a1   : real :
c     a0   : real :
c
c --- OUTPUT
c
c --- RETURNS
c     rmcroot :   real : maximum real root of the cubic equation 
c
c --- called by:  host subroutines
c --- calls:      none
c----------------------------------------------------------------------
      function rmcroot(a3, a2 , a1, a0)
     & result(z)       
c INPUTS
      real(8) :: a3
      real(8) :: a2
      real(8) :: a1
      real(8) :: a0
      real(8) :: z
c      real(8) :: rmcroot      
c DECLARATIONS      
      real(8) :: A
      real(8) :: b
      real(8) :: C
      real(8) :: Disc
      real(8) :: rk13
      
      rk13 = 1./3.

      A = a2 / a3
      b = a1 / a3
      C = a0 / a3
      p = (-A*A*rk13 + b)*rk13
      q = (9.*A*b - 2.*A*A*A - 27.*C) / 54.
      Disc = q*q + p*p*p
      if( Disc .gt. 0.) Then
          h = q + Disc**0.5
          y = abs(h)**rk13
          if( h .lt. 0.) y = -y
          z = y - p / y - A*rk13
      Else
          theta = atan((-Disc)**0.5 / q)
          c1 = cos(theta*rk13)
          if( q .lt. 0. ) Then
              s1 = Sin(theta*rk13)
              c1 = (c1 - s1 * 3**0.5)*0.5
          End If
          z1 = 2. * (-p)**0.5 * c1 - A*rk13
          m = A + z1
          r = (m*m - 4. * (b + m * z1))**0.5
          z2 = (-m + r)*0.5
          z3 = (-m - r)*0.5
          z = z1
          if(z2 .gt. z) z = z2
          if(z3 .gt. z) z = z3
      end if
c      mcroot = z
      return
      end 
